@@ -21,7 +21,42 @@ import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
 
-import org.apache.poi.hssf.record.*;
+import org.apache.poi.hssf.record.BOFRecord;
+import org.apache.poi.hssf.record.CFHeaderRecord;
+import org.apache.poi.hssf.record.CalcCountRecord;
+import org.apache.poi.hssf.record.CalcModeRecord;
+import org.apache.poi.hssf.record.CellValueRecordInterface;
+import org.apache.poi.hssf.record.ColumnInfoRecord;
+import org.apache.poi.hssf.record.DVALRecord;
+import org.apache.poi.hssf.record.DefaultColWidthRecord;
+import org.apache.poi.hssf.record.DefaultRowHeightRecord;
+import org.apache.poi.hssf.record.DeltaRecord;
+import org.apache.poi.hssf.record.DimensionsRecord;
+import org.apache.poi.hssf.record.DrawingRecord;
+import org.apache.poi.hssf.record.EOFRecord;
+import org.apache.poi.hssf.record.EscherAggregate;
+import org.apache.poi.hssf.record.FeatHdrRecord;
+import org.apache.poi.hssf.record.FeatRecord;
+import org.apache.poi.hssf.record.GridsetRecord;
+import org.apache.poi.hssf.record.GutsRecord;
+import org.apache.poi.hssf.record.IndexRecord;
+import org.apache.poi.hssf.record.IterationRecord;
+import org.apache.poi.hssf.record.MergeCellsRecord;
+import org.apache.poi.hssf.record.NoteRecord;
+import org.apache.poi.hssf.record.PaneRecord;
+import org.apache.poi.hssf.record.PrintGridlinesRecord;
+import org.apache.poi.hssf.record.PrintHeadersRecord;
+import org.apache.poi.hssf.record.Record;
+import org.apache.poi.hssf.record.RecordBase;
+import org.apache.poi.hssf.record.RecordFormatException;
+import org.apache.poi.hssf.record.RefModeRecord;
+import org.apache.poi.hssf.record.RowRecord;
+import org.apache.poi.hssf.record.SCLRecord;
+import org.apache.poi.hssf.record.SaveRecalcRecord;
+import org.apache.poi.hssf.record.SelectionRecord;
+import org.apache.poi.hssf.record.UncalcedRecord;
+import org.apache.poi.hssf.record.WSBoolRecord;
+import org.apache.poi.hssf.record.WindowTwoRecord;
 import org.apache.poi.hssf.record.aggregates.ChartSubstreamRecordAggregate;
 import org.apache.poi.hssf.record.aggregates.ColumnInfoRecordsAggregate;
 import org.apache.poi.hssf.record.aggregates.ConditionalFormattingTable;
@@ -131,11 +166,27 @@ public final class InternalSheet {
         if (rs.peekNextSid() != BOFRecord.sid) {
             throw new RuntimeException("BOF record expected");
         }
+        
         BOFRecord bof = (BOFRecord) rs.getNext();
-        if (bof.getType() != BOFRecord.TYPE_WORKSHEET) {
-            // TODO - fix junit tests throw new RuntimeException("Bad BOF record type");
+        if (bof.getType() == BOFRecord.TYPE_WORKSHEET) {
+            // Good, well supported
+        } else if (bof.getType() == BOFRecord.TYPE_CHART ||
+                   bof.getType() == BOFRecord.TYPE_EXCEL_4_MACRO) {
+            // These aren't really typical sheets... Let it go though,
+            //  we can handle them roughly well enough as a "normal" one
+        } else {
+            // Not a supported type
+            // Skip onto the EOF, then complain
+            while (rs.hasNext()) {
+                Record rec = rs.getNext();
+                if (rec instanceof EOFRecord) {
+                    break;
+                }
+            }
+            throw new UnsupportedBOFType(bof.getType());
         }
         records.add(bof);
+        
         while (rs.hasNext()) {
             int recSid = rs.peekNextSid();
 
@@ -292,7 +343,9 @@ public final class InternalSheet {
                 // Not clear which application wrote these files.
                 rra = new RowRecordsAggregate();
             } else {
-                log.log(POILogger.WARN, "DIMENSION record not found even though row/cells present");
+                if (log.check(POILogger.WARN)) {
+                    log.log(POILogger.WARN, "DIMENSION record not found even though row/cells present");
+                }
                 // Not sure if any tools write files like this, but Excel reads them OK
             }
             dimsloc = findFirstRecordLocBySid(WindowTwoRecord.sid);
@@ -316,6 +369,18 @@ public final class InternalSheet {
                 recs.add(r);
             }});
     }
+    
+    public static class UnsupportedBOFType extends RecordFormatException {
+        private final int type;
+        protected UnsupportedBOFType(int type) {
+            super("BOF not of a supported type, found " + type);
+            this.type = type;
+        }
+        
+        public int getType() {
+            return type;
+        }
+    }
 
     private static final class RecordCloner implements RecordVisitor {
 
@@ -1562,9 +1627,9 @@ public final class InternalSheet {
 
     private void recalcRowGutter() {
         int maxLevel = 0;
-        Iterator iterator = _rowsAggregate.getIterator();
+        Iterator<RowRecord> iterator = _rowsAggregate.getIterator();
         while (iterator.hasNext()) {
-            RowRecord rowRecord = (RowRecord) iterator.next();
+            RowRecord rowRecord = iterator.next();
             maxLevel = Math.max(rowRecord.getOutlineLevel(), maxLevel);
         }
 
@@ -1602,4 +1667,8 @@ public final class InternalSheet {
         temp.toArray(result);
         return result;
     }
+    
+    public int getColumnOutlineLevel(int columnIndex) {
+        return _columnInfos.getOutlineLevel(columnIndex);
+    }
 }
