@@ -26,12 +26,66 @@ import java.util.Locale;
 import java.util.Map;
 import java.util.Map.Entry;
 
-import org.apache.poi.ddf.*;
-import org.apache.poi.hssf.record.*;
+import org.apache.poi.ddf.EscherBSERecord;
+import org.apache.poi.ddf.EscherBoolProperty;
+import org.apache.poi.ddf.EscherContainerRecord;
+import org.apache.poi.ddf.EscherDgRecord;
+import org.apache.poi.ddf.EscherDggRecord;
+import org.apache.poi.ddf.EscherOptRecord;
+import org.apache.poi.ddf.EscherProperties;
+import org.apache.poi.ddf.EscherRGBProperty;
+import org.apache.poi.ddf.EscherRecord;
+import org.apache.poi.ddf.EscherSimpleProperty;
+import org.apache.poi.ddf.EscherSpRecord;
+import org.apache.poi.ddf.EscherSplitMenuColorsRecord;
+import org.apache.poi.hssf.record.BOFRecord;
+import org.apache.poi.hssf.record.BackupRecord;
+import org.apache.poi.hssf.record.BookBoolRecord;
+import org.apache.poi.hssf.record.BoundSheetRecord;
+import org.apache.poi.hssf.record.CodepageRecord;
+import org.apache.poi.hssf.record.CountryRecord;
+import org.apache.poi.hssf.record.DSFRecord;
+import org.apache.poi.hssf.record.DateWindow1904Record;
+import org.apache.poi.hssf.record.DrawingGroupRecord;
+import org.apache.poi.hssf.record.EOFRecord;
+import org.apache.poi.hssf.record.EscherAggregate;
+import org.apache.poi.hssf.record.ExtSSTRecord;
+import org.apache.poi.hssf.record.ExtendedFormatRecord;
+import org.apache.poi.hssf.record.ExternSheetRecord;
+import org.apache.poi.hssf.record.FileSharingRecord;
+import org.apache.poi.hssf.record.FnGroupCountRecord;
+import org.apache.poi.hssf.record.FontRecord;
+import org.apache.poi.hssf.record.FormatRecord;
+import org.apache.poi.hssf.record.HideObjRecord;
+import org.apache.poi.hssf.record.HyperlinkRecord;
+import org.apache.poi.hssf.record.InterfaceEndRecord;
+import org.apache.poi.hssf.record.InterfaceHdrRecord;
+import org.apache.poi.hssf.record.MMSRecord;
+import org.apache.poi.hssf.record.NameCommentRecord;
+import org.apache.poi.hssf.record.NameRecord;
+import org.apache.poi.hssf.record.PaletteRecord;
+import org.apache.poi.hssf.record.PasswordRecord;
+import org.apache.poi.hssf.record.PasswordRev4Record;
+import org.apache.poi.hssf.record.PrecisionRecord;
+import org.apache.poi.hssf.record.ProtectRecord;
+import org.apache.poi.hssf.record.ProtectionRev4Record;
+import org.apache.poi.hssf.record.RecalcIdRecord;
+import org.apache.poi.hssf.record.Record;
+import org.apache.poi.hssf.record.RefreshAllRecord;
+import org.apache.poi.hssf.record.SSTRecord;
+import org.apache.poi.hssf.record.StyleRecord;
+import org.apache.poi.hssf.record.SupBookRecord;
+import org.apache.poi.hssf.record.TabIdRecord;
+import org.apache.poi.hssf.record.UseSelFSRecord;
+import org.apache.poi.hssf.record.WindowOneRecord;
+import org.apache.poi.hssf.record.WindowProtectRecord;
+import org.apache.poi.hssf.record.WriteAccessRecord;
+import org.apache.poi.hssf.record.WriteProtectRecord;
 import org.apache.poi.hssf.record.common.UnicodeString;
 import org.apache.poi.hssf.util.HSSFColor;
 import org.apache.poi.ss.formula.EvaluationWorkbook.ExternalName;
 import org.apache.poi.ss.formula.EvaluationWorkbook.ExternalSheet;
+import org.apache.poi.ss.formula.EvaluationWorkbook.ExternalSheetRange;
 import org.apache.poi.ss.formula.FormulaShifter;
 import org.apache.poi.ss.formula.ptg.Area3DPtg;
 import org.apache.poi.ss.formula.ptg.NameXPtg;
@@ -40,6 +94,7 @@ import org.apache.poi.ss.formula.ptg.Ptg;
 import org.apache.poi.ss.formula.ptg.Ref3DPtg;
 import org.apache.poi.ss.formula.udf.UDFFinder;
 import org.apache.poi.ss.usermodel.BuiltinFormats;
+import org.apache.poi.ss.usermodel.Workbook;
 import org.apache.poi.util.Internal;
 import org.apache.poi.util.POILogFactory;
 import org.apache.poi.util.POILogger;
@@ -514,7 +569,7 @@ public final class InternalWorkbook {
 
     /**
      * sets the name for a given sheet.  If the boundsheet record doesn't exist and
-     * its only one more than we have, go ahead and create it.  If it's > 1 more than
+     * its only one more than we have, go ahead and create it.  If it's &gt; 1 more than
      * we have, except
      *
      * @param sheetnum the sheet number (0 based)
@@ -716,9 +771,14 @@ public final class InternalWorkbook {
                 // Bump down by one, so still points
                 //  at the same sheet
                 nr.setSheetNumber(nr.getSheetNumber()-1);
-                
-                // also update the link-table as otherwise references might point at invalid sheets
-                linkTable.updateIndexToInternalSheet(i, -1);
+            }
+        }
+        
+        if (linkTable != null) {
+            // also tell the LinkTable about the removed sheet
+            // +1 because we already removed it from the count of sheets!
+            for(int i = sheetIndex+1;i < getNumSheets()+1;i++) {
+                linkTable.removeSheet(i);
             }
         }
     }
@@ -1745,34 +1805,50 @@ public final class InternalWorkbook {
         }
         return linkTable;
     }
+    
+    public int linkExternalWorkbook(String name, Workbook externalWorkbook) {
+        return getOrCreateLinkTable().linkExternalWorkbook(name, externalWorkbook);
+    }
 
-    /** finds the sheet name by his extern sheet index
+    /** 
+     * Finds the first sheet name by his extern sheet index
      * @param externSheetIndex extern sheet index
-     * @return sheet name.
+     * @return first sheet name.
      */
-    public String findSheetNameFromExternSheet(int externSheetIndex){
-
-        int indexToSheet = linkTable.getIndexToInternalSheet(externSheetIndex);
-        if (indexToSheet < 0) {
+    public String findSheetFirstNameFromExternSheet(int externSheetIndex){
+        int indexToSheet = linkTable.getFirstInternalSheetIndexForExtIndex(externSheetIndex);
+        return findSheetNameFromIndex(indexToSheet);
+    }
+    public String findSheetLastNameFromExternSheet(int externSheetIndex){
+        int indexToSheet = linkTable.getLastInternalSheetIndexForExtIndex(externSheetIndex);
+        return findSheetNameFromIndex(indexToSheet);
+    }
+    private String findSheetNameFromIndex(int internalSheetIndex) {
+        if (internalSheetIndex < 0) {
             // TODO - what does '-1' mean here?
             //error check, bail out gracefully!
             return "";
         }
-        if (indexToSheet >= boundsheets.size()) {
+        if (internalSheetIndex >= boundsheets.size()) {
             // Not sure if this can ever happen (See bug 45798)
             return ""; // Seems to be what excel would do in this case
         }
-        return getSheetName(indexToSheet);
+        return getSheetName(internalSheetIndex);
     }
+    
     public ExternalSheet getExternalSheet(int externSheetIndex) {
         String[] extNames = linkTable.getExternalBookAndSheetName(externSheetIndex);
         if (extNames == null) {
             return null;
         }
-        return new ExternalSheet(extNames[0], extNames[1]);
+        if (extNames.length == 2) {
+            return new ExternalSheet(extNames[0], extNames[1]);
+        } else {
+            return new ExternalSheetRange(extNames[0], extNames[1], extNames[2]);
+        }
     }
     public ExternalName getExternalName(int externSheetIndex, int externNameIndex) {
-       String nameName = linkTable.resolveNameXText(externSheetIndex, externNameIndex);
+       String nameName = linkTable.resolveNameXText(externSheetIndex, externNameIndex, this);
        if(nameName == null) {
           return null;
        }
@@ -1781,26 +1857,51 @@ public final class InternalWorkbook {
     }
 
     /**
-     * Finds the sheet index for a particular external sheet number.
+     * Finds the (first) sheet index for a particular external sheet number.
      * @param externSheetNumber     The external sheet number to convert
      * @return  The index to the sheet found.
      */
-    public int getSheetIndexFromExternSheetIndex(int externSheetNumber)
+    public int getFirstSheetIndexFromExternSheetIndex(int externSheetNumber)
     {
-        return linkTable.getSheetIndexFromExternSheetIndex(externSheetNumber);
+        return linkTable.getFirstInternalSheetIndexForExtIndex(externSheetNumber);
     }
 
-    /** returns the extern sheet number for specific sheet number ,
-     *  if this sheet doesn't exist in extern sheet , add it
-     * @param sheetNumber sheet number
+    /**
+     * Finds the last sheet index for a particular external sheet number,
+     *  which may be the same as the first (except for multi-sheet references)
+     * @param externSheetNumber     The external sheet number to convert
+     * @return  The index to the sheet found.
+     */
+    public int getLastSheetIndexFromExternSheetIndex(int externSheetNumber)
+    {
+        return linkTable.getLastInternalSheetIndexForExtIndex(externSheetNumber);
+    }
+
+    /** 
+     * Returns the extern sheet number for specific sheet number.
+     * If this sheet doesn't exist in extern sheet, add it
+     * @param sheetNumber local sheet number
      * @return index to extern sheet
      */
     public short checkExternSheet(int sheetNumber){
         return (short)getOrCreateLinkTable().checkExternSheet(sheetNumber);
     }
+    /** 
+     * Returns the extern sheet number for specific range of sheets.
+     * If this sheet range doesn't exist in extern sheet, add it
+     * @param firstSheetNumber first local sheet number
+     * @param lastSheetNumber last local sheet number
+     * @return index to extern sheet
+     */
+    public short checkExternSheet(int firstSheetNumber, int lastSheetNumber){
+        return (short)getOrCreateLinkTable().checkExternSheet(firstSheetNumber, lastSheetNumber);
+    }
 
     public int getExternalSheetIndex(String workbookName, String sheetName) {
-        return getOrCreateLinkTable().getExternalSheetIndex(workbookName, sheetName);
+        return getOrCreateLinkTable().getExternalSheetIndex(workbookName, sheetName, sheetName);
+    }
+    public int getExternalSheetIndex(String workbookName, String firstSheetName, String lastSheetName) {
+        return getOrCreateLinkTable().getExternalSheetIndex(workbookName, firstSheetName, lastSheetName);
     }
 
 
@@ -2295,18 +2396,19 @@ public final class InternalWorkbook {
      * @return the string representation of the defined or external name
      */
     public String resolveNameXText(int refIndex, int definedNameIndex) {
-        return linkTable.resolveNameXText(refIndex, definedNameIndex);
+        return linkTable.resolveNameXText(refIndex, definedNameIndex, this);
     }
 
     /**
      *
      * @param name the  name of an external function, typically a name of a UDF
+     * @param sheetRefIndex the sheet ref index, or -1 if not known
      * @param udf  locator of user-defiend functions to resolve names of VBA and Add-In functions
      * @return the external name or null
      */
-    public NameXPtg getNameXPtg(String name, UDFFinder udf) {
+    public NameXPtg getNameXPtg(String name, int sheetRefIndex, UDFFinder udf) {
         LinkTable lnk = getOrCreateLinkTable();
-        NameXPtg xptg = lnk.getNameXPtg(name);
+        NameXPtg xptg = lnk.getNameXPtg(name, sheetRefIndex);
 
         if(xptg == null && udf.findFunction(name) != null) {
             // the name was not found in the list of external names
@@ -2315,6 +2417,9 @@ public final class InternalWorkbook {
         }
         return xptg;
     }
+    public NameXPtg getNameXPtg(String name, UDFFinder udf) {
+        return getNameXPtg(name, -1, udf);
+    }
 
     /**
      * Check if the cloned sheet has drawings. If yes, then allocate a new drawing group ID and
