@@ -19,16 +19,16 @@
 package org.apache.poi.hssf.usermodel;
 
 import java.awt.*;
+import java.awt.font.FontRenderContext;
+import java.awt.font.GlyphVector;
 import java.awt.geom.AffineTransform;
 import java.awt.image.BufferedImage;
 import java.awt.image.BufferedImageOp;
 import java.awt.image.ImageObserver;
 import java.awt.image.RenderedImage;
 import java.awt.image.renderable.RenderableImage;
-import java.awt.font.GlyphVector;
-import java.awt.font.FontRenderContext;
-import java.util.Map;
 import java.text.AttributedCharacterIterator;
+import java.util.Map;
 
 public class DummyGraphics2d
         extends Graphics2D
@@ -42,7 +42,7 @@ public class DummyGraphics2d
         g2D = (Graphics2D)img.getGraphics();
     }
 
-    public void addRenderingHints(Map hints)
+    public void addRenderingHints(Map<?,?> hints)
     {
         System.out.println( "addRenderingHinds(Map):" );
         System.out.println( "  hints = " + hints );
@@ -275,7 +275,7 @@ public class DummyGraphics2d
         g2D.setRenderingHint( hintKey, hintValue );
     }
 
-    public void setRenderingHints(Map hints)
+    public void setRenderingHints(Map<?,?> hints)
     {
         System.out.println( "setRenderingHints(Map):" );
         System.out.println( "hints = " + hints );
@@ -675,6 +675,8 @@ public class DummyGraphics2d
     public void finalize() {
         System.out.println( "finalize():" );
         g2D.finalize();
+
+        super.finalize();
     }
 
     public Shape getClip()
