@@ -538,18 +538,17 @@ public class HSSFCell implements Cell {
 
     public void setCellValue(RichTextString value)
     {
-        HSSFRichTextString hvalue = (HSSFRichTextString) value;
         int row=_record.getRow();
         short col=_record.getColumn();
         short styleIndex=_record.getXFIndex();
-        if (hvalue == null)
+        if (value == null)
         {
             notifyFormulaChanging();
             setCellType(CELL_TYPE_BLANK, false, row, col, styleIndex);
             return;
         }
 
-        if(hvalue.length() > SpreadsheetVersion.EXCEL97.getMaxTextLength()){
+        if(value.length() > SpreadsheetVersion.EXCEL97.getMaxTextLength()){
             throw new IllegalArgumentException("The maximum length of cell contents (text) is 32,767 characters");
         }
 
@@ -557,7 +556,7 @@ public class HSSFCell implements Cell {
             // Set the 'pre-evaluated result' for the formula
             // note - formulas do not preserve text formatting.
             FormulaRecordAggregate fr = (FormulaRecordAggregate) _record;
-            fr.setCachedStringResult(hvalue.getString());
+            fr.setCachedStringResult(value.getString());
             // Update our local cache to the un-formatted version
             _stringValue = new HSSFRichTextString(value.getString());
 
@@ -573,6 +572,7 @@ public class HSSFCell implements Cell {
         }
         int index = 0;
 
+        HSSFRichTextString hvalue = (HSSFRichTextString) value;
         UnicodeString str = hvalue.getUnicodeString();
         index = _book.getWorkbook().addSSTString(str);
         (( LabelSSTRecord ) _record).setSSTIndex(index);
@@ -899,7 +899,13 @@ public class HSSFCell implements Cell {
         setCellStyle( (HSSFCellStyle)style );
     }
     public void setCellStyle(HSSFCellStyle style) {
-        // Verify it really does belong to our workbook
+        // A style of null means resetting back to the default style
+        if (style == null) {
+            _record.setXFIndex((short)0xf);
+            return;
+        }
+
+        // Verify the style really does belong to our workbook
         style.verifyBelongsToWorkbook(_book);
 
         short styleIndex;
@@ -1056,11 +1062,17 @@ public class HSSFCell implements Cell {
     }
 
     /**
-     * Assign a hyperlink to this cell
+     * Assign a hyperlink to this cell. If the supplied hyperlink is null, the
+     * hyperlink for this cell will be removed.
      *
      * @param hyperlink hyperlink associated with this cell
      */
     public void setHyperlink(Hyperlink hyperlink){
+        if (hyperlink == null) {
+            removeHyperlink();
+            return;
+        }
+
         HSSFHyperlink link = (HSSFHyperlink)hyperlink;
 
         link.setFirstRow(_record.getRow());
@@ -1085,6 +1097,23 @@ public class HSSFCell implements Cell {
         int eofLoc = records.size() - 1;
         records.add( eofLoc, link.record );
     }
+
+    /**
+     * Removes the hyperlink for this cell, if there is one.
+     */
+    public void removeHyperlink() {
+        for (Iterator<RecordBase> it = _sheet.getSheet().getRecords().iterator(); it.hasNext();) {
+            RecordBase rec = it.next();
+            if (rec instanceof HyperlinkRecord) {
+                HyperlinkRecord link = (HyperlinkRecord) rec;
+                if (link.getFirstColumn() == _record.getColumn() && link.getFirstRow() == _record.getRow()) {
+                    it.remove();
+                    return;
+                }
+            }
+        }
+    }
+
     /**
      * Only valid for formula cells
      * @return one of ({@link #CELL_TYPE_NUMERIC}, {@link #CELL_TYPE_STRING},
