@@ -23,10 +23,10 @@ import java.util.NoSuchElementException;
 import org.apache.poi.hssf.record.CellValueRecordInterface;
 import org.apache.poi.hssf.record.ExtendedFormatRecord;
 import org.apache.poi.hssf.record.RowRecord;
+import org.apache.poi.ss.SpreadsheetVersion;
 import org.apache.poi.ss.usermodel.Cell;
 import org.apache.poi.ss.usermodel.CellStyle;
 import org.apache.poi.ss.usermodel.Row;
-import org.apache.poi.ss.SpreadsheetVersion;
 import org.apache.poi.util.Configurator;
 
 /**
@@ -268,9 +268,8 @@ public final class HSSFRow implements Row {
      * Returns the rows outline level. Increased as you
      *  put it into more groups (outlines), reduced as
      *  you take it out of them.
-     * TODO - Should this really be public?
      */
-    protected int getOutlineLevel() {
+    public int getOutlineLevel() {
         return row.getOutlineLevel();
     }
 
@@ -618,7 +617,7 @@ public final class HSSFRow implements Row {
      * Alias for {@link #cellIterator} to allow
      *  foreach loops
      */
-    public Iterator iterator() {
+    public Iterator<Cell> iterator() {
        return cellIterator();
     }
 
@@ -684,6 +683,7 @@ public final class HSSFRow implements Row {
         return -1;
     }
 
+    @Override
     public boolean equals(Object obj)
     {
         if (!(obj instanceof HSSFRow))
@@ -698,4 +698,10 @@ public final class HSSFRow implements Row {
         }
         return false;
     }
+
+    @Override
+    public int hashCode() {
+        assert false : "hashCode not designed";
+        return 42; // any arbitrary constant will do
+    }
 }
