@@ -28,9 +28,23 @@ import org.apache.poi.hssf.model.DrawingManager2;
 import org.apache.poi.hssf.model.HSSFFormulaParser;
 import org.apache.poi.hssf.model.InternalSheet;
 import org.apache.poi.hssf.model.InternalWorkbook;
-import org.apache.poi.hssf.record.*;
+import org.apache.poi.hssf.record.AutoFilterInfoRecord;
+import org.apache.poi.hssf.record.CellValueRecordInterface;
+import org.apache.poi.hssf.record.DVRecord;
+import org.apache.poi.hssf.record.DimensionsRecord;
+import org.apache.poi.hssf.record.DrawingRecord;
+import org.apache.poi.hssf.record.EscherAggregate;
+import org.apache.poi.hssf.record.ExtendedFormatRecord;
+import org.apache.poi.hssf.record.NameRecord;
+import org.apache.poi.hssf.record.NoteRecord;
+import org.apache.poi.hssf.record.Record;
+import org.apache.poi.hssf.record.RowRecord;
+import org.apache.poi.hssf.record.SCLRecord;
+import org.apache.poi.hssf.record.WSBoolRecord;
+import org.apache.poi.hssf.record.WindowTwoRecord;
 import org.apache.poi.hssf.record.aggregates.DataValidityTable;
 import org.apache.poi.hssf.record.aggregates.FormulaRecordAggregate;
+import org.apache.poi.hssf.record.aggregates.RecordAggregate.RecordVisitor;
 import org.apache.poi.hssf.record.aggregates.WorksheetProtectionBlock;
 import org.apache.poi.hssf.util.PaneInformation;
 import org.apache.poi.ss.SpreadsheetVersion;
@@ -47,6 +61,7 @@ import org.apache.poi.ss.usermodel.DataValidation;
 import org.apache.poi.ss.usermodel.DataValidationHelper;
 import org.apache.poi.ss.usermodel.Row;
 import org.apache.poi.ss.util.CellRangeAddress;
+import org.apache.poi.ss.util.CellRangeAddressList;
 import org.apache.poi.ss.util.CellReference;
 import org.apache.poi.ss.util.SSCellRange;
 import org.apache.poi.ss.util.SheetUtil;
@@ -61,6 +76,16 @@ public final class HSSFSheet implements org.apache.poi.ss.usermodel.Sheet {
     private static final POILogger log = POILogFactory.getLogger(HSSFSheet.class);
     private static final int DEBUG = POILogger.DEBUG;
 
+    /**
+     * width of 1px in columns with default width in units of 1/256 of a character width
+     */
+    private static final float PX_DEFAULT = 32.00f;
+    /**
+     * width of 1px in columns with overridden width in units of 1/256 of a character width
+     */
+    private static final float PX_MODIFIED = 36.56f;
+
+    
     /**
      * Used for compile-time optimization.  This is the initial size for the collection of
      * rows.  It is currently set to 20.  If you generate larger sheets you may benefit
@@ -381,6 +406,35 @@ public final class HSSFSheet implements org.apache.poi.ss.usermodel.Sheet {
         return _lastrow;
     }
 
+    public List<HSSFDataValidation> getDataValidations() {
+        DataValidityTable dvt = _sheet.getOrCreateDataValidityTable();
+        final List<HSSFDataValidation> hssfValidations = new ArrayList<HSSFDataValidation>();
+        RecordVisitor visitor = new RecordVisitor() {
+            private HSSFEvaluationWorkbook book = HSSFEvaluationWorkbook.create(getWorkbook());
+
+            @Override
+            public void visitRecord(Record r) {
+                if (!(r instanceof DVRecord)) {
+                    return;
+                }
+                DVRecord dvRecord = (DVRecord) r;
+                CellRangeAddressList regions = dvRecord.getCellRangeAddress().copy();
+                DVConstraint constraint = DVConstraint.createDVConstraint(dvRecord, book);
+                HSSFDataValidation hssfDataValidation = new HSSFDataValidation(regions, constraint);
+                hssfDataValidation.setErrorStyle(dvRecord.getErrorStyle());
+                hssfDataValidation.setEmptyCellAllowed(dvRecord.getEmptyCellAllowed());
+                hssfDataValidation.setSuppressDropDownArrow(dvRecord.getSuppressDropdownArrow());
+                hssfDataValidation.createPromptBox(dvRecord.getPromptTitle(), dvRecord.getPromptText());
+                hssfDataValidation.setShowPromptBox(dvRecord.getShowPromptOnCellSelected());
+                hssfDataValidation.createErrorBox(dvRecord.getErrorTitle(), dvRecord.getErrorText());
+                hssfDataValidation.setShowErrorBox(dvRecord.getShowErrorOnInvalidValue());
+                hssfValidations.add(hssfDataValidation);
+            }
+        };
+        dvt.visitContainedRecords(visitor);
+        return hssfValidations;
+    }
+
     /**
      * Creates a data validation object
      *
@@ -511,6 +565,14 @@ public final class HSSFSheet implements org.apache.poi.ss.usermodel.Sheet {
         return _sheet.getColumnWidth(columnIndex);
     }
 
+    public float getColumnWidthInPixels(int column){
+        int cw = getColumnWidth(column);
+        int def = getDefaultColumnWidth()*256;
+        float px = (cw == def ? PX_DEFAULT : PX_MODIFIED);
+
+        return cw/px;
+    }
+    
     /**
      * get the default column width for the sheet (if the columns do not define their own width) in
      * characters
@@ -1491,8 +1553,10 @@ public final class HSSFSheet implements org.apache.poi.ss.usermodel.Sheet {
         // Update any formulas on this sheet that point to
         //  rows which have been moved
         int sheetIndex = _workbook.getSheetIndex(this);
+        String sheetName = _workbook.getSheetName(sheetIndex);
         short externSheetIndex = _book.checkExternSheet(sheetIndex);
-        FormulaShifter shifter = FormulaShifter.createForRowShift(externSheetIndex, startRow, endRow, n);
+        FormulaShifter shifter = FormulaShifter.createForRowShift(
+                         externSheetIndex, sheetName, startRow, endRow, n);
         _sheet.updateFormulasAfterCellShift(shifter, externSheetIndex);
 
         int nSheets = _workbook.getNumberOfSheets();
@@ -2115,7 +2179,7 @@ public final class HSSFSheet implements org.apache.poi.ss.usermodel.Sheet {
             }
             if (shape instanceof HSSFComment) {
                 HSSFComment comment = (HSSFComment) shape;
-                if (comment.getColumn() == column && comment.getRow() == row) {
+                if (comment.hasPosition() && comment.getColumn() == column && comment.getRow() == row) {
                     return comment;
                 }
             }
@@ -2276,6 +2340,13 @@ public final class HSSFSheet implements org.apache.poi.ss.usermodel.Sheet {
         }
         return _workbook.getNameRecord(recIndex);
     }
-
-
+    
+    /**
+     * Returns the column outline level. Increased as you
+     *  put it into more groups (outlines), reduced as
+     *  you take it out of them.
+     */
+    public int getColumnOutlineLevel(int columnIndex) {
+        return _sheet.getColumnOutlineLevel(columnIndex);
+    }
 }
