@@ -46,16 +46,31 @@ import org.apache.poi.hssf.OldExcelFormatException;
 import org.apache.poi.hssf.model.DrawingManager2;
 import org.apache.poi.hssf.model.HSSFFormulaParser;
 import org.apache.poi.hssf.model.InternalSheet;
+import org.apache.poi.hssf.model.InternalSheet.UnsupportedBOFType;
 import org.apache.poi.hssf.model.InternalWorkbook;
 import org.apache.poi.hssf.model.RecordStream;
-import org.apache.poi.hssf.record.*;
+import org.apache.poi.hssf.record.AbstractEscherHolderRecord;
+import org.apache.poi.hssf.record.BackupRecord;
+import org.apache.poi.hssf.record.DrawingGroupRecord;
+import org.apache.poi.hssf.record.ExtendedFormatRecord;
+import org.apache.poi.hssf.record.FontRecord;
+import org.apache.poi.hssf.record.LabelRecord;
+import org.apache.poi.hssf.record.LabelSSTRecord;
+import org.apache.poi.hssf.record.NameRecord;
+import org.apache.poi.hssf.record.RecalcIdRecord;
+import org.apache.poi.hssf.record.Record;
+import org.apache.poi.hssf.record.RecordFactory;
+import org.apache.poi.hssf.record.SSTRecord;
+import org.apache.poi.hssf.record.UnknownRecord;
 import org.apache.poi.hssf.record.aggregates.RecordAggregate.RecordVisitor;
 import org.apache.poi.hssf.record.common.UnicodeString;
 import org.apache.poi.hssf.util.CellReference;
+import org.apache.poi.poifs.crypt.Decryptor;
 import org.apache.poi.poifs.filesystem.DirectoryEntry;
 import org.apache.poi.poifs.filesystem.DirectoryNode;
 import org.apache.poi.poifs.filesystem.EntryUtils;
 import org.apache.poi.poifs.filesystem.FilteringDirectoryNode;
+import org.apache.poi.poifs.filesystem.NPOIFSFileSystem;
 import org.apache.poi.poifs.filesystem.Ole10Native;
 import org.apache.poi.poifs.filesystem.POIFSFileSystem;
 import org.apache.poi.ss.formula.FormulaShifter;
@@ -65,6 +80,7 @@ import org.apache.poi.ss.formula.udf.AggregatingUDFFinder;
 import org.apache.poi.ss.formula.udf.IndexedUDFFinder;
 import org.apache.poi.ss.formula.udf.UDFFinder;
 import org.apache.poi.ss.usermodel.Row.MissingCellPolicy;
+import org.apache.poi.ss.usermodel.Workbook;
 import org.apache.poi.ss.util.CellRangeAddress;
 import org.apache.poi.ss.util.WorkbookUtil;
 import org.apache.poi.util.Configurator;
@@ -170,23 +186,33 @@ public final class HSSFWorkbook extends POIDocument implements org.apache.poi.ss
         this(InternalWorkbook.createWorkbook());
     }
 
-	private HSSFWorkbook(InternalWorkbook book) {
-		super((DirectoryNode)null);
-		workbook = book;
-		_sheets = new ArrayList<HSSFSheet>(INITIAL_CAPACITY);
-		names = new ArrayList<HSSFName>(INITIAL_CAPACITY);
-	}
+    private HSSFWorkbook(InternalWorkbook book) {
+        super((DirectoryNode)null);
+        workbook = book;
+        _sheets = new ArrayList<HSSFSheet>(INITIAL_CAPACITY);
+        names = new ArrayList<HSSFName>(INITIAL_CAPACITY);
+    }
 
+    /**
+     * Given a POI POIFSFileSystem object, read in its Workbook along
+     *  with all related nodes, and populate the high and low level models.
+     * <p>This calls {@link #HSSFWorkbook(POIFSFileSystem, boolean)} with
+     *  preserve nodes set to true. 
+     * 
+     * @see #HSSFWorkbook(POIFSFileSystem, boolean)
+     * @see org.apache.poi.poifs.filesystem.POIFSFileSystem
+     * @exception IOException if the stream cannot be read
+     */
     public HSSFWorkbook(POIFSFileSystem fs) throws IOException {
         this(fs,true);
     }
 
     /**
-     * given a POI POIFSFileSystem object, read in its Workbook and populate the high and
-     * low level models.  If you're reading in a workbook...start here.
+     * Given a POI POIFSFileSystem object, read in its Workbook and populate 
+     * the high and low level models.  If you're reading in a workbook... start here!
      *
      * @param fs the POI filesystem that contains the Workbook stream.
-     * @param preserveNodes whether to preseve other nodes, such as
+     * @param preserveNodes whether to preserve other nodes, such as
      *        macros.  This takes more memory, so only say yes if you
      *        need to. If set, will store all of the POIFSFileSystem
      *        in memory
@@ -194,8 +220,7 @@ public final class HSSFWorkbook extends POIDocument implements org.apache.poi.ss
      * @exception IOException if the stream cannot be read
      */
     public HSSFWorkbook(POIFSFileSystem fs, boolean preserveNodes)
-            throws IOException
-    {
+            throws IOException {
         this(fs.getRoot(), fs, preserveNodes);
     }
 
@@ -224,7 +249,7 @@ public final class HSSFWorkbook extends POIDocument implements org.apache.poi.ss
         
         // check for an encrypted .xlsx file - they get OLE2 wrapped
         try {
-        	directory.getEntry("EncryptedPackage");
+        	directory.getEntry(Decryptor.DEFAULT_POIFS_ENTRY);
         	throw new EncryptedDocumentException("The supplied spreadsheet seems to be an Encrypted .xlsx file. " +
         			"It must be decrypted before use by XSSF, it cannot be used by HSSF");
         } catch (FileNotFoundException e) {
@@ -251,7 +276,7 @@ public final class HSSFWorkbook extends POIDocument implements org.apache.poi.ss
      *
      * @param directory the POI filesystem directory to process from
      * @param fs the POI filesystem that contains the Workbook stream.
-     * @param preserveNodes whether to preseve other nodes, such as
+     * @param preserveNodes whether to preserve other nodes, such as
      *        macros.  This takes more memory, so only say yes if you
      *        need to. If set, will store all of the POIFSFileSystem
      *        in memory
@@ -270,7 +295,7 @@ public final class HSSFWorkbook extends POIDocument implements org.apache.poi.ss
      *  low level models.  If you're reading in a workbook...start here.
      *
      * @param directory the POI filesystem directory to process from
-     * @param preserveNodes whether to preseve other nodes, such as
+     * @param preserveNodes whether to preserve other nodes, such as
      *        macros.  This takes more memory, so only say yes if you
      *        need to. If set, will store all of the POIFSFileSystem
      *        in memory
@@ -308,8 +333,13 @@ public final class HSSFWorkbook extends POIDocument implements org.apache.poi.ss
         convertLabelRecords(records, recOffset);
         RecordStream rs = new RecordStream(records, recOffset);
         while (rs.hasNext()) {
-            InternalSheet sheet = InternalSheet.createSheet(rs);
-            _sheets.add(new HSSFSheet(this, sheet));
+            try {
+                InternalSheet sheet = InternalSheet.createSheet(rs);
+                _sheets.add(new HSSFSheet(this, sheet));
+            } catch (UnsupportedBOFType eb) {
+                // Hopefully there's a supported one after this!
+                log.log(POILogger.WARN, "Unsupported BOF found of type " + eb.getType());
+            }
         }
 
         for (int i = 0 ; i < workbook.getNumNames() ; ++i){
@@ -319,23 +349,33 @@ public final class HSSFWorkbook extends POIDocument implements org.apache.poi.ss
         }
     }
 
-     public HSSFWorkbook(InputStream s) throws IOException {
-         this(s,true);
-     }
+    /**
+     * Companion to HSSFWorkbook(POIFSFileSystem), this constructs the 
+     *  POI filesystem around your {@link InputStream}, including all nodes.
+     * <p>This calls {@link #HSSFWorkbook(InputStream, boolean)} with
+     *  preserve nodes set to true. 
+     *
+     * @see #HSSFWorkbook(InputStream, boolean)
+     * @see #HSSFWorkbook(POIFSFileSystem)
+     * @see org.apache.poi.poifs.filesystem.POIFSFileSystem
+     * @exception IOException if the stream cannot be read
+     */
+    public HSSFWorkbook(InputStream s) throws IOException {
+        this(s,true);
+    }
 
     /**
-     * Companion to HSSFWorkbook(POIFSFileSystem), this constructs the POI filesystem around your
-     * inputstream.
+     * Companion to HSSFWorkbook(POIFSFileSystem), this constructs the 
+     * POI filesystem around your {@link InputStream}.
      *
      * @param s  the POI filesystem that contains the Workbook stream.
-     * @param preserveNodes whether to preseve other nodes, such as
+     * @param preserveNodes whether to preserve other nodes, such as
      *        macros.  This takes more memory, so only say yes if you
      *        need to.
      * @see org.apache.poi.poifs.filesystem.POIFSFileSystem
      * @see #HSSFWorkbook(POIFSFileSystem)
      * @exception IOException if the stream cannot be read
      */
-
     public HSSFWorkbook(InputStream s, boolean preserveNodes)
             throws IOException
     {
@@ -358,9 +398,9 @@ public final class HSSFWorkbook extends POIDocument implements org.apache.poi.ss
       * you have to read in a sheet that contains Label records, be aware that the rest
       * of the API doesn't deal with them, the low level structure only provides read-only
       * semi-immutable structures (the sets are there for interface conformance with NO
-      * impelmentation).  In short, you need to call this function passing it a reference
+      * Implementation).  In short, you need to call this function passing it a reference
       * to the Workbook object.  All labels will be converted to LabelSST records and their
-      * contained strings will be written to the Shared String tabel (SSTRecord) within
+      * contained strings will be written to the Shared String table (SSTRecord) within
       * the Workbook.
       *
       * @param records a collection of sheet's records.
@@ -441,6 +481,21 @@ public final class HSSFWorkbook extends POIDocument implements org.apache.poi.ss
 
         workbook.updateNamesAfterCellShift(shifter);
 
+        // adjust active sheet if necessary
+        int active = getActiveSheetIndex();
+        if(active == oldSheetIndex) {
+            // moved sheet was the active one
+            setActiveSheet(pos);
+        } else if ((active < oldSheetIndex && active < pos) ||
+                (active > oldSheetIndex && active > pos)) {
+            // not affected
+        } else if (pos > oldSheetIndex) {
+            // moved sheet was below before and is above now => active is one less
+            setActiveSheet(active-1);
+        } else {
+            // remaining case: moved sheet was higher than active before and is lower now => active is one more
+            setActiveSheet(active+1);
+        }
     }
 
     private void validateSheetIndex(int index) {
@@ -662,7 +717,7 @@ public final class HSSFWorkbook extends POIDocument implements org.apache.poi.ss
     @Deprecated
 	public String findSheetNameFromExternSheet(int externSheetIndex){
         // TODO - don't expose internal ugliness like externSheet indexes to the user model API
-        return workbook.findSheetNameFromExternSheet(externSheetIndex);
+        return workbook.findSheetFirstNameFromExternSheet(externSheetIndex);
     }
     /**
      * @deprecated for POI internal use only (formula rendering).  This method is likely to
@@ -832,8 +887,14 @@ public final class HSSFWorkbook extends POIDocument implements org.apache.poi.ss
         return _sheets.size();
     }
 
+    /**
+     * @deprecated for POI internal use only (formula parsing).  This method is likely to
+     * be removed in future versions of POI.
+     */
+    @Deprecated
     public int getSheetIndexFromExternSheetIndex(int externSheetNumber) {
-        return workbook.getSheetIndexFromExternSheetIndex(externSheetNumber);
+        // TODO - don't expose internal ugliness like externSheet indexes to the user model API
+        return workbook.getFirstSheetIndexFromExternSheetIndex(externSheetNumber);
     }
 
     private HSSFSheet[] getSheets() {
@@ -892,7 +953,6 @@ public final class HSSFWorkbook extends POIDocument implements org.apache.poi.ss
      */
     public void removeSheetAt(int index) {
         validateSheetIndex(index);
-        boolean wasActive = getSheetAt(index).isActive();
         boolean wasSelected = getSheetAt(index).isSelected();
 
         _sheets.remove(index);
@@ -909,9 +969,6 @@ public final class HSSFWorkbook extends POIDocument implements org.apache.poi.ss
         if (newSheetIndex >= nSheets) {
             newSheetIndex = nSheets-1;
         }
-        if (wasActive) {
-            setActiveSheet(newSheetIndex);
-        }
 
         if (wasSelected) {
             boolean someOtherSheetIsStillSelected = false;
@@ -925,6 +982,16 @@ public final class HSSFWorkbook extends POIDocument implements org.apache.poi.ss
                 setSelectedTab(newSheetIndex);
             }
         }
+
+        // adjust active sheet
+        int active = getActiveSheetIndex();
+        if(active == index) {
+            // removed sheet was the active one, reset active sheet if there is still one left now
+            setActiveSheet(newSheetIndex);
+        } else if (active > index) {
+            // removed sheet was below the active one => active is one less now
+            setActiveSheet(active-1);
+        }
     }
 
     /**
@@ -1178,9 +1245,25 @@ public final class HSSFWorkbook extends POIDocument implements org.apache.poi.ss
 
         return style;
     }
+    
+    /**
+     * Closes the underlying {@link NPOIFSFileSystem} from which
+     *  the Workbook was read, if any. Has no effect on Workbooks
+     *  opened from an InputStream, or newly created ones.
+     */
+    @Override
+    public void close() throws IOException
+    {
+        if (directory != null) {
+            if (directory.getNFileSystem() != null) {
+                directory.getNFileSystem().close();
+                directory = null;
+            }
+        }
+    }
 
     /**
-     * Method write - write out this workbook to an Outputstream.  Constructs
+     * Method write - write out this workbook to an {@link OutputStream}.  Constructs
      * a new POI POIFSFileSystem, passes in the workbook binary representation  and
      * writes it out.
      *
@@ -1189,7 +1272,6 @@ public final class HSSFWorkbook extends POIDocument implements org.apache.poi.ss
      * @exception IOException if anything can't be written.
      * @see org.apache.poi.poifs.filesystem.POIFSFileSystem
      */
-
     @Override
 	public void write(OutputStream stream)
             throws IOException
@@ -1791,6 +1873,19 @@ public final class HSSFWorkbook extends POIDocument implements org.apache.poi.ss
     	return storageId;
     }
     
+    /**
+     * Adds the LinkTable records required to allow formulas referencing
+     *  the specified external workbook to be added to this one. Allows
+     *  formulas such as "[MyOtherWorkbook]Sheet3!$A$5" to be added to the 
+     *  file, for workbooks not already referenced.
+     *
+     * @param name The name the workbook will be referenced as in formulas
+     * @param workbook The open workbook to fetch the link required information from
+     */
+    public int linkExternalWorkbook(String name, Workbook workbook) {
+        return this.workbook.linkExternalWorkbook(name, workbook);
+    }
+    
     /**
      * Is the workbook protected with a password (not encrypted)?
      */
@@ -1920,9 +2015,9 @@ public final class HSSFWorkbook extends POIDocument implements org.apache.poi.ss
 
 	/**
 	 * Changes an external referenced file to another file.
-	 * A formular in Excel which refers a cell in another file is saved in two parts:
+	 * A formula in Excel which references a cell in another file is saved in two parts:
 	 * The referenced file is stored in an reference table. the row/cell information is saved separate.
-	 * This method invokation will only change the reference in the lookup-table itself.
+	 * This method invocation will only change the reference in the lookup-table itself.
 	 * @param oldUrl The old URL to search for and which is to be replaced
 	 * @param newUrl The URL replacement
 	 * @return true if the oldUrl was found and replaced with newUrl. Otherwise false
