@@ -75,7 +75,7 @@ public class HSSFColor implements Color {
 
             Integer index1 = Integer.valueOf(color.getIndex());
             if (result.containsKey(index1)) {
-                HSSFColor prevColor = (HSSFColor)result.get(index1);
+                HSSFColor prevColor = result.get(index1);
                 throw new RuntimeException("Dup color index (" + index1
                         + ") for colors (" + prevColor.getClass().getName()
                         + "),(" + color.getClass().getName() + ")");
@@ -164,7 +164,7 @@ public class HSSFColor implements Color {
 
             String hexString = color.getHexString();
             if (result.containsKey(hexString)) {
-            	HSSFColor other = (HSSFColor)result.get(hexString);
+            	HSSFColor other = result.get(hexString);
                 throw new RuntimeException(
                 		"Dup color hexString (" + hexString
                         + ") for color (" + color.getClass().getName() + ") - "
