@@ -27,7 +27,6 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.PushbackInputStream;
-import java.io.RandomAccessFile;
 import java.nio.ByteBuffer;
 import java.nio.channels.Channels;
 import java.nio.channels.FileChannel;
@@ -37,6 +36,7 @@ import java.util.Collections;
 import java.util.Iterator;
 import java.util.List;
 
+import org.apache.poi.EmptyFileException;
 import org.apache.poi.poifs.common.POIFSBigBlockSize;
 import org.apache.poi.poifs.common.POIFSConstants;
 import org.apache.poi.poifs.dev.POIFSViewable;
@@ -44,6 +44,7 @@ import org.apache.poi.poifs.nio.ByteArrayBackedDataSource;
 import org.apache.poi.poifs.nio.DataSource;
 import org.apache.poi.poifs.nio.FileBackedDataSource;
 import org.apache.poi.poifs.property.DirectoryProperty;
+import org.apache.poi.poifs.property.DocumentProperty;
 import org.apache.poi.poifs.property.NPropertyTable;
 import org.apache.poi.poifs.storage.BATBlock;
 import org.apache.poi.poifs.storage.BATBlock.BATBlockAndIndex;
@@ -57,9 +58,9 @@ import org.apache.poi.util.IOUtils;
 import org.apache.poi.util.LongField;
 
 /**
- * This is the main class of the POIFS system; it manages the entire
- * life cycle of the filesystem.
- * This is the new NIO version
+ * <p>This is the main class of the POIFS system; it manages the entire
+ * life cycle of the filesystem.</p>
+ * <p>This is the new NIO version, which uses less memory</p>
  */
 
 public class NPOIFSFileSystem extends BlockStore
@@ -117,21 +118,23 @@ public class NPOIFSFileSystem extends BlockStore
         // Mark us as having a single empty BAT at offset 0
         _header.setBATCount(1);
         _header.setBATArray(new int[] { 0 });
-        _bat_blocks.add(BATBlock.createEmptyBATBlock(bigBlockSize, false));
+        BATBlock bb = BATBlock.createEmptyBATBlock(bigBlockSize, false);
+        bb.setOurBlockIndex(0);
+        _bat_blocks.add(bb);
+
         setNextBlock(0, POIFSConstants.FAT_SECTOR_BLOCK);
-        
-        // Now associate the properties with the empty block
-        _property_table.setStartBlock(1);
         setNextBlock(1, POIFSConstants.END_OF_CHAIN);
+
+        _property_table.setStartBlock(POIFSConstants.END_OF_CHAIN);
     }
 
     /**
-     * Creates a POIFSFileSystem from a <tt>File</tt>. This uses less memory than
-     *  creating from an <tt>InputStream</tt>. The File will be opened read-only
+     * <p>Creates a POIFSFileSystem from a <tt>File</tt>. This uses less memory than
+     *  creating from an <tt>InputStream</tt>. The File will be opened read-only</p>
      *  
-     * Note that with this constructor, you will need to call {@link #close()}
+     * <p>Note that with this constructor, you will need to call {@link #close()}
      *  when you're done to have the underlying file closed, as the file is
-     *  kept open during normal operation to read the data out. 
+     *  kept open during normal operation to read the data out.</p> 
      *  
      * @param file the File from which to read the data
      *
@@ -144,34 +147,32 @@ public class NPOIFSFileSystem extends BlockStore
     }
     
     /**
-     * Creates a POIFSFileSystem from a <tt>File</tt>. This uses less memory than
-     *  creating from an <tt>InputStream</tt>.
+     * <p>Creates a POIFSFileSystem from a <tt>File</tt>. This uses less memory than
+     *  creating from an <tt>InputStream</tt>.</p>
      *  
-     * Note that with this constructor, you will need to call {@link #close()}
+     * <p>Note that with this constructor, you will need to call {@link #close()}
      *  when you're done to have the underlying file closed, as the file is
-     *  kept open during normal operation to read the data out. 
+     *  kept open during normal operation to read the data out.</p> 
      *  
-     * @param file the File from which to read the data
+     * @param file the File from which to read or read/write the data
+     * @param readOnly whether the POIFileSystem will only be used in read-only mode
      *
      * @exception IOException on errors reading, or on invalid data
      */
-    @SuppressWarnings("resource")
     public NPOIFSFileSystem(File file, boolean readOnly)
          throws IOException
     {
-       this(
-           (new RandomAccessFile(file, readOnly? "r" : "rw")).getChannel(),
-           true
-       );
+       this(null, file, readOnly, true);
     }
     
     /**
-     * Creates a POIFSFileSystem from an open <tt>FileChannel</tt>. This uses 
-     *  less memory than creating from an <tt>InputStream</tt>.
+     * <p>Creates a POIFSFileSystem from an open <tt>FileChannel</tt>. This uses 
+     *  less memory than creating from an <tt>InputStream</tt>. The stream will
+     *  be used in read-only mode.</p>
      *  
-     * Note that with this constructor, you will need to call {@link #close()}
+     * <p>Note that with this constructor, you will need to call {@link #close()}
      *  when you're done to have the underlying Channel closed, as the channel is
-     *  kept open during normal operation to read the data out. 
+     *  kept open during normal operation to read the data out.</p> 
      *  
      * @param channel the FileChannel from which to read the data
      *
@@ -180,15 +181,46 @@ public class NPOIFSFileSystem extends BlockStore
     public NPOIFSFileSystem(FileChannel channel)
          throws IOException
     {
-       this(channel, false);
+       this(channel, true);
     }
     
-    private NPOIFSFileSystem(FileChannel channel, boolean closeChannelOnError)
+    /**
+     * <p>Creates a POIFSFileSystem from an open <tt>FileChannel</tt>. This uses 
+     *  less memory than creating from an <tt>InputStream</tt>.</p>
+     *  
+     * <p>Note that with this constructor, you will need to call {@link #close()}
+     *  when you're done to have the underlying Channel closed, as the channel is
+     *  kept open during normal operation to read the data out.</p> 
+     *  
+     * @param channel the FileChannel from which to read or read/write the data
+     * @param readOnly whether the POIFileSystem will only be used in read-only mode
+     *
+     * @exception IOException on errors reading, or on invalid data
+     */
+    public NPOIFSFileSystem(FileChannel channel, boolean readOnly)
+         throws IOException
+    {
+       this(channel, null, readOnly, false);
+    }
+    
+    private NPOIFSFileSystem(FileChannel channel, File srcFile, boolean readOnly, boolean closeChannelOnError)
          throws IOException
     {
        this(false);
 
        try {
+          // Initialize the datasource
+          if (srcFile != null) {
+              if (srcFile.length() == 0)
+                  throw new EmptyFileException();
+              
+              FileBackedDataSource d = new FileBackedDataSource(srcFile, readOnly);
+              channel = d.getChannel();
+              _data = d;
+          } else {
+              _data = new FileBackedDataSource(channel, readOnly);
+          }
+           
           // Get the header
           ByteBuffer headerBuffer = ByteBuffer.allocate(POIFSConstants.SMALLER_BIG_BLOCK_SIZE);
           IOUtils.readFully(channel, headerBuffer);
@@ -197,7 +229,6 @@ public class NPOIFSFileSystem extends BlockStore
           _header = new HeaderBlock(headerBuffer);
           
           // Now process the various entries
-          _data = new FileBackedDataSource(channel);
           readCoreContents();
        } catch(IOException e) {
           if(closeChannelOnError) {
@@ -209,7 +240,10 @@ public class NPOIFSFileSystem extends BlockStore
           // TODO Decide if we can handle these better whilst
           //  still sticking to the iterator contract
           if(closeChannelOnError) {
-             channel.close();
+              if (channel != null) {
+                  channel.close();
+                  channel = null;
+              }
           }
           throw e;
        }
@@ -269,12 +303,17 @@ public class NPOIFSFileSystem extends BlockStore
            // We need to buffer the whole file into memory when
            //  working with an InputStream.
            // The max possible size is when each BAT block entry is used
-           int maxSize = BATBlock.calculateMaximumSize(_header); 
-           ByteBuffer data = ByteBuffer.allocate(maxSize);
+           long maxSize = BATBlock.calculateMaximumSize(_header); 
+           if (maxSize > Integer.MAX_VALUE) {
+               throw new IllegalArgumentException("Unable read a >2gb file via an InputStream");
+           }
+           ByteBuffer data = ByteBuffer.allocate((int)maxSize);
+           
            // Copy in the header
            headerBuffer.position(0);
            data.put(headerBuffer);
            data.position(headerBuffer.capacity());
+           
            // Now read the rest of the stream
            IOUtils.readFully(channel, data);
            success = true;
@@ -422,8 +461,13 @@ public class NPOIFSFileSystem extends BlockStore
     @Override
     protected ByteBuffer getBlockAt(final int offset) throws IOException {
        // The header block doesn't count, so add one
-       long startAt = (offset+1) * bigBlockSize.getBigBlockSize();
-       return _data.read(bigBlockSize.getBigBlockSize(), startAt);
+       long blockWanted = offset + 1;
+       long startAt = blockWanted * bigBlockSize.getBigBlockSize();
+       try {
+           return _data.read(bigBlockSize.getBigBlockSize(), startAt);
+       } catch (IndexOutOfBoundsException e) {
+           throw new IndexOutOfBoundsException("Block " + offset + " not found - " + e);
+       }
     }
     
     /**
@@ -483,13 +527,11 @@ public class NPOIFSFileSystem extends BlockStore
      */
     @Override
     protected int getFreeBlock() throws IOException {
+        int numSectors = bigBlockSize.getBATEntriesPerBlock();
+
        // First up, do we have any spare ones?
        int offset = 0;
-       for(int i=0; i<_bat_blocks.size(); i++) {
-          int numSectors = bigBlockSize.getBATEntriesPerBlock();
-
-          // Check this one
-          BATBlock bat = _bat_blocks.get(i);
+       for (BATBlock bat : _bat_blocks) {
           if(bat.hasFreeSectors()) {
              // Claim one of them and return it
              for(int j=0; j<numSectors; j++) {
@@ -524,7 +566,9 @@ public class NPOIFSFileSystem extends BlockStore
           if(xbat == null) {
              // Oh joy, we need a new XBAT too...
              xbat = createBAT(offset+1, false);
+             // Allocate our new BAT as the first block in the XBAT
              xbat.setValueAt(0, offset);
+             // And allocate the XBAT in the BAT
              bat.setValueAt(1, POIFSConstants.DIFAT_SECTOR_BLOCK);
              
              // Will go one place higher as XBAT added in
@@ -540,12 +584,14 @@ public class NPOIFSFileSystem extends BlockStore
              }
              _xbat_blocks.add(xbat);
              _header.setXBATCount(_xbat_blocks.size());
-          }
-          // Allocate us in the XBAT
-          for(int i=0; i<bigBlockSize.getXBATEntriesPerBlock(); i++) {
-             if(xbat.getValueAt(i) == POIFSConstants.UNUSED_BLOCK) {
-                xbat.setValueAt(i, offset);
-             }
+          } else {
+              // Allocate our BAT in the existing XBAT with space
+              for(int i=0; i<bigBlockSize.getXBATEntriesPerBlock(); i++) {
+                 if(xbat.getValueAt(i) == POIFSConstants.UNUSED_BLOCK) {
+                    xbat.setValueAt(i, offset);
+                    break;
+                 }
+              }
           }
        } else {
           // Store us in the header
@@ -560,6 +606,10 @@ public class NPOIFSFileSystem extends BlockStore
        return offset+1;
     }
     
+    protected long size() throws IOException {
+        return _data.size();
+    }
+    
     @Override
     protected ChainLoopDetector getChainLoopDetector() throws IOException {
       return new ChainLoopDetector(_data.size());
@@ -673,6 +723,12 @@ public class NPOIFSFileSystem extends BlockStore
                 "not be called. Use writeFilesystem(OutputStream) instead"
           );
        }
+       if (! ((FileBackedDataSource)_data).isWriteable()) {
+           throw new IllegalArgumentException(
+                "POIFS opened in read only mode, so writeFilesystem() may " +
+                "not be called. Open the FileSystem in read-write mode first"
+           );
+       }
        syncWithDataSource();
     }
 
@@ -699,8 +755,13 @@ public class NPOIFSFileSystem extends BlockStore
      * Has our in-memory objects write their state
      *  to their backing blocks 
      */
-    private void syncWithDataSource() throws IOException
-    {
+    private void syncWithDataSource() throws IOException {
+        // Properties
+        NPOIFSStream propStream = new NPOIFSStream(this, _header.getPropertyStart());
+        _property_table.preWrite();
+        _property_table.write(propStream);
+        // _header.setPropertyStart has been updated on write ...
+        
        // HeaderBlock
        HeaderBlockWriter hbw = new HeaderBlockWriter(_header);
        hbw.writeBlock( getBlockAt(-1) );
@@ -710,14 +771,14 @@ public class NPOIFSFileSystem extends BlockStore
           ByteBuffer block = getBlockAt(bat.getOurBlockIndex());
           BlockAllocationTableWriter.writeBlock(bat, block);
        }
+       // XBats
+       for(BATBlock bat : _xbat_blocks) {
+           ByteBuffer block = getBlockAt(bat.getOurBlockIndex());
+           BlockAllocationTableWriter.writeBlock(bat, block);
+        }
        
        // SBATs
        _mini_store.syncWithDataSource();
-       
-       // Properties
-       _property_table.write(
-             new NPOIFSStream(this, _header.getPropertyStart())
-       );
     }
     
     /**
@@ -801,9 +862,15 @@ public class NPOIFSFileSystem extends BlockStore
      *
      * @param entry to be removed
      */
-
-    void remove(EntryNode entry)
+    void remove(EntryNode entry) throws IOException
     {
+        // If it's a document, free the blocks
+        if (entry instanceof DocumentEntry) {
+            NPOIFSDocument doc = new NPOIFSDocument((DocumentProperty)entry.getProperty(), this);
+            doc.free();
+        }
+        
+        // Now zap it from the properties list
         _property_table.removeProperty(entry.getProperty());
     }
     
