@@ -16,8 +16,6 @@
 ==================================================================== */
 package org.apache.poi.ss.format;
 
-import org.apache.poi.ss.format.CellFormatPart.PartHandler;
-
 import java.text.DecimalFormat;
 import java.text.FieldPosition;
 import java.util.BitSet;
@@ -31,6 +29,8 @@ import java.util.Set;
 import java.util.TreeSet;
 import java.util.regex.Matcher;
 
+import org.apache.poi.ss.format.CellFormatPart.PartHandler;
+
 /**
  * This class implements printing out a value using a number format.
  *
@@ -595,7 +595,11 @@ public class CellNumberFormatter extends CellFormatter {
         } else {
             StringBuffer result = new StringBuffer();
             Formatter f = new Formatter(result);
-            f.format(LOCALE, printfFmt, value);
+            try {
+                f.format(LOCALE, printfFmt, value);
+            } finally {
+                f.close();
+            }
 
             if (numerator == null) {
                 writeFractional(result, output);
@@ -658,7 +662,7 @@ public class CellNumberFormatter extends CellFormatter {
                             delEndPos + adjust; // delete end point in current
 
                     if (modPos < modEndPos) {
-                        if (nextChange.toAdd == "")
+                        if ("".equals(nextChange.toAdd))
                             output.delete(modPos, modEndPos);
                         else {
                             char fillCh = nextChange.toAdd.charAt(0);
@@ -866,7 +870,11 @@ public class CellNumberFormatter extends CellFormatter {
 
         StringBuffer sb = new StringBuffer();
         Formatter formatter = new Formatter(sb);
-        formatter.format(LOCALE, fmt, num);
+        try {
+            formatter.format(LOCALE, fmt, num);
+        } finally {
+            formatter.close();
+        }
         writeInteger(sb, output, numSpecials, mods, false);
     }
 
