@@ -56,7 +56,7 @@ public class DateUtil {
     private static final Pattern date_ptrn1 = Pattern.compile("^\\[\\$\\-.*?\\]");
     private static final Pattern date_ptrn2 = Pattern.compile("^\\[[a-zA-Z]+\\]");
     private static final Pattern date_ptrn3a = Pattern.compile("[yYmMdDhHsS]");
-    private static final Pattern date_ptrn3b = Pattern.compile("^[\\[\\]yYmMdDhHsS\\-/,. :\"\\\\]+0*[ampAMP/]*$");
+    private static final Pattern date_ptrn3b = Pattern.compile("^[\\[\\]yYmMdDhHsS\\-T/,. :\"\\\\]+0*[ampAMP/]*$");
     //  elapsed time patterns: [h],[m] and [s]
     private static final Pattern date_ptrn4 = Pattern.compile("^\\[([hH]+|[mM]+|[sS]+)\\]");
 
@@ -185,8 +185,29 @@ public class DateUtil {
      *  @return Java representation of the date, or null if date is not a valid Excel date
      */
     public static Date getJavaDate(double date, boolean use1904windowing, TimeZone tz) {
-        return getJavaCalendar(date, use1904windowing, tz).getTime();
+        return getJavaCalendar(date, use1904windowing, tz, false).getTime();
     }
+    
+    /**
+     *  Given an Excel date with either 1900 or 1904 date windowing,
+     *  converts it to a java.util.Date.
+     *  
+     *  Excel Dates and Times are stored without any timezone 
+     *  information. If you know (through other means) that your file 
+     *  uses a different TimeZone to the system default, you can use
+     *  this version of the getJavaDate() method to handle it.
+     *   
+     *  @param date  The Excel date.
+     *  @param tz The TimeZone to evaluate the date in
+     *  @param use1904windowing  true if date uses 1904 windowing,
+     *   or false if using 1900 date windowing.
+     *  @param roundSeconds round to closest second
+     *  @return Java representation of the date, or null if date is not a valid Excel date
+     */
+    public static Date getJavaDate(double date, boolean use1904windowing, TimeZone tz, boolean roundSeconds) {
+        return getJavaCalendar(date, use1904windowing, tz, roundSeconds).getTime();
+    }
+    
     /**
      *  Given an Excel date with either 1900 or 1904 date windowing,
      *  converts it to a java.util.Date.
@@ -207,12 +228,12 @@ public class DateUtil {
      *  @see java.util.TimeZone
      */
     public static Date getJavaDate(double date, boolean use1904windowing) {
-        return getJavaCalendar(date, use1904windowing).getTime();
+        return getJavaCalendar(date, use1904windowing, null, false).getTime();
     }
 
 
     public static void setCalendar(Calendar calendar, int wholeDays,
-            int millisecondsInDay, boolean use1904windowing) {
+            int millisecondsInDay, boolean use1904windowing, boolean roundSeconds) {
         int startYear = 1900;
         int dayAdjust = -1; // Excel thinks 2/29/1900 is a valid date, which it isn't
         if (use1904windowing) {
@@ -225,10 +246,24 @@ public class DateUtil {
             dayAdjust = 0;
         }
         calendar.set(startYear,0, wholeDays + dayAdjust, 0, 0, 0);
-        calendar.set(GregorianCalendar.MILLISECOND, millisecondsInDay);
+        calendar.set(Calendar.MILLISECOND, millisecondsInDay);
+        if (roundSeconds) {
+            calendar.add(Calendar.MILLISECOND, 500);
+            calendar.clear(Calendar.MILLISECOND);
+        }
     }
 
 
+    /**
+     * Get EXCEL date as Java Calendar (with default time zone).
+     * This is like {@link #getJavaDate(double)} but returns a Calendar object.
+     *  @param date  The Excel date.
+     *  @return Java representation of the date, or null if date is not a valid Excel date
+     */
+    public static Calendar getJavaCalendar(double date) {
+        return getJavaCalendar(date, false, (TimeZone)null, false);
+    }
+
     /**
      * Get EXCEL date as Java Calendar (with default time zone).
      * This is like {@link #getJavaDate(double, boolean)} but returns a Calendar object.
@@ -238,29 +273,45 @@ public class DateUtil {
      *  @return Java representation of the date, or null if date is not a valid Excel date
      */
     public static Calendar getJavaCalendar(double date, boolean use1904windowing) {
-    	return getJavaCalendar(date, use1904windowing, (TimeZone)null);
+        return getJavaCalendar(date, use1904windowing, (TimeZone)null, false);
     }
 
     /**
      * Get EXCEL date as Java Calendar with UTC time zone.
      * This is similar to {@link #getJavaDate(double, boolean)} but returns a
      * Calendar object that has UTC as time zone, so no daylight saving hassle.
-     *  @param date  The Excel date.
-     *  @param use1904windowing  true if date uses 1904 windowing,
-     *   or false if using 1900 date windowing.
-     *  @return Java representation of the date in UTC, or null if date is not a valid Excel date
+     * @param date  The Excel date.
+     * @param use1904windowing  true if date uses 1904 windowing,
+     *  or false if using 1900 date windowing.
+     * @return Java representation of the date in UTC, or null if date is not a valid Excel date
      */
     public static Calendar getJavaCalendarUTC(double date, boolean use1904windowing) {
-    	return getJavaCalendar(date, use1904windowing, TIMEZONE_UTC);
+    	return getJavaCalendar(date, use1904windowing, TIMEZONE_UTC, false);
     }
 
 
     /**
      * Get EXCEL date as Java Calendar with given time zone.
-     * @see #getJavaDate(double, TimeZone)
+     * @param date  The Excel date.
+     * @param use1904windowing  true if date uses 1904 windowing,
+     *  or false if using 1900 date windowing.
+     * @param timeZone The TimeZone to evaluate the date in
      * @return Java representation of the date, or null if date is not a valid Excel date
      */
     public static Calendar getJavaCalendar(double date, boolean use1904windowing, TimeZone timeZone) {
+        return getJavaCalendar(date, use1904windowing, timeZone, false);
+    }
+        
+    /**
+     * Get EXCEL date as Java Calendar with given time zone.
+     * @param date  The Excel date.
+     * @param use1904windowing  true if date uses 1904 windowing,
+     *  or false if using 1900 date windowing.
+     * @param timeZone The TimeZone to evaluate the date in
+     * @param roundSeconds round to closest second
+     * @return Java representation of the date, or null if date is not a valid Excel date
+     */
+    public static Calendar getJavaCalendar(double date, boolean use1904windowing, TimeZone timeZone, boolean roundSeconds) {
         if (!isValidExcelDate(date)) {
             return null;
         }
@@ -272,19 +323,34 @@ public class DateUtil {
         } else {
             calendar = new GregorianCalendar();     // using default time-zone
         }
-        setCalendar(calendar, wholeDays, millisecondsInDay, use1904windowing);
+        setCalendar(calendar, wholeDays, millisecondsInDay, use1904windowing, roundSeconds);
         return calendar;
     }
 
-
     // variables for performance optimization:
     // avoid re-checking DataUtil.isADateFormat(int, String) if a given format
     // string represents a date format if the same string is passed multiple times.
     // see https://issues.apache.org/bugzilla/show_bug.cgi?id=55611
-    private static int lastFormatIndex = -1;
-    private static String lastFormatString = null;
-    private static boolean cached = false;
+    private static ThreadLocal<Integer> lastFormatIndex = new ThreadLocal<Integer>() {
+        protected Integer initialValue() {
+            return -1;
+        }
+    };
+    private static ThreadLocal<String> lastFormatString = new ThreadLocal<String>();
+    private static ThreadLocal<Boolean> lastCachedResult = new ThreadLocal<Boolean>();
 
+    private static boolean isCached(String formatString, int formatIndex) {
+        String cachedFormatString = lastFormatString.get();
+        return cachedFormatString != null && formatIndex == lastFormatIndex.get()
+                && formatString.equals(cachedFormatString);
+    }
+
+    private static void cache(String formatString, int formatIndex, boolean cached) {
+        lastFormatIndex.set(formatIndex);
+        lastFormatString.set(formatString);
+        lastCachedResult.set(Boolean.valueOf(cached));
+    }
+    
     /**
      * Given a format ID and its format String, will check to see if the
      *  format represents a date format or not.
@@ -299,27 +365,23 @@ public class DateUtil {
      * @see #isInternalDateFormat(int)
      */
 
-    public static synchronized boolean isADateFormat(int formatIndex, String formatString) {
-       
-         if (formatString != null && formatIndex == lastFormatIndex && formatString.equals(lastFormatString)) {
-           		return cached;
-         }
+    public static boolean isADateFormat(int formatIndex, String formatString) {
         // First up, is this an internal date format?
         if(isInternalDateFormat(formatIndex)) {
-            lastFormatIndex = formatIndex;
-            lastFormatString = formatString;
-            cached = true;
+            cache(formatString, formatIndex, true);
             return true;
         }
 
-        // If we didn't get a real string, it can't be
+        // If we didn't get a real string, don't even cache it as we can always find this out quickly
         if(formatString == null || formatString.length() == 0) {
-            lastFormatIndex = formatIndex;
-            lastFormatString = formatString;
-            cached = false;
             return false;
         }
 
+        // check the cache first
+        if (isCached(formatString, formatIndex)) {
+            return lastCachedResult.get();
+        }
+
         String fs = formatString;
         /*if (false) {
             // Normalize the format string. The code below is equivalent
@@ -368,9 +430,7 @@ public class DateUtil {
 
         // short-circuit if it indicates elapsed time: [h], [m] or [s]
         if(date_ptrn4.matcher(fs).matches()){
-            lastFormatIndex = formatIndex;
-            lastFormatString = formatString;
-            cached = true;
+            cache(formatString, formatIndex, true);
             return true;
         }
 
@@ -394,13 +454,11 @@ public class DateUtil {
         }
         
         // If we get here, check it's only made up, in any case, of:
-        //  y m d h s - \ / , . : [ ]
+        //  y m d h s - \ / , . : [ ] T
         // optionally followed by AM/PM
 
         boolean result = date_ptrn3b.matcher(fs).matches();
-        lastFormatIndex = formatIndex;
-        lastFormatString = formatString;
-        cached = result;
+        cache(formatString, formatIndex, result);
         return result;
     }
 
@@ -537,6 +595,7 @@ public class DateUtil {
     }
 
 
+    @SuppressWarnings("serial")
     private static final class FormatException extends Exception {
         public FormatException(String msg) {
             super(msg);
