@@ -18,6 +18,7 @@
 package org.apache.poi.ss.usermodel;
 
 import java.util.Iterator;
+import java.util.List;
 
 import org.apache.poi.hssf.util.PaneInformation;
 import org.apache.poi.ss.util.CellRangeAddress;
@@ -185,11 +186,26 @@ public interface Sheet extends Iterable<Row> {
      * using the default font (first font in the workbook)
      * </p>
      *
-     * @param columnIndex - the column to set (0-based)
+     * @param columnIndex - the column to get (0-based)
      * @return width - the width in units of 1/256th of a character width
      */
     int getColumnWidth(int columnIndex);
 
+    /**
+     * get the width in pixel
+     * 
+     * <p>
+     * Please note, that this method works correctly only for workbooks
+     * with the default font size (Arial 10pt for .xls and Calibri 11pt for .xlsx).
+     * If the default font is changed the column width can be streched
+     * </p>
+     *
+     * @param columnIndex - the column to set (0-based)
+     * @return width in pixels
+     */
+    float getColumnWidthInPixels(int columnIndex);
+    
+    
     /**
      * Set the default column width for the sheet (if the columns do not define their own width)
      * in characters
@@ -213,7 +229,7 @@ public interface Sheet extends Iterable<Row> {
      * @return  default row height measured in twips (1/20 of  a point)
      */
     short getDefaultRowHeight();
-
+    
     /**
      * Get the default row height for the sheet (if the rows do not define their own height) in
      * points.
@@ -922,6 +938,12 @@ public interface Sheet extends Iterable<Row> {
     public DataValidationHelper getDataValidationHelper();
 
 	/**
+     * Returns the list of DataValidation in the sheet.
+     * @return list of DataValidation in the sheet
+     */
+    public List<? extends DataValidation> getDataValidations();
+
+	/**
 	 * Creates a data validation object
 	 * @param dataValidation The Data validation object settings
 	 */
@@ -1031,5 +1053,11 @@ public interface Sheet extends Iterable<Row> {
      *        columns for the Sheet, or null.
      */
     void setRepeatingColumns(CellRangeAddress columnRangeRef);
-
+    
+    /**
+     * Returns the column outline level. Increased as you
+     *  put it into more groups (outlines), reduced as
+     *  you take it out of them.
+     */
+    int getColumnOutlineLevel(int columnIndex);
 }
