@@ -133,10 +133,10 @@ public class CellReference {
 		// TODO - "-1" is a special value being temporarily used for whole row and whole column area references.
 		// so these checks are currently N.Q.R.
 		if(pRow < -1) {
-			throw new IllegalArgumentException("row index may not be negative");
+			throw new IllegalArgumentException("row index may not be negative, but had " + pRow);
 		}
 		if(pCol < -1) {
-			throw new IllegalArgumentException("column index may not be negative");
+			throw new IllegalArgumentException("column index may not be negative, but had " + pCol);
 		}
 		_sheetName = pSheetName;
 		_rowIndex=pRow;
@@ -499,22 +499,35 @@ public class CellReference {
 		}
 	}
 
-	/**
-	 * Checks whether this cell reference is equal to another object.
-	 * <p>
-	 *  Two cells references are assumed to be equal if their string representations
-	 *  ({@link #formatAsString()}  are equal.
-	 * </p>
-	 */
-	@Override
-	public boolean equals(Object o){
-		if(!(o instanceof CellReference)) {
-			return false;
-		}
-		CellReference cr = (CellReference) o;
-		return _rowIndex == cr._rowIndex
-			&& _colIndex == cr._colIndex
-			&& _isRowAbs == cr._isColAbs
-			&& _isColAbs == cr._isColAbs;
-	}
+   /**
+    * Checks whether this cell reference is equal to another object.
+    * <p>
+    *  Two cells references are assumed to be equal if their string representations
+    *  ({@link #formatAsString()}  are equal.
+    * </p>
+    */
+   @Override
+   public boolean equals(Object o){
+      if (this == o) {
+         return true;
+      }
+      if(!(o instanceof CellReference)) {
+         return false;
+      }
+      CellReference cr = (CellReference) o;
+      return _rowIndex == cr._rowIndex
+              && _colIndex == cr._colIndex
+              && _isRowAbs == cr._isRowAbs
+              && _isColAbs == cr._isColAbs;
+    }
+
+    @Override
+    public int hashCode() {
+      int result = 17;
+      result = 31 * result + _rowIndex;
+      result = 31 * result + _colIndex;
+      result = 31 * result + (_isRowAbs ? 1 : 0);
+      result = 31 * result + (_isColAbs ? 1 : 0);
+      return result;
+   }
 }
