@@ -29,6 +29,7 @@ import java.io.OutputStream;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.Date;
 import java.util.Hashtable;
 import java.util.List;
@@ -403,6 +404,11 @@ public abstract class OPCPackage implements RelationshipSource, Closeable {
 			revert();
 			return;
 		}
+		if (this.contentTypeManager == null) {
+		    logger.log(POILogger.WARN,
+		            "Unable to call close() on a package that hasn't been fully opened yet");
+		    return;
+		}
 
 		// Save the content
 		ReentrantReadWriteLock l = new ReentrantReadWriteLock();
@@ -576,6 +582,7 @@ public abstract class OPCPackage implements RelationshipSource, Closeable {
 			if (part.getContentType().equals(contentType))
 				retArr.add(part);
 		}
+		Collections.sort(retArr);
 		return retArr;
 	}
 
@@ -594,24 +601,36 @@ public abstract class OPCPackage implements RelationshipSource, Closeable {
 			throw new IllegalArgumentException("relationshipType");
 		ArrayList<PackagePart> retArr = new ArrayList<PackagePart>();
 		for (PackageRelationship rel : getRelationshipsByType(relationshipType)) {
-			retArr.add(getPart(rel));
+			PackagePart part = getPart(rel);
+			if (part != null) {
+			    retArr.add(part);
+			}
 		}
+		Collections.sort(retArr);
 		return retArr;
 	}
 
+	/**
+	 * Retrieve parts by name
+	 *
+	 * @param namePattern
+	 *            The pattern for matching the names
+	 * @return All parts associated to the specified content type, sorted
+	 * in alphanumerically by the part-name
+	 */
 	public List<PackagePart> getPartsByName(final Pattern namePattern) {
 	    if (namePattern == null) {
 	        throw new IllegalArgumentException("name pattern must not be null");
 	    }
+	    Matcher matcher = namePattern.matcher("");
 	    ArrayList<PackagePart> result = new ArrayList<PackagePart>();
 	    for (PackagePart part : partList.values()) {
 	        PackagePartName partName = part.getPartName();
-	        String name = partName.getName();
-	        Matcher matcher = namePattern.matcher(name);
-	        if (matcher.matches()) {
+	        if (matcher.reset(partName.getName()).matches()) {
 	            result.add(part);
 	        }
 	    }
+	    Collections.sort(result);
 	    return result;
 	}
 
@@ -719,7 +738,9 @@ public abstract class OPCPackage implements RelationshipSource, Closeable {
 				}
 			}
 		}
-		return new ArrayList<PackagePart>(partList.values());
+		ArrayList<PackagePart> result = new ArrayList<PackagePart>(partList.values());
+		java.util.Collections.sort(result);
+		return result;
 	}
 
 	/**
@@ -731,7 +752,7 @@ public abstract class OPCPackage implements RelationshipSource, Closeable {
 	 * @param contentType
 	 *            Part content type.
 	 * @return The newly created part.
-	 * @throws InvalidFormatException
+	 * @throws PartAlreadyExistsException
 	 *             If rule M1.12 is not verified : Packages shall not contain
 	 *             equivalent part names and package implementers shall neither
 	 *             create nor recognize packages with equivalent part names.
@@ -754,7 +775,7 @@ public abstract class OPCPackage implements RelationshipSource, Closeable {
 	 *            Specify if the existing relationship part, if any, logically
 	 *            associated to the newly created part will be loaded.
 	 * @return The newly created part.
-	 * @throws InvalidFormatException
+	 * @throws PartAlreadyExistsException
 	 *             If rule M1.12 is not verified : Packages shall not contain
 	 *             equivalent part names and package implementers shall neither
 	 *             create nor recognize packages with equivalent part names.
@@ -1401,8 +1422,11 @@ public abstract class OPCPackage implements RelationshipSource, Closeable {
 		} catch (FileNotFoundException e) {
 			throw new IOException(e.getLocalizedMessage());
 		}
-		this.save(fos);
-		fos.close();
+		try {
+			this.save(fos);
+		} finally {
+			fos.close();
+		}
 	}
 
 	/**
