@@ -27,7 +27,6 @@ import java.io.OutputStream;
 import java.lang.reflect.Field;
 import java.net.URI;
 import java.util.HashMap;
-import java.util.Iterator;
 import java.util.List;
 import java.util.TreeMap;
 import java.util.regex.Pattern;
@@ -40,15 +39,13 @@ import org.apache.poi.openxml4j.exceptions.InvalidOperationException;
 import org.apache.poi.openxml4j.opc.internal.ContentTypeManager;
 import org.apache.poi.openxml4j.opc.internal.FileHelper;
 import org.apache.poi.openxml4j.opc.internal.PackagePropertiesPart;
+import org.apache.poi.util.DocumentHelper;
 import org.apache.poi.util.POILogFactory;
 import org.apache.poi.util.POILogger;
-import org.apache.poi.util.SAXHelper;
 import org.apache.poi.util.TempFile;
-import org.dom4j.Document;
-import org.dom4j.DocumentHelper;
-import org.dom4j.Element;
-import org.dom4j.Namespace;
-import org.dom4j.QName;
+import org.w3c.dom.Document;
+import org.w3c.dom.Element;
+import org.w3c.dom.NodeList;
 
 public final class TestPackage extends TestCase {
     private static final POILogger logger = POILogFactory.getLogger(TestPackage.class);
@@ -61,12 +58,17 @@ public final class TestPackage extends TestCase {
 		File targetFile = OpenXML4JTestDataSamples.getOutputFile("TestPackageOpenSaveTMP.docx");
 
 		OPCPackage p = OPCPackage.open(originalFile, PackageAccess.READ_WRITE);
-		p.save(targetFile.getAbsoluteFile());
-
-		// Compare the original and newly saved document
-		assertTrue(targetFile.exists());
-		ZipFileAssert.assertEquals(new File(originalFile), targetFile);
-		assertTrue(targetFile.delete());
+		try {
+    		p.save(targetFile.getAbsoluteFile());
+    
+    		// Compare the original and newly saved document
+    		assertTrue(targetFile.exists());
+    		ZipFileAssert.assertEquals(new File(originalFile), targetFile);
+    		assertTrue(targetFile.delete());
+		} finally {
+            // use revert to not re-write the input file
+            p.revert();
+		}
 	}
 
 	/**
@@ -127,18 +129,17 @@ public final class TestPackage extends TestCase {
                         "application/vnd.openxmlformats-officedocument.wordprocessingml.document.main+xml");
 
         Document doc = DocumentHelper.createDocument();
-        Namespace nsWordprocessinML = new Namespace("w",
-                "http://schemas.openxmlformats.org/wordprocessingml/2006/main");
-        Element elDocument = doc.addElement(new QName("document",
-                nsWordprocessinML));
-        Element elBody = elDocument.addElement(new QName("body",
-                nsWordprocessinML));
-        Element elParagraph = elBody.addElement(new QName("p",
-                nsWordprocessinML));
-        Element elRun = elParagraph
-                .addElement(new QName("r", nsWordprocessinML));
-        Element elText = elRun.addElement(new QName("t", nsWordprocessinML));
-        elText.setText("Hello Open XML !");
+        Element elDocument = doc.createElementNS("http://schemas.openxmlformats.org/wordprocessingml/2006/main", "w:document");
+        doc.appendChild(elDocument);
+        Element elBody = doc.createElementNS("http://schemas.openxmlformats.org/wordprocessingml/2006/main", "w:body");
+        elDocument.appendChild(elBody);
+        Element elParagraph = doc.createElementNS("http://schemas.openxmlformats.org/wordprocessingml/2006/main", "w:p");
+        elBody.appendChild(elParagraph);
+        Element elRun = doc.createElementNS("http://schemas.openxmlformats.org/wordprocessingml/2006/main", "w:r");
+        elParagraph.appendChild(elRun);
+        Element elText = doc.createElementNS("http://schemas.openxmlformats.org/wordprocessingml/2006/main", "w:t");
+        elRun.appendChild(elText);
+        elText.setTextContent("Hello Open XML !");
 
         StreamHelper.saveXmlInStream(doc, corePart.getOutputStream());
         pkg.close();
@@ -172,6 +173,8 @@ public final class TestPackage extends TestCase {
         PackageRelationship rel =
         	 corePart.addRelationship(sheetPartName, TargetMode.INTERNAL, "http://schemas.openxmlformats.org/officeDocument/2006/relationships/worksheet", "rSheet1");
         PackagePart part = pkg.createPart(sheetPartName, "application/vnd.openxmlformats-officedocument.spreadsheetml.worksheet+xml");
+        assertNotNull(part);
+
         // Dummy content again
         coreOut = corePart.getOutputStream();
         coreOut.write("<dummy-xml2 />".getBytes());
@@ -193,45 +196,50 @@ public final class TestPackage extends TestCase {
         // Save and re-load
         pkg.close();
         File tmp = TempFile.createTempFile("testCreatePackageWithCoreDocument", ".zip");
-            FileOutputStream fout = new FileOutputStream(tmp);
-        fout.write(baos.toByteArray());
-        fout.close();
+        OutputStream fout = new FileOutputStream(tmp);
+        try {
+            fout.write(baos.toByteArray());
+        } finally {
+            fout.close();
+        }
         pkg = OPCPackage.open(tmp.getPath());
         //tmp.delete();
 
-        // Check still right
-        coreRels = pkg.getRelationshipsByType(PackageRelationshipTypes.CORE_DOCUMENT);
-        assertEquals(1, coreRels.size());
-        coreRel = coreRels.getRelationship(0);
-
-        assertEquals("/", coreRel.getSourceURI().toString());
-        assertEquals("/xl/workbook.xml", coreRel.getTargetURI().toString());
-        corePart = pkg.getPart(coreRel);
-        assertNotNull(corePart);
-
-        PackageRelationshipCollection rels = corePart.getRelationshipsByType("http://schemas.openxmlformats.org/officeDocument/2006/relationships/hyperlink");
-        assertEquals(1, rels.size());
-        rel = rels.getRelationship(0);
-        assertEquals("Sheet1!A1", rel.getTargetURI().getRawFragment());
-
-        assertMSCompatibility(pkg);
+        try {
+            // Check still right
+            coreRels = pkg.getRelationshipsByType(PackageRelationshipTypes.CORE_DOCUMENT);
+            assertEquals(1, coreRels.size());
+            coreRel = coreRels.getRelationship(0);
+    
+            assertEquals("/", coreRel.getSourceURI().toString());
+            assertEquals("/xl/workbook.xml", coreRel.getTargetURI().toString());
+            corePart = pkg.getPart(coreRel);
+            assertNotNull(corePart);
+    
+            PackageRelationshipCollection rels = corePart.getRelationshipsByType("http://schemas.openxmlformats.org/officeDocument/2006/relationships/hyperlink");
+            assertEquals(1, rels.size());
+            rel = rels.getRelationship(0);
+            assertEquals("Sheet1!A1", rel.getTargetURI().getRawFragment());
+    
+            assertMSCompatibility(pkg);
+        } finally {
+            pkg.close();
+        }
     }
 
     private void assertMSCompatibility(OPCPackage pkg) throws Exception {
         PackagePartName relName = PackagingURIHelper.createPartName(PackageRelationship.getContainerPartRelationship());
         PackagePart relPart = pkg.getPart(relName);
 
-        Document xmlRelationshipsDoc = SAXHelper.readSAXDocument(relPart.getInputStream());
+        Document xmlRelationshipsDoc = DocumentHelper.readDocument(relPart.getInputStream());
 
-        Element root = xmlRelationshipsDoc.getRootElement();
-        for (Iterator i = root
-                .elementIterator(PackageRelationship.RELATIONSHIP_TAG_NAME); i
-                .hasNext();) {
-            Element element = (Element) i.next();
-            String value = element.attribute(
-                    PackageRelationship.TARGET_ATTRIBUTE_NAME)
-                    .getValue();
-            assertTrue("Root target must not start with a leadng slash ('/'): " + value, value.charAt(0) != '/');
+        Element root = xmlRelationshipsDoc.getDocumentElement();
+        NodeList nodeList = root.getElementsByTagName(PackageRelationship.RELATIONSHIP_TAG_NAME);
+        int nodeCount = nodeList.getLength();
+        for (int i = 0; i < nodeCount; i++) {
+            Element element = (Element) nodeList.item(i);
+            String value = element.getAttribute(PackageRelationship.TARGET_ATTRIBUTE_NAME);
+            assertTrue("Root target must not start with a leading slash ('/'): " + value, value.charAt(0) != '/');
         }
 
     }
@@ -268,18 +276,17 @@ public final class TestPackage extends TestCase {
 
 		// Create a content
 		Document doc = DocumentHelper.createDocument();
-		Namespace nsWordprocessinML = new Namespace("w",
-				"http://schemas.openxmlformats.org/wordprocessingml/2006/main");
-		Element elDocument = doc.addElement(new QName("document",
-				nsWordprocessinML));
-		Element elBody = elDocument.addElement(new QName("body",
-				nsWordprocessinML));
-		Element elParagraph = elBody.addElement(new QName("p",
-				nsWordprocessinML));
-		Element elRun = elParagraph
-				.addElement(new QName("r", nsWordprocessinML));
-		Element elText = elRun.addElement(new QName("t", nsWordprocessinML));
-		elText.setText("Hello Open XML !");
+        Element elDocument = doc.createElementNS("http://schemas.openxmlformats.org/wordprocessingml/2006/main", "w:document");
+        doc.appendChild(elDocument);
+        Element elBody = doc.createElementNS("http://schemas.openxmlformats.org/wordprocessingml/2006/main", "w:body");
+        elDocument.appendChild(elBody);
+        Element elParagraph = doc.createElementNS("http://schemas.openxmlformats.org/wordprocessingml/2006/main", "w:p");
+        elBody.appendChild(elParagraph);
+        Element elRun = doc.createElementNS("http://schemas.openxmlformats.org/wordprocessingml/2006/main", "w:r");
+        elParagraph.appendChild(elRun);
+        Element elText = doc.createElementNS("http://schemas.openxmlformats.org/wordprocessingml/2006/main", "w:t");
+        elRun.appendChild(elText);
+        elText.setTextContent("Hello Open XML !");
 
 		StreamHelper.saveXmlInStream(doc, corePart.getOutputStream());
 
@@ -304,14 +311,22 @@ public final class TestPackage extends TestCase {
 		File targetFile = OpenXML4JTestDataSamples.getOutputFile("TestPackageOpenSaveTMP.docx");
 
 		OPCPackage p = OPCPackage.open(originalFile, PackageAccess.READ_WRITE);
-		FileOutputStream fout = new FileOutputStream(targetFile);
-		p.save(fout);
-		fout.close();
-
-		// Compare the original and newly saved document
-		assertTrue(targetFile.exists());
-		ZipFileAssert.assertEquals(new File(originalFile), targetFile);
-		assertTrue(targetFile.delete());
+		try {
+    		FileOutputStream fout = new FileOutputStream(targetFile);
+    		try {
+    		    p.save(fout);
+    		} finally {
+    		    fout.close();
+    		}
+    
+    		// Compare the original and newly saved document
+    		assertTrue(targetFile.exists());
+    		ZipFileAssert.assertEquals(new File(originalFile), targetFile);
+    		assertTrue(targetFile.delete());
+		} finally {
+		    // use revert to not re-write the input file
+		    p.revert();
+		}
 	}
 
 	/**
@@ -454,7 +469,7 @@ public final class TestPackage extends TestCase {
 	 *  write changes to it.
 	 */
 	public void testOpenFileThenOverwrite() throws Exception {
-        File tempFile = File.createTempFile("poiTesting","tmp");
+        File tempFile = TempFile.createTempFile("poiTesting","tmp");
         File origFile = OpenXML4JTestDataSamples.getSampleFile("TestPackageCommon.docx");
         FileHelper.copyFile(origFile, tempFile);
         
@@ -491,8 +506,8 @@ public final class TestPackage extends TestCase {
      *  to another file, then delete both
      */
     public void testOpenFileThenSaveDelete() throws Exception {
-        File tempFile = File.createTempFile("poiTesting","tmp");
-        File tempFile2 = File.createTempFile("poiTesting","tmp");
+        File tempFile = TempFile.createTempFile("poiTesting","tmp");
+        File tempFile2 = TempFile.createTempFile("poiTesting","tmp");
         File origFile = OpenXML4JTestDataSamples.getSampleFile("TestPackageCommon.docx");
         FileHelper.copyFile(origFile, tempFile);
         
@@ -518,48 +533,56 @@ public final class TestPackage extends TestCase {
         String filepath =  OpenXML4JTestDataSamples.getSampleFileName("sample.docx");
 
         OPCPackage pkg = OPCPackage.open(filepath, PackageAccess.READ_WRITE);
-        List<PackagePart> rs =  pkg.getPartsByName(Pattern.compile("/word/.*?\\.xml"));
-        HashMap<String, PackagePart>  selected = new HashMap<String, PackagePart>();
-
-        for(PackagePart p : rs)
-            selected.put(p.getPartName().getName(), p);
-
-        assertEquals(6, selected.size());
-        assertTrue(selected.containsKey("/word/document.xml"));
-        assertTrue(selected.containsKey("/word/fontTable.xml"));
-        assertTrue(selected.containsKey("/word/settings.xml"));
-        assertTrue(selected.containsKey("/word/styles.xml"));
-        assertTrue(selected.containsKey("/word/theme/theme1.xml"));
-        assertTrue(selected.containsKey("/word/webSettings.xml"));
+        try {
+            List<PackagePart> rs =  pkg.getPartsByName(Pattern.compile("/word/.*?\\.xml"));
+            HashMap<String, PackagePart>  selected = new HashMap<String, PackagePart>();
+    
+            for(PackagePart p : rs)
+                selected.put(p.getPartName().getName(), p);
+    
+            assertEquals(6, selected.size());
+            assertTrue(selected.containsKey("/word/document.xml"));
+            assertTrue(selected.containsKey("/word/fontTable.xml"));
+            assertTrue(selected.containsKey("/word/settings.xml"));
+            assertTrue(selected.containsKey("/word/styles.xml"));
+            assertTrue(selected.containsKey("/word/theme/theme1.xml"));
+            assertTrue(selected.containsKey("/word/webSettings.xml"));
+        } finally {
+            // use revert to not re-write the input file
+            pkg.revert();
+        }
     }
     
     public void testGetPartSize() throws Exception {
        String filepath =  OpenXML4JTestDataSamples.getSampleFileName("sample.docx");
        OPCPackage pkg = OPCPackage.open(filepath, PackageAccess.READ);
-
-       int checked = 0;
-       for (PackagePart part : pkg.getParts()) {
-          // Can get the size of zip parts
-          if (part.getPartName().getName().equals("/word/document.xml")) {
-             checked++;
-             assertEquals(ZipPackagePart.class, part.getClass());
-             assertEquals(6031l, part.getSize());
-          }
-          if (part.getPartName().getName().equals("/word/fontTable.xml")) {
-             checked++;
-             assertEquals(ZipPackagePart.class, part.getClass());
-             assertEquals(1312l, part.getSize());
-          }
-          
-          // But not from the others
-          if (part.getPartName().getName().equals("/docProps/core.xml")) {
-             checked++;
-             assertEquals(PackagePropertiesPart.class, part.getClass());
-             assertEquals(-1, part.getSize());
-          }
+       try {
+           int checked = 0;
+           for (PackagePart part : pkg.getParts()) {
+              // Can get the size of zip parts
+              if (part.getPartName().getName().equals("/word/document.xml")) {
+                 checked++;
+                 assertEquals(ZipPackagePart.class, part.getClass());
+                 assertEquals(6031l, part.getSize());
+              }
+              if (part.getPartName().getName().equals("/word/fontTable.xml")) {
+                 checked++;
+                 assertEquals(ZipPackagePart.class, part.getClass());
+                 assertEquals(1312l, part.getSize());
+              }
+              
+              // But not from the others
+              if (part.getPartName().getName().equals("/docProps/core.xml")) {
+                 checked++;
+                 assertEquals(PackagePropertiesPart.class, part.getClass());
+                 assertEquals(-1, part.getSize());
+              }
+           }
+           // Ensure we actually found the parts we want to check
+           assertEquals(3, checked);
+       } finally {
+           pkg.close();
        }
-       // Ensure we actually found the parts we want to check
-       assertEquals(3, checked);
     }
 
     public void testReplaceContentType() throws Exception {
