@@ -188,7 +188,11 @@ public class TestRelationships extends TestCase {
 	    // Write out and re-load
 	    ByteArrayOutputStream baos = new ByteArrayOutputStream();
 	    pkg.save(baos);
-	    ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
+        
+	    // use revert to not re-write the input file
+        pkg.revert();
+
+        ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
 	    pkg = OPCPackage.open(bais);
 	    
 	    // Check again
@@ -280,7 +284,6 @@ public class TestRelationships extends TestCase {
 
 
     public void testTargetWithSpecialChars() throws Exception{
-
         OPCPackage pkg;
 
         String filepath = OpenXML4JTestDataSamples.getSampleFileName("50154.xlsx");
@@ -289,6 +292,10 @@ public class TestRelationships extends TestCase {
 
         ByteArrayOutputStream baos = new ByteArrayOutputStream();
         pkg.save(baos);
+
+        // use revert to not re-write the input file
+        pkg.revert();
+
         ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
         pkg = OPCPackage.open(bais);
 
@@ -416,7 +423,7 @@ public class TestRelationships extends TestCase {
         assertTrue("Ext Props Relationship not found in " + p.getRelationships(), foundExtPropRel);
         
         // Should have normal work parts
-        boolean foundCoreProps = false, foundDocument = false;
+        boolean foundCoreProps = false, foundDocument = false, foundTheme1 = false;
         for (PackagePart part : p.getParts()) {
             if (part.getPartName().toString().equals("/docProps/core.xml")) {
                 assertEquals(ContentTypes.CORE_PROPERTIES_PART, part.getContentType());
@@ -426,8 +433,13 @@ public class TestRelationships extends TestCase {
                 assertEquals(XWPFRelation.DOCUMENT.getContentType(), part.getContentType());
                 foundDocument = true;
             }
+            if (part.getPartName().toString().equals("/word/theme/theme1.xml")) {
+                assertEquals(XWPFRelation.THEME.getContentType(), part.getContentType());
+                foundTheme1 = true;
+            }
         }
         assertTrue("Core not found in " + p.getParts(), foundCoreProps);
         assertTrue("Document not found in " + p.getParts(), foundDocument);
+        assertTrue("Theme1 not found in " + p.getParts(), foundTheme1);
     }
 }
