@@ -16,14 +16,17 @@
 ==================================================================== */
 package org.apache.poi.xssf.usermodel;
 
+import static org.junit.Assert.assertArrayEquals;
+
 import java.awt.Color;
-import java.util.Arrays;
+import java.io.IOException;
 import java.util.List;
 
 import junit.framework.TestCase;
 
 import org.apache.poi.POIXMLDocumentPart;
 import org.apache.poi.openxml4j.opc.OPCPackage;
+import org.apache.poi.ss.usermodel.ClientAnchor;
 import org.apache.poi.ss.usermodel.FontUnderline;
 import org.apache.poi.xssf.XSSFTestDataSamples;
 import org.openxmlformats.schemas.drawingml.x2006.main.CTTextCharacterProperties;
@@ -35,7 +38,7 @@ import org.openxmlformats.schemas.drawingml.x2006.spreadsheetDrawing.CTDrawing;
  * @author Yegor Kozlov
  */
 public class TestXSSFDrawing extends TestCase {
-    public void testRead(){
+    public void testRead() throws IOException{
         XSSFWorkbook wb = XSSFTestDataSamples.openSampleWorkbook("WithDrawing.xlsx");
         XSSFSheet sheet = wb.getSheetAt(0);
         //the sheet has one relationship and it is XSSFDrawing
@@ -141,7 +144,7 @@ public class TestXSSFDrawing extends TestCase {
         assertNotNull(XSSFTestDataSamples.writeOutAndReadBack(wb));
     }
     
-    public void testMultipleDrawings(){
+    public void testMultipleDrawings() throws IOException{
         XSSFWorkbook wb = new XSSFWorkbook();
         for (int i = 0; i < 3; i++) {
             XSSFSheet sheet = wb.createSheet();
@@ -149,9 +152,13 @@ public class TestXSSFDrawing extends TestCase {
             assertNotNull(drawing);
         }
         OPCPackage pkg = wb.getPackage();
-        assertEquals(3, pkg.getPartsByContentType(XSSFRelation.DRAWINGS.getContentType()).size());
+        try {
+            assertEquals(3, pkg.getPartsByContentType(XSSFRelation.DRAWINGS.getContentType()).size());
         
-        assertNotNull(XSSFTestDataSamples.writeOutAndReadBack(wb));
+            assertNotNull(XSSFTestDataSamples.writeOutAndReadBack(wb));
+        } finally {
+            pkg.close();
+        }
     }
 
     public void testClone() throws Exception{
@@ -218,9 +225,9 @@ public class TestXSSFDrawing extends TestCase {
         assertEquals(true, rPr.getB());
         assertEquals(true, rPr.getI());
         assertEquals(STTextUnderlineType.SNG, rPr.getU());
-        assertTrue(Arrays.equals(
+        assertArrayEquals(
                 new byte[]{0, (byte)128, (byte)128} ,
-                rPr.getSolidFill().getSrgbClr().getVal()));
+                rPr.getSolidFill().getSrgbClr().getVal());
         
         assertNotNull(XSSFTestDataSamples.writeOutAndReadBack(wb));
     }
@@ -283,9 +290,9 @@ public class TestXSSFDrawing extends TestCase {
 
         CTTextCharacterProperties rPr = pr.getRArray(0).getRPr();
         assertEquals("Arial", rPr.getLatin().getTypeface());
-        assertTrue(Arrays.equals(
+        assertArrayEquals(
                 new byte[]{0, (byte)128, (byte)128} ,
-                rPr.getSolidFill().getSrgbClr().getVal()));
+                rPr.getSolidFill().getSrgbClr().getVal());
         
         assertNotNull(XSSFTestDataSamples.writeOutAndReadBack(wb));
     }
@@ -317,9 +324,9 @@ public class TestXSSFDrawing extends TestCase {
         assertEquals("Arial", runs.get(0).getFontFamily());
         
         Color clr = runs.get(0).getFontColor(); 
-        assertTrue(Arrays.equals(
+        assertArrayEquals(
                 new int[] { 0, 255, 255 } ,
-                new int[] { clr.getRed(), clr.getGreen(), clr.getBlue() }));
+                new int[] { clr.getRed(), clr.getGreen(), clr.getBlue() });
         
         assertNotNull(XSSFTestDataSamples.writeOutAndReadBack(wb));
     }
@@ -393,26 +400,26 @@ public class TestXSSFDrawing extends TestCase {
         assertEquals("Arial", runs.get(0).getFontFamily());
 
         Color clr = runs.get(0).getFontColor(); 
-        assertTrue(Arrays.equals(
+        assertArrayEquals(
                 new int[] { 0, 255, 255 } ,
-                new int[] { clr.getRed(), clr.getGreen(), clr.getBlue() }));
+                new int[] { clr.getRed(), clr.getGreen(), clr.getBlue() });
 
         // second run properties        
         assertEquals("Rich Text", runs.get(1).getText());
         assertEquals(XSSFFont.DEFAULT_FONT_NAME, runs.get(1).getFontFamily());
 
         clr = runs.get(1).getFontColor(); 
-        assertTrue(Arrays.equals(
+        assertArrayEquals(
                 new int[] { 0, 255, 0 } ,
-                new int[] { clr.getRed(), clr.getGreen(), clr.getBlue() }));        
+                new int[] { clr.getRed(), clr.getGreen(), clr.getBlue() });        
         
         // third run properties
         assertEquals(" String", runs.get(2).getText());
         assertEquals("Arial", runs.get(2).getFontFamily());
         clr = runs.get(2).getFontColor(); 
-        assertTrue(Arrays.equals(
+        assertArrayEquals(
                 new int[] { 0, 255, 255 } ,
-                new int[] { clr.getRed(), clr.getGreen(), clr.getBlue() }));
+                new int[] { clr.getRed(), clr.getGreen(), clr.getBlue() });
         
         assertNotNull(XSSFTestDataSamples.writeOutAndReadBack(wb));
     }    
@@ -542,19 +549,19 @@ public class TestXSSFDrawing extends TestCase {
         assertEquals(TextAlign.RIGHT, paras.get(2).getTextAlign());
         
         Color clr = paras.get(0).getTextRuns().get(0).getFontColor(); 
-        assertTrue(Arrays.equals(
+        assertArrayEquals(
                 new int[] { 255, 0, 0 } ,
-                new int[] { clr.getRed(), clr.getGreen(), clr.getBlue() }));
+                new int[] { clr.getRed(), clr.getGreen(), clr.getBlue() });
         
         clr = paras.get(1).getTextRuns().get(0).getFontColor(); 
-        assertTrue(Arrays.equals(
+        assertArrayEquals(
                 new int[] { 0, 255, 0 } ,
-                new int[] { clr.getRed(), clr.getGreen(), clr.getBlue() }));
+                new int[] { clr.getRed(), clr.getGreen(), clr.getBlue() });
         
         clr = paras.get(2).getTextRuns().get(0).getFontColor(); 
-        assertTrue(Arrays.equals(
+        assertArrayEquals(
                 new int[] { 0, 0, 255 } ,
-                new int[] { clr.getRed(), clr.getGreen(), clr.getBlue() }));
+                new int[] { clr.getRed(), clr.getGreen(), clr.getBlue() });
         
         assertNotNull(XSSFTestDataSamples.writeOutAndReadBack(wb));
     }
@@ -693,4 +700,46 @@ public class TestXSSFDrawing extends TestCase {
         
         assertNotNull(XSSFTestDataSamples.writeOutAndReadBack(wb));
     }
+
+    public void testXSSFSimpleShapeCausesNPE56514() throws Exception {
+        XSSFWorkbook wb = XSSFTestDataSamples.openSampleWorkbook("56514.xlsx");
+        XSSFSheet sheet = wb.getSheetAt(0);
+        XSSFDrawing drawing = sheet.createDrawingPatriarch();
+        List<XSSFShape> shapes = drawing.getShapes();
+        assertEquals(4, shapes.size());
+        
+        wb = XSSFTestDataSamples.writeOutAndReadBack(wb);
+        
+        shapes = drawing.getShapes();
+        assertEquals(4, shapes.size());
+        
+/*        OutputStream stream = new FileOutputStream(new File("C:\\temp\\56514.xlsx"));
+        try {
+            wb.write(stream);
+        } finally {
+            stream.close();
+        }*/
+    }
+
+    public void testBug56835CellComment() throws Exception {
+        XSSFWorkbook wb = new XSSFWorkbook();
+        try {
+            XSSFSheet sheet = wb.createSheet();
+            XSSFDrawing drawing = sheet.createDrawingPatriarch();
+            
+            // first comment works
+            ClientAnchor anchor = new XSSFClientAnchor(1, 1, 2, 2, 3, 3, 4, 4);
+            XSSFComment comment = drawing.createCellComment(anchor);
+            assertNotNull(comment);
+            
+            try {
+                drawing.createCellComment(anchor);
+                fail("Should fail if we try to add the same comment for the same cell");
+            } catch (IllegalArgumentException e) {
+                // expected
+            }
+        } finally {
+            wb.close();
+        }
+    }
 }
