@@ -22,6 +22,11 @@ import java.util.TreeMap;
 
 import junit.framework.TestCase;
 
+import org.apache.poi.ss.usermodel.Row;
+import org.apache.poi.ss.usermodel.Sheet;
+import org.apache.poi.xssf.XSSFTestDataSamples;
+import org.apache.poi.xssf.model.StylesTable;
+import org.junit.Test;
 import org.openxmlformats.schemas.spreadsheetml.x2006.main.CTRPrElt;
 import org.openxmlformats.schemas.spreadsheetml.x2006.main.CTRst;
 import org.openxmlformats.schemas.spreadsheetml.x2006.main.STXstring;
@@ -34,7 +39,6 @@ import org.openxmlformats.schemas.spreadsheetml.x2006.main.STXstring;
 public final class TestXSSFRichTextString extends TestCase {
 
     public void testCreate() {
-
         XSSFRichTextString rt = new XSSFRichTextString("Apache POI");
         assertEquals("Apache POI", rt.getString());
 
@@ -49,9 +53,14 @@ public final class TestXSSFRichTextString extends TestCase {
         assertEquals("Apache POI is cool stuff", rt.getString());
     }
 
+    public void testEmpty() {
+        XSSFRichTextString rt = new XSSFRichTextString();
+        assertEquals(0, rt.getIndexOfFormattingRun(9999));
+        assertEquals(-1, rt.getLengthOfFormattingRun(9999));
+        assertNull(rt.getFontAtIndex(9999));
+    }
 
     public void testApplyFont() {
-
         XSSFRichTextString rt = new XSSFRichTextString();
         rt.append("123");
         rt.append("4567");
@@ -78,6 +87,65 @@ public final class TestXSSFRichTextString extends TestCase {
         assertEquals(7, rt.getIndexOfFormattingRun(3));
         assertEquals(2, rt.getLengthOfFormattingRun(3));
         assertEquals("89", rt.getCTRst().getRArray(3).getT());
+        
+        
+        assertEquals(-1, rt.getIndexOfFormattingRun(9999));
+        assertEquals(-1, rt.getLengthOfFormattingRun(9999));
+        assertNull(rt.getFontAtIndex(9999));
+    }
+
+    public void testApplyFontIndex() {
+        XSSFRichTextString rt = new XSSFRichTextString("Apache POI");
+        rt.applyFont(0, 10, (short)1);
+        
+        rt.applyFont((short)1);
+        
+        assertNotNull(rt.getFontAtIndex(0));
+    }
+
+    public void testApplyFontWithStyles() {
+        XSSFRichTextString rt = new XSSFRichTextString("Apache POI");
+        
+        StylesTable tbl = new StylesTable();
+        rt.setStylesTableReference(tbl);
+        
+        try {
+            rt.applyFont(0, 10, (short)1);
+            fail("Fails without styles in the table");
+        } catch (IndexOutOfBoundsException e) {
+            // expected
+        }
+        
+        tbl.putFont(new XSSFFont());
+        rt.applyFont(0, 10, (short)1);
+        rt.applyFont((short)1);
+    }
+
+    public void testApplyFontException() {
+        XSSFRichTextString rt = new XSSFRichTextString("Apache POI");
+        
+        rt.applyFont(0, 0, (short)1);
+
+        try {
+            rt.applyFont(11, 10, (short)1);
+            fail("Should catch Exception here");
+        } catch (IllegalArgumentException e) {
+            assertTrue(e.getMessage().contains("11"));
+        }
+
+        try {
+            rt.applyFont(-1, 10, (short)1);
+            fail("Should catch Exception here");
+        } catch (IllegalArgumentException e) {
+            assertTrue(e.getMessage().contains("-1"));
+        }
+
+        try {
+            rt.applyFont(0, 555, (short)1);
+            fail("Should catch Exception here");
+        } catch (IllegalArgumentException e) {
+            assertTrue(e.getMessage().contains("555"));
+        }
     }
 
     public void testClearFormatting() {
@@ -308,22 +376,22 @@ public final class TestXSSFRichTextString extends TestCase {
 
         str = new XSSFRichTextString("Incorrect\nLine-Breaking");
         str.applyFont(0, 8, font);
-        t1 = str.getCTRst().getRList().get(0).xgetT();
-        t2 = str.getCTRst().getRList().get(1).xgetT();
+        t1 = str.getCTRst().getRArray(0).xgetT();
+        t2 = str.getCTRst().getRArray(1).xgetT();
         assertEquals("<xml-fragment>Incorrec</xml-fragment>", t1.xmlText());
         assertEquals("<xml-fragment>t\nLine-Breaking</xml-fragment>", t2.xmlText());
 
         str = new XSSFRichTextString("Incorrect\nLine-Breaking");
         str.applyFont(0, 9, font);
-        t1 = str.getCTRst().getRList().get(0).xgetT();
-        t2 = str.getCTRst().getRList().get(1).xgetT();
+        t1 = str.getCTRst().getRArray(0).xgetT();
+        t2 = str.getCTRst().getRArray(1).xgetT();
         assertEquals("<xml-fragment>Incorrect</xml-fragment>", t1.xmlText());
         assertEquals("<xml-fragment xml:space=\"preserve\">\nLine-Breaking</xml-fragment>", t2.xmlText());
 
         str = new XSSFRichTextString("Incorrect\n Line-Breaking");
         str.applyFont(0, 9, font);
-        t1 = str.getCTRst().getRList().get(0).xgetT();
-        t2 = str.getCTRst().getRList().get(1).xgetT();
+        t1 = str.getCTRst().getRArray(0).xgetT();
+        t2 = str.getCTRst().getRArray(1).xgetT();
         assertEquals("<xml-fragment>Incorrect</xml-fragment>", t1.xmlText());
         assertEquals("<xml-fragment xml:space=\"preserve\">\n Line-Breaking</xml-fragment>", t2.xmlText());
 
@@ -333,15 +401,15 @@ public final class TestXSSFRichTextString extends TestCase {
         assertEquals("<xml-fragment xml:space=\"preserve\">Tab\tseparated\n</xml-fragment>", t1.xmlText());
 
         str.applyFont(0, 3, font);
-        t1 = str.getCTRst().getRList().get(0).xgetT();
-        t2 = str.getCTRst().getRList().get(1).xgetT();
+        t1 = str.getCTRst().getRArray(0).xgetT();
+        t2 = str.getCTRst().getRArray(1).xgetT();
         assertEquals("<xml-fragment>Tab</xml-fragment>", t1.xmlText());
         assertEquals("<xml-fragment xml:space=\"preserve\">\tseparated\n</xml-fragment>", t2.xmlText());
 
         str = new XSSFRichTextString("Tab\tseparated\n");
         str.applyFont(0, 4, font);
-        t1 = str.getCTRst().getRList().get(0).xgetT();
-        t2 = str.getCTRst().getRList().get(1).xgetT();
+        t1 = str.getCTRst().getRArray(0).xgetT();
+        t2 = str.getCTRst().getRArray(1).xgetT();
         // YK: don't know why, but XmlBeans converts leading tab characters to spaces
         //assertEquals("<xml-fragment>Tab\t</xml-fragment>", t1.xmlText());
         assertEquals("<xml-fragment xml:space=\"preserve\">separated\n</xml-fragment>", t2.xmlText());
@@ -349,12 +417,91 @@ public final class TestXSSFRichTextString extends TestCase {
         str = new XSSFRichTextString("\n\n\nNew Line\n\n");
         str.applyFont(0, 3, font);
         str.applyFont(11, 13, font);
-        t1 = str.getCTRst().getRList().get(0).xgetT();
-        t2 = str.getCTRst().getRList().get(1).xgetT();
-        t3 = str.getCTRst().getRList().get(2).xgetT();
+        t1 = str.getCTRst().getRArray(0).xgetT();
+        t2 = str.getCTRst().getRArray(1).xgetT();
+        t3 = str.getCTRst().getRArray(2).xgetT();
         // YK: don't know why, but XmlBeans converts leading tab characters to spaces
         assertEquals("<xml-fragment xml:space=\"preserve\">\n\n\n</xml-fragment>", t1.xmlText());
         assertEquals("<xml-fragment>New Line</xml-fragment>", t2.xmlText());
         assertEquals("<xml-fragment xml:space=\"preserve\">\n\n</xml-fragment>", t3.xmlText());
     }
+
+    
+    @Test
+    public void testBug56511() {
+        XSSFWorkbook wb = XSSFTestDataSamples.openSampleWorkbook("56511.xlsx");
+        for (XSSFSheet sheet : wb) {
+            int lastRow = sheet.getLastRowNum();
+            for (int rowIdx = sheet.getFirstRowNum(); rowIdx <= lastRow; rowIdx++) {
+                XSSFRow row = sheet.getRow(rowIdx);
+                if(row != null) {
+                    int lastCell = row.getLastCellNum();
+    
+                    for (int cellIdx = row.getFirstCellNum(); cellIdx <= lastCell; cellIdx++) {
+    
+                        XSSFCell cell = row.getCell(cellIdx);
+                        if (cell != null) {
+                            //System.out.println("row " + rowIdx + " column " + cellIdx + ": " + cell.getCellType() + ": " + cell.toString());
+                            
+                            XSSFRichTextString richText = cell.getRichStringCellValue();
+                            int anzFormattingRuns = richText.numFormattingRuns();
+                            for (int run = 0; run < anzFormattingRuns; run++) {
+                                /*XSSFFont font =*/ richText.getFontOfFormattingRun(run);
+                                //System.out.println("  run " + run
+                                //        + " font " + (font == null ? "<null>" : font.getFontName()));
+                            }
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    @Test
+    public void testBug56511_values() {
+        XSSFWorkbook wb = XSSFTestDataSamples.openSampleWorkbook("56511.xlsx");
+        Sheet sheet = wb.getSheetAt(0);
+        Row row = sheet.getRow(0);
+
+        // verify the values to ensure future changes keep the returned information equal 
+        XSSFRichTextString rt = (XSSFRichTextString) row.getCell(0).getRichStringCellValue();
+        assertEquals(0, rt.numFormattingRuns());
+        assertNull(rt.getFontOfFormattingRun(0));
+        assertEquals(-1, rt.getLengthOfFormattingRun(0));
+        
+        rt = (XSSFRichTextString) row.getCell(1).getRichStringCellValue();
+        assertEquals(0, row.getCell(1).getRichStringCellValue().numFormattingRuns());
+        assertNull(rt.getFontOfFormattingRun(1));
+        assertEquals(-1, rt.getLengthOfFormattingRun(1));
+               
+        rt = (XSSFRichTextString) row.getCell(2).getRichStringCellValue();
+        assertEquals(2, rt.numFormattingRuns());
+        assertNotNull(rt.getFontOfFormattingRun(0));
+        assertEquals(4, rt.getLengthOfFormattingRun(0));
+        
+        assertNotNull(rt.getFontOfFormattingRun(1));
+        assertEquals(9, rt.getLengthOfFormattingRun(1));
+
+        assertNull(rt.getFontOfFormattingRun(2));
+        
+        rt = (XSSFRichTextString) row.getCell(3).getRichStringCellValue();
+        assertEquals(3, rt.numFormattingRuns());
+        assertNull(rt.getFontOfFormattingRun(0));
+        assertEquals(1, rt.getLengthOfFormattingRun(0));
+        
+        assertNotNull(rt.getFontOfFormattingRun(1));
+        assertEquals(3, rt.getLengthOfFormattingRun(1));
+        
+        assertNotNull(rt.getFontOfFormattingRun(2));
+        assertEquals(9, rt.getLengthOfFormattingRun(2));
+    }
+
+    public void testToString() {
+        XSSFRichTextString rt = new XSSFRichTextString("Apache POI");
+        assertNotNull(rt.toString());
+        
+        // TODO: normally toString() should never return null, should we adjust this?
+        rt = new XSSFRichTextString();
+        assertNull(rt.toString());
+    }
 }
