@@ -17,61 +17,51 @@
 
 package org.apache.poi.xssf.usermodel;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.assertNotSame;
-import static org.junit.Assert.assertNull;
-import static org.junit.Assert.assertSame;
-import static org.junit.Assert.assertTrue;
-import static org.junit.Assert.fail;
+import static junit.framework.TestCase.assertNotNull;
+import static junit.framework.TestCase.assertTrue;
+import static org.apache.poi.xssf.XSSFTestDataSamples.openSampleWorkbook;
+import static org.apache.poi.xssf.XSSFTestDataSamples.writeOutAndReadBack;
+import static org.junit.Assert.*;
 
+import java.io.IOException;
+import java.util.Arrays;
+import java.util.HashSet;
 import java.util.List;
+import java.util.Set;
 
+import org.apache.poi.POIXMLException;
 import org.apache.poi.hssf.HSSFTestDataSamples;
-import org.apache.poi.hssf.record.PasswordRecord;
+import org.apache.poi.poifs.crypt.CryptoFunctions;
+import org.apache.poi.poifs.crypt.HashAlgorithm;
 import org.apache.poi.ss.usermodel.AutoFilter;
 import org.apache.poi.ss.usermodel.BaseTestSheet;
 import org.apache.poi.ss.usermodel.Cell;
-import org.apache.poi.ss.usermodel.CreationHelper;
-import org.apache.poi.ss.usermodel.RichTextString;
+import org.apache.poi.ss.usermodel.Row;
 import org.apache.poi.ss.usermodel.Sheet;
 import org.apache.poi.ss.usermodel.Workbook;
+import org.apache.poi.ss.util.AreaReference;
 import org.apache.poi.ss.util.CellRangeAddress;
-import org.apache.poi.util.HexDump;
-import org.apache.poi.xssf.SXSSFITestDataProvider;
+import org.apache.poi.ss.util.CellReference;
 import org.apache.poi.xssf.XSSFITestDataProvider;
 import org.apache.poi.xssf.XSSFTestDataSamples;
 import org.apache.poi.xssf.model.CalculationChain;
 import org.apache.poi.xssf.model.CommentsTable;
 import org.apache.poi.xssf.model.StylesTable;
-import org.apache.poi.xssf.streaming.SXSSFSheet;
 import org.apache.poi.xssf.streaming.SXSSFWorkbook;
 import org.apache.poi.xssf.usermodel.helpers.ColumnHelper;
 import org.junit.Test;
-import org.openxmlformats.schemas.spreadsheetml.x2006.main.CTCalcPr;
-import org.openxmlformats.schemas.spreadsheetml.x2006.main.CTCell;
-import org.openxmlformats.schemas.spreadsheetml.x2006.main.CTCol;
-import org.openxmlformats.schemas.spreadsheetml.x2006.main.CTCols;
-import org.openxmlformats.schemas.spreadsheetml.x2006.main.CTComments;
-import org.openxmlformats.schemas.spreadsheetml.x2006.main.CTRow;
-import org.openxmlformats.schemas.spreadsheetml.x2006.main.CTSheetData;
-import org.openxmlformats.schemas.spreadsheetml.x2006.main.CTSheetProtection;
-import org.openxmlformats.schemas.spreadsheetml.x2006.main.CTWorksheet;
-import org.openxmlformats.schemas.spreadsheetml.x2006.main.CTXf;
-import org.openxmlformats.schemas.spreadsheetml.x2006.main.STCalcMode;
-import org.openxmlformats.schemas.spreadsheetml.x2006.main.STPane;
+import org.openxmlformats.schemas.spreadsheetml.x2006.main.*;
 
 
+@SuppressWarnings("resource")
 public final class TestXSSFSheet extends BaseTestSheet {
 
-    private static final int ROW_COUNT = 40000;
-
     public TestXSSFSheet() {
         super(XSSFITestDataProvider.instance);
     }
 
     //TODO column styles are not yet supported by XSSF
+    @Override
     @Test
     public void defaultColumnStyle() {
         //super.defaultColumnStyle();
@@ -192,39 +182,6 @@ public final class TestXSSFSheet extends BaseTestSheet {
         assertTrue(col.getBestFit());
     }
 
-    /**
-     * XSSFSheet autoSizeColumn() on empty RichTextString fails
-     */
-    @Test
-    public void bug48325() {
-        XSSFWorkbook wb = new XSSFWorkbook();
-        XSSFSheet sheet = wb.createSheet("Test");
-        CreationHelper factory = wb.getCreationHelper();
-
-        XSSFRow row = sheet.createRow(0);
-        XSSFCell cell = row.createCell(0);
-
-        XSSFFont font = wb.createFont();
-        RichTextString rts = factory.createRichTextString("");
-        rts.applyFont(font);
-        cell.setCellValue(rts);
-
-        sheet.autoSizeColumn(0);
-    }
-
-    @Test
-    public void getCellComment() {
-        XSSFWorkbook workbook = new XSSFWorkbook();
-        XSSFSheet sheet = workbook.createSheet();
-        XSSFDrawing dg = sheet.createDrawingPatriarch();
-        XSSFComment comment = dg.createCellComment(new XSSFClientAnchor());
-        XSSFCell cell = sheet.createRow(9).createCell(2);
-        comment.setAuthor("test C10 author");
-        cell.setCellComment(comment);
-
-        assertNotNull(sheet.getCellComment(9, 2));
-        assertEquals("test C10 author", sheet.getCellComment(9, 2).getAuthor());
-    }
 
     @Test
     public void setCellComment() {
@@ -272,16 +229,6 @@ public final class TestXSSFSheet extends BaseTestSheet {
         assertEquals(STPane.BOTTOM_RIGHT, ctWorksheet.getSheetViews().getSheetViewArray(0).getPane().getActivePane());
     }
 
-    @Test
-    public void newMergedRegionAt() {
-        XSSFWorkbook workbook = new XSSFWorkbook();
-        XSSFSheet sheet = workbook.createSheet();
-        CellRangeAddress region = CellRangeAddress.valueOf("B2:D4");
-        sheet.addMergedRegion(region);
-        assertEquals("B2:D4", sheet.getMergedRegion(0).formatAsString());
-        assertEquals(1, sheet.getNumMergedRegions());
-    }
-
     @Test
     public void removeMergedRegion_lowlevel() {
         XSSFWorkbook workbook = new XSSFWorkbook();
@@ -290,6 +237,7 @@ public final class TestXSSFSheet extends BaseTestSheet {
         CellRangeAddress region_1 = CellRangeAddress.valueOf("A1:B2");
         CellRangeAddress region_2 = CellRangeAddress.valueOf("C3:D4");
         CellRangeAddress region_3 = CellRangeAddress.valueOf("E5:F6");
+        CellRangeAddress region_4 = CellRangeAddress.valueOf("G7:H8");
         sheet.addMergedRegion(region_1);
         sheet.addMergedRegion(region_2);
         sheet.addMergedRegion(region_3);
@@ -303,6 +251,17 @@ public final class TestXSSFSheet extends BaseTestSheet {
         assertEquals(0, sheet.getNumMergedRegions());
         assertNull(" CTMergeCells should be deleted after removing the last merged " +
                 "region on the sheet.", sheet.getCTWorksheet().getMergeCells());
+        sheet.addMergedRegion(region_1);
+        sheet.addMergedRegion(region_2);
+        sheet.addMergedRegion(region_3);
+        sheet.addMergedRegion(region_4);
+        // test invalid indexes OOBE
+        Set<Integer> rmIdx = new HashSet<Integer>(Arrays.asList(5,6));
+        sheet.removeMergedRegions(rmIdx);
+        rmIdx = new HashSet<Integer>(Arrays.asList(1,3));
+        sheet.removeMergedRegions(rmIdx);
+        assertEquals("A1:B2", ctWorksheet.getMergeCells().getMergeCellArray(0).getRef());
+        assertEquals("E5:F6", ctWorksheet.getMergeCells().getMergeCellArray(1).getRef());
     }
 
     @Test
@@ -332,6 +291,7 @@ public final class TestXSSFSheet extends BaseTestSheet {
 
 
     @Test
+    @SuppressWarnings("deprecation")
     public void groupUngroupColumn() {
         XSSFWorkbook workbook = new XSSFWorkbook();
         XSSFSheet sheet = workbook.createSheet();
@@ -341,36 +301,37 @@ public final class TestXSSFSheet extends BaseTestSheet {
         sheet.groupColumn(10, 11);
         CTCols cols = sheet.getCTWorksheet().getColsArray(0);
         assertEquals(2, cols.sizeOfColArray());
-        List<CTCol> colArray = cols.getColList();
+        CTCol[] colArray = cols.getColArray();
         assertNotNull(colArray);
-        assertEquals(2 + 1, colArray.get(0).getMin()); // 1 based
-        assertEquals(7 + 1, colArray.get(0).getMax()); // 1 based
-        assertEquals(1, colArray.get(0).getOutlineLevel());
+        assertEquals(2 + 1, colArray[0].getMin()); // 1 based
+        assertEquals(7 + 1, colArray[0].getMax()); // 1 based
+        assertEquals(1, colArray[0].getOutlineLevel());
+        assertEquals(0, sheet.getColumnOutlineLevel(0));
 
         //two level
         sheet.groupColumn(1, 2);
         cols = sheet.getCTWorksheet().getColsArray(0);
         assertEquals(4, cols.sizeOfColArray());
-        colArray = cols.getColList();
-        assertEquals(2, colArray.get(1).getOutlineLevel());
+        colArray = cols.getColArray();
+        assertEquals(2, colArray[1].getOutlineLevel());
 
         //three level
         sheet.groupColumn(6, 8);
         sheet.groupColumn(2, 3);
         cols = sheet.getCTWorksheet().getColsArray(0);
         assertEquals(7, cols.sizeOfColArray());
-        colArray = cols.getColList();
-        assertEquals(3, colArray.get(1).getOutlineLevel());
+        colArray = cols.getColArray();
+        assertEquals(3, colArray[1].getOutlineLevel());
         assertEquals(3, sheet.getCTWorksheet().getSheetFormatPr().getOutlineLevelCol());
 
         sheet.ungroupColumn(8, 10);
-        colArray = cols.getColList();
+        colArray = cols.getColArray();
         //assertEquals(3, colArray[1].getOutlineLevel());
 
         sheet.ungroupColumn(4, 6);
         sheet.ungroupColumn(2, 2);
-        colArray = cols.getColList();
-        assertEquals(4, colArray.size());
+        colArray = cols.getColArray();
+        assertEquals(4, colArray.length);
         assertEquals(2, sheet.getCTWorksheet().getSheetFormatPr().getOutlineLevelCol());
     }
 
@@ -755,9 +716,8 @@ public final class TestXSSFSheet extends BaseTestSheet {
         XSSFSheet xs = sheet;
         CTWorksheet cts = xs.getCTWorksheet();
 
-        List<CTCols> cols_s = cts.getColsList();
-        assertEquals(1, cols_s.size());
-        CTCols cols = cols_s.get(0);
+        assertEquals(1, cts.sizeOfColsArray());
+        CTCols cols = cts.getColsArray(0);
         assertEquals(1, cols.sizeOfColArray());
         CTCol col = cols.getColArray(0);
 
@@ -770,9 +730,8 @@ public final class TestXSSFSheet extends BaseTestSheet {
         // Now set another
         sheet.setColumnWidth(3, 33 * 256);
 
-        cols_s = cts.getColsList();
-        assertEquals(1, cols_s.size());
-        cols = cols_s.get(0);
+        assertEquals(1, cts.sizeOfColsArray());
+        cols = cts.getColsArray(0);
         assertEquals(2, cols.sizeOfColArray());
 
         col = cols.getColArray(0);
@@ -956,7 +915,9 @@ public final class TestXSSFSheet extends BaseTestSheet {
      * Rows and cells can be created in random order,
      * but CTRows are kept in ascending order
      */
+    @Override
     @Test
+    @SuppressWarnings("deprecation")
     public void createRow() {
         XSSFWorkbook workbook = new XSSFWorkbook();
         XSSFSheet sheet = workbook.createSheet();
@@ -980,27 +941,27 @@ public final class TestXSSFSheet extends BaseTestSheet {
         row3.createCell(5);
 
 
-        List<CTRow> xrow = sheetData.getRowList();
-        assertEquals(3, xrow.size());
+        CTRow[] xrow = sheetData.getRowArray();
+        assertEquals(3, xrow.length);
 
         //rows are sorted: {0, 1, 2}
-        assertEquals(4, xrow.get(0).sizeOfCArray());
-        assertEquals(1, xrow.get(0).getR());
-        assertTrue(xrow.get(0).equals(row3.getCTRow()));
+        assertEquals(4, xrow[0].sizeOfCArray());
+        assertEquals(1, xrow[0].getR());
+        assertTrue(xrow[0].equals(row3.getCTRow()));
 
-        assertEquals(3, xrow.get(1).sizeOfCArray());
-        assertEquals(2, xrow.get(1).getR());
-        assertTrue(xrow.get(1).equals(row2.getCTRow()));
+        assertEquals(3, xrow[1].sizeOfCArray());
+        assertEquals(2, xrow[1].getR());
+        assertTrue(xrow[1].equals(row2.getCTRow()));
 
-        assertEquals(2, xrow.get(2).sizeOfCArray());
-        assertEquals(3, xrow.get(2).getR());
-        assertTrue(xrow.get(2).equals(row1.getCTRow()));
+        assertEquals(2, xrow[2].sizeOfCArray());
+        assertEquals(3, xrow[2].getR());
+        assertTrue(xrow[2].equals(row1.getCTRow()));
 
-        List<CTCell> xcell = xrow.get(0).getCList();
-        assertEquals("D1", xcell.get(0).getR());
-        assertEquals("A1", xcell.get(1).getR());
-        assertEquals("C1", xcell.get(2).getR());
-        assertEquals("F1", xcell.get(3).getR());
+        CTCell[] xcell = xrow[0].getCArray();
+        assertEquals("D1", xcell[0].getR());
+        assertEquals("A1", xcell[1].getR());
+        assertEquals("C1", xcell[2].getR());
+        assertEquals("F1", xcell[3].getR());
 
         //re-creating a row does NOT add extra data to the parent
         row2 = sheet.createRow(1);
@@ -1012,25 +973,25 @@ public final class TestXSSFSheet extends BaseTestSheet {
         workbook = XSSFTestDataSamples.writeOutAndReadBack(workbook);
         sheet = workbook.getSheetAt(0);
         wsh = sheet.getCTWorksheet();
-        xrow = sheetData.getRowList();
-        assertEquals(3, xrow.size());
+        xrow = sheetData.getRowArray();
+        assertEquals(3, xrow.length);
 
         //rows are sorted: {0, 1, 2}
-        assertEquals(4, xrow.get(0).sizeOfCArray());
-        assertEquals(1, xrow.get(0).getR());
+        assertEquals(4, xrow[0].sizeOfCArray());
+        assertEquals(1, xrow[0].getR());
         //cells are now sorted
-        xcell = xrow.get(0).getCList();
-        assertEquals("A1", xcell.get(0).getR());
-        assertEquals("C1", xcell.get(1).getR());
-        assertEquals("D1", xcell.get(2).getR());
-        assertEquals("F1", xcell.get(3).getR());
+        xcell = xrow[0].getCArray();
+        assertEquals("A1", xcell[0].getR());
+        assertEquals("C1", xcell[1].getR());
+        assertEquals("D1", xcell[2].getR());
+        assertEquals("F1", xcell[3].getR());
 
 
-        assertEquals(0, xrow.get(1).sizeOfCArray());
-        assertEquals(2, xrow.get(1).getR());
+        assertEquals(0, xrow[1].sizeOfCArray());
+        assertEquals(2, xrow[1].getR());
 
-        assertEquals(2, xrow.get(2).sizeOfCArray());
-        assertEquals(3, xrow.get(2).getR());
+        assertEquals(2, xrow[2].sizeOfCArray());
+        assertEquals(3, xrow[2].getR());
 
     }
 
@@ -1065,13 +1026,27 @@ public final class TestXSSFSheet extends BaseTestSheet {
         assertTrue("sheet protection should be on", pr.isSetSheet());
         assertTrue("object protection should be on", pr.isSetObjects());
         assertTrue("scenario protection should be on", pr.isSetScenarios());
-        String hash = String.valueOf(HexDump.shortToHex(PasswordRecord.hashPassword(password))).substring(2);
-        assertEquals("well known value for top secret hash should be "+ hash, hash, pr.xgetPassword().getStringValue());
+        int hashVal = CryptoFunctions.createXorVerifier1(password);
+        int actualVal = Integer.parseInt(pr.xgetPassword().getStringValue(),16);
+        assertEquals("well known value for top secret hash should match", hashVal, actualVal);
 
         sheet.protectSheet(null);
         assertNull("protectSheet(null) should unset CTSheetProtection", sheet.getCTWorksheet().getSheetProtection());
     }
 
+    @Test
+    public void protectSheet_lowlevel_2013() {
+        String password = "test";
+        XSSFWorkbook wb = new XSSFWorkbook();
+        XSSFSheet xs = wb.createSheet();
+        xs.setSheetPassword(password, HashAlgorithm.sha384);
+        wb = writeOutAndReadBack(wb);
+        assertTrue(wb.getSheetAt(0).validateSheetPassword(password));
+        
+        wb = openSampleWorkbook("workbookProtection-sheet_password-2013.xlsx");
+        assertTrue(wb.getSheetAt(0).validateSheetPassword("pwd"));
+    }
+    
 
     @Test
     public void bug49966() {
@@ -1129,6 +1104,8 @@ public final class TestXSSFSheet extends BaseTestSheet {
         XSSFWorkbook workbook = new XSSFWorkbook();
         XSSFSheet sheet = workbook.createSheet("Sheet 1");
 
+        assertFalse(sheet.getForceFormulaRecalculation());
+        
         // Set
         sheet.setForceFormulaRecalculation(true);
         assertEquals(true, sheet.getForceFormulaRecalculation());
@@ -1151,7 +1128,7 @@ public final class TestXSSFSheet extends BaseTestSheet {
     }
 
     @Test
-    public void bug54607() {
+    public void bug54607() throws IOException {
         // run with the file provided in the Bug-Report
         runGetTopRow("54607.xlsx", true, 1, 0, 0);
         runGetLeftCol("54607.xlsx", true, 0, 0, 0);
@@ -1165,7 +1142,7 @@ public final class TestXSSFSheet extends BaseTestSheet {
         runGetLeftCol("TwoSheetsNoneHidden.xls", false, 0, 0);
     }
 
-    private void runGetTopRow(String file, boolean isXSSF, int... topRows) {
+    private void runGetTopRow(String file, boolean isXSSF, int... topRows) throws IOException {
         final Workbook wb;
         if(isXSSF) {
             wb = XSSFTestDataSamples.openSampleWorkbook(file);
@@ -1181,15 +1158,19 @@ public final class TestXSSFSheet extends BaseTestSheet {
         // for XSSF also test with SXSSF
         if(isXSSF) {
             Workbook swb = new SXSSFWorkbook((XSSFWorkbook) wb);
-            for (int si = 0; si < swb.getNumberOfSheets(); si++) {
-                Sheet sh = swb.getSheetAt(si);
-                assertNotNull(sh.getSheetName());
-                assertEquals("Did not match for sheet " + si, topRows[si], sh.getTopRow());
+            try {
+                for (int si = 0; si < swb.getNumberOfSheets(); si++) {
+                    Sheet sh = swb.getSheetAt(si);
+                    assertNotNull(sh.getSheetName());
+                    assertEquals("Did not match for sheet " + si, topRows[si], sh.getTopRow());
+                }
+            } finally {
+                swb.close();
             }
         }
     }
 
-    private void runGetLeftCol(String file, boolean isXSSF, int... topRows) {
+    private void runGetLeftCol(String file, boolean isXSSF, int... topRows) throws IOException {
         final Workbook wb;
         if(isXSSF) {
             wb = XSSFTestDataSamples.openSampleWorkbook(file);
@@ -1210,56 +1191,10 @@ public final class TestXSSFSheet extends BaseTestSheet {
                 assertNotNull(sh.getSheetName());
                 assertEquals("Did not match for sheet " + si, topRows[si], sh.getLeftCol());
             }
+            swb.close();
         }
     }
 
-    @Test
-    public void showInPaneManyRowsBug55248() {
-        XSSFWorkbook workbook = new XSSFWorkbook();
-        XSSFSheet sheet = workbook.createSheet("Sheet 1");
-
-        sheet.showInPane(0, 0);
-
-        for(int i = ROW_COUNT/2;i < ROW_COUNT;i++) {
-            sheet.createRow(i);
-            sheet.showInPane(i, 0);
-            // this one fails: sheet.showInPane((short)i, 0);
-        }
-
-        int i = 0;
-        sheet.showInPane(i, i);
-
-        XSSFWorkbook wb = XSSFTestDataSamples.writeOutAndReadBack(workbook);
-        checkRowCount(wb);
-    }
-
-    @Test
-    public void showInPaneManyRowsBug55248SXSSF() {
-        SXSSFWorkbook workbook = new SXSSFWorkbook(new XSSFWorkbook());
-        SXSSFSheet sheet = (SXSSFSheet) workbook.createSheet("Sheet 1");
-
-        sheet.showInPane(0, 0);
-
-        for(int i = ROW_COUNT/2;i < ROW_COUNT;i++) {
-            sheet.createRow(i);
-            sheet.showInPane(i, 0);
-            // this one fails: sheet.showInPane((short)i, 0);
-        }
-
-        int i = 0;
-        sheet.showInPane(i, i);
-
-        Workbook wb = SXSSFITestDataProvider.instance.writeOutAndReadBack(workbook);
-        checkRowCount(wb);
-    }
-
-    private void checkRowCount(Workbook wb) {
-        assertNotNull(wb);
-        final Sheet sh = wb.getSheet("Sheet 1");
-        assertNotNull(sh);
-        assertEquals(ROW_COUNT-1, sh.getLastRowNum());
-    }
-
     @Test
     public void bug55745() throws Exception {
         XSSFWorkbook wb = XSSFTestDataSamples.openSampleWorkbook("55745.xlsx");
@@ -1311,4 +1246,134 @@ public final class TestXSSFSheet extends BaseTestSheet {
         XSSFTestDataSamples.openSampleWorkbook("51585.xlsx");
     }
 
+    private XSSFWorkbook setupSheet(){
+        //set up workbook
+        XSSFWorkbook wb = new XSSFWorkbook();
+        XSSFSheet sheet = wb.createSheet();
+
+        Row row1 = sheet.createRow((short) 0);
+        Cell cell = row1.createCell((short) 0);
+        cell.setCellValue("Names");
+        Cell cell2 = row1.createCell((short) 1);
+        cell2.setCellValue("#");
+
+        Row row2 = sheet.createRow((short) 1);
+        Cell cell3 = row2.createCell((short) 0);
+        cell3.setCellValue("Jane");
+        Cell cell4 = row2.createCell((short) 1);
+        cell4.setCellValue(3);
+
+        Row row3 = sheet.createRow((short) 2);
+        Cell cell5 = row3.createCell((short) 0);
+        cell5.setCellValue("John");
+        Cell cell6 = row3.createCell((short) 1);
+        cell6.setCellValue(3);
+
+        return wb;
+    }
+
+    @Test
+    public void testCreateTwoPivotTablesInOneSheet(){
+        XSSFWorkbook wb = setupSheet();
+        XSSFSheet sheet = wb.getSheetAt(0);
+
+        assertNotNull(wb);
+        assertNotNull(sheet);
+        XSSFPivotTable pivotTable = sheet.createPivotTable(new AreaReference("A1:B2"), new CellReference("H5"));
+        assertNotNull(pivotTable);
+        assertTrue(wb.getPivotTables().size() > 0);
+        XSSFPivotTable pivotTable2 = sheet.createPivotTable(new AreaReference("A1:B2"), new CellReference("L5"), sheet);
+        assertNotNull(pivotTable2);
+        assertTrue(wb.getPivotTables().size() > 1);
+    }
+
+    @Test
+    public void testCreateTwoPivotTablesInTwoSheets(){
+        XSSFWorkbook wb = setupSheet();
+        XSSFSheet sheet = wb.getSheetAt(0);
+
+        assertNotNull(wb);
+        assertNotNull(sheet);
+        XSSFPivotTable pivotTable = sheet.createPivotTable(new AreaReference("A1:B2"), new CellReference("H5"));
+        assertNotNull(pivotTable);
+        assertTrue(wb.getPivotTables().size() > 0);
+        assertNotNull(wb);
+        XSSFSheet sheet2 = wb.createSheet();
+        XSSFPivotTable pivotTable2 = sheet2.createPivotTable(new AreaReference("A1:B2"), new CellReference("H5"), sheet);
+        assertNotNull(pivotTable2);
+        assertTrue(wb.getPivotTables().size() > 1);
+    }
+
+    @Test
+    public void testCreatePivotTable(){
+        XSSFWorkbook wb = setupSheet();
+        XSSFSheet sheet = wb.getSheetAt(0);
+
+        assertNotNull(wb);
+        assertNotNull(sheet);
+        XSSFPivotTable pivotTable = sheet.createPivotTable(new AreaReference("A1:B2"), new CellReference("H5"));
+        assertNotNull(pivotTable);
+        assertTrue(wb.getPivotTables().size() > 0);
+    }
+
+    @Test
+    public void testCreatePivotTableInOtherSheetThanDataSheet(){
+        XSSFWorkbook wb = setupSheet();
+        XSSFSheet sheet1 = wb.getSheetAt(0);
+        XSSFSheet sheet2 = wb.createSheet();
+
+        XSSFPivotTable pivotTable = sheet2.createPivotTable
+                (new AreaReference("A1:B2"), new CellReference("H5"), sheet1);
+        assertEquals(0, pivotTable.getRowLabelColumns().size());
+        
+        assertEquals(1, wb.getPivotTables().size());
+        assertEquals(0, sheet1.getPivotTables().size());
+        assertEquals(1, sheet2.getPivotTables().size());
+    }
+
+    @Test
+    public void testCreatePivotTableInOtherSheetThanDataSheetUsingAreaReference(){
+        XSSFWorkbook wb = setupSheet();
+        XSSFSheet sheet = wb.getSheetAt(0);
+        XSSFSheet sheet2 = wb.createSheet();
+
+        XSSFPivotTable pivotTable = sheet2.createPivotTable
+                (new AreaReference(sheet.getSheetName()+"!A$1:B$2"), new CellReference("H5"));
+        assertEquals(0, pivotTable.getRowLabelColumns().size());
+    }
+
+    @Test
+    public void testCreatePivotTableWithConflictingDataSheets(){
+        XSSFWorkbook wb = setupSheet();
+        XSSFSheet sheet = wb.getSheetAt(0);
+        XSSFSheet sheet2 = wb.createSheet();
+
+        try {
+            sheet2.createPivotTable(new AreaReference(sheet.getSheetName()+"!A$1:B$2"), new CellReference("H5"), sheet2);
+        } catch(IllegalArgumentException e) {
+            return;
+        }
+        fail();
+    }
+    
+    @Test
+    public void testReadFails() {
+        XSSFWorkbook wb = new XSSFWorkbook();
+        XSSFSheet sheet = wb.createSheet();
+        
+        try {
+            sheet.onDocumentRead();
+            fail("Throws exception because we cannot read here");
+        } catch (POIXMLException e) {
+            // expected here
+        }
+    }
+    
+    @SuppressWarnings("deprecation")
+    @Test
+    public void testCreateComment() {
+        XSSFWorkbook wb = new XSSFWorkbook();
+        XSSFSheet sheet = wb.createSheet();
+        assertNotNull(sheet.createComment());
+    }
 }
\ No newline at end of file
