@@ -17,6 +17,14 @@
 
 package org.apache.poi.xssf.usermodel;
 
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertSame;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
+
 import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FileOutputStream;
@@ -34,7 +42,9 @@ import org.apache.poi.openxml4j.opc.PackagePartName;
 import org.apache.poi.openxml4j.opc.PackagingURIHelper;
 import org.apache.poi.openxml4j.opc.internal.MemoryPackagePart;
 import org.apache.poi.openxml4j.opc.internal.PackagePropertiesPart;
+
 import org.apache.poi.ss.usermodel.BaseTestWorkbook;
+import org.apache.poi.ss.usermodel.Cell;
 import org.apache.poi.ss.usermodel.CellStyle;
 import org.apache.poi.ss.usermodel.Font;
 import org.apache.poi.ss.usermodel.IndexedColors;
@@ -42,12 +52,17 @@ import org.apache.poi.ss.usermodel.RichTextString;
 import org.apache.poi.ss.usermodel.Row;
 import org.apache.poi.ss.usermodel.Sheet;
 import org.apache.poi.ss.usermodel.Workbook;
+import org.apache.poi.ss.usermodel.WorkbookFactory;
+import org.apache.poi.ss.util.AreaReference;
+import org.apache.poi.ss.util.CellReference;
 import org.apache.poi.util.IOUtils;
 import org.apache.poi.util.TempFile;
 import org.apache.poi.xssf.XSSFITestDataProvider;
 import org.apache.poi.xssf.XSSFTestDataSamples;
 import org.apache.poi.xssf.model.StylesTable;
+import org.junit.Test;
 import org.openxmlformats.schemas.spreadsheetml.x2006.main.CTCalcPr;
+import org.openxmlformats.schemas.spreadsheetml.x2006.main.CTPivotCache;
 import org.openxmlformats.schemas.spreadsheetml.x2006.main.CTWorkbook;
 import org.openxmlformats.schemas.spreadsheetml.x2006.main.CTWorkbookPr;
 import org.openxmlformats.schemas.spreadsheetml.x2006.main.STCalcMode;
@@ -61,7 +76,8 @@ public final class TestXSSFWorkbook extends BaseTestWorkbook {
 	/**
 	 * Tests that we can save, and then re-load a new document
 	 */
-	public void testSaveLoadNew() throws Exception {
+	@Test
+	public void saveLoadNew() throws Exception {
 		XSSFWorkbook workbook = new XSSFWorkbook();
 
 		//check that the default date system is set to 1900
@@ -130,7 +146,8 @@ public final class TestXSSFWorkbook extends BaseTestWorkbook {
 		pkg.close();
 	}
 
-	public void testExisting() throws Exception {
+    @Test
+	public void existing() throws Exception {
 
 		XSSFWorkbook workbook = XSSFTestDataSamples.openSampleWorkbook("Formatting.xlsx");
 		assertNotNull(workbook.getSharedStringSource());
@@ -148,48 +165,64 @@ public final class TestXSSFWorkbook extends BaseTestWorkbook {
 		pkg.close();
 	}
 
-	public void testGetCellStyleAt(){
+    @Test
+	public void getCellStyleAt() throws IOException{
 	 	XSSFWorkbook workbook = new XSSFWorkbook();
-		short i = 0;
-		//get default style
-		CellStyle cellStyleAt = workbook.getCellStyleAt(i);
-		assertNotNull(cellStyleAt);
-
-		//get custom style
-		StylesTable styleSource = workbook.getStylesSource();
-		XSSFCellStyle customStyle = new XSSFCellStyle(styleSource);
-		XSSFFont font = new XSSFFont();
-		font.setFontName("Verdana");
-		customStyle.setFont(font);
-		int x = styleSource.putStyle(customStyle);
-		cellStyleAt = workbook.getCellStyleAt((short)x);
-		assertNotNull(cellStyleAt);
+	 	try {
+    		short i = 0;
+    		//get default style
+    		CellStyle cellStyleAt = workbook.getCellStyleAt(i);
+    		assertNotNull(cellStyleAt);
+    
+    		//get custom style
+    		StylesTable styleSource = workbook.getStylesSource();
+    		XSSFCellStyle customStyle = new XSSFCellStyle(styleSource);
+    		XSSFFont font = new XSSFFont();
+    		font.setFontName("Verdana");
+    		customStyle.setFont(font);
+    		int x = styleSource.putStyle(customStyle);
+    		cellStyleAt = workbook.getCellStyleAt((short)x);
+    		assertNotNull(cellStyleAt);
+	 	} finally {
+	 	    workbook.close();
+	 	}
 	}
 
-	public void testGetFontAt(){
+    @Test
+	public void getFontAt() throws IOException{
 	 	XSSFWorkbook workbook = new XSSFWorkbook();
-		StylesTable styleSource = workbook.getStylesSource();
-		short i = 0;
-		//get default font
-		Font fontAt = workbook.getFontAt(i);
-		assertNotNull(fontAt);
-
-		//get customized font
-		XSSFFont customFont = new XSSFFont();
-		customFont.setItalic(true);
-		int x = styleSource.putFont(customFont);
-		fontAt = workbook.getFontAt((short)x);
-		assertNotNull(fontAt);
+	 	try {
+    		StylesTable styleSource = workbook.getStylesSource();
+    		short i = 0;
+    		//get default font
+    		Font fontAt = workbook.getFontAt(i);
+    		assertNotNull(fontAt);
+    
+    		//get customized font
+    		XSSFFont customFont = new XSSFFont();
+    		customFont.setItalic(true);
+    		int x = styleSource.putFont(customFont);
+    		fontAt = workbook.getFontAt((short)x);
+    		assertNotNull(fontAt);
+	 	} finally {
+	 	    workbook.close();
+	 	}
 	}
 
-	public void testGetNumCellStyles(){
+    @Test
+	public void getNumCellStyles() throws IOException{
 	 	XSSFWorkbook workbook = new XSSFWorkbook();
-		short i = workbook.getNumCellStyles();
-		//get default cellStyles
-		assertEquals(1, i);
+	 	try {
+    		short i = workbook.getNumCellStyles();
+    		//get default cellStyles
+    		assertEquals(1, i);
+	 	} finally {
+	 	    workbook.close();
+	 	}
 	}
 
-	public void testLoadSave() {
+    @Test
+	public void loadSave() {
 		XSSFWorkbook workbook = XSSFTestDataSamples.openSampleWorkbook("Formatting.xlsx");
 		assertEquals(3, workbook.getNumberOfSheets());
 		assertEquals("dd/mm/yyyy", workbook.getSheetAt(0).getRow(1).getCell(0).getRichStringCellValue().getString());
@@ -212,7 +245,8 @@ public final class TestXSSFWorkbook extends BaseTestWorkbook {
 		assertNotNull(wb2.getStylesSource());
 	}
 
-	public void testStyles() {
+    @Test
+	public void styles() {
 		XSSFWorkbook workbook = XSSFTestDataSamples.openSampleWorkbook("Formatting.xlsx");
 
 		StylesTable ss = workbook.getStylesSource();
@@ -250,49 +284,65 @@ public final class TestXSSFWorkbook extends BaseTestWorkbook {
 		assertEquals(1, st.getBorders().size());
 	}
 
-	public void testIncrementSheetId() {
+    @Test
+	public void incrementSheetId() throws IOException {
 		XSSFWorkbook wb = new XSSFWorkbook();
-		int sheetId = (int)wb.createSheet().sheet.getSheetId();
-		assertEquals(1, sheetId);
-		sheetId = (int)wb.createSheet().sheet.getSheetId();
-		assertEquals(2, sheetId);
-
-		//test file with gaps in the sheetId sequence
-		wb = XSSFTestDataSamples.openSampleWorkbook("47089.xlsm");
-		int lastSheetId = (int)wb.getSheetAt(wb.getNumberOfSheets() - 1).sheet.getSheetId();
-		sheetId = (int)wb.createSheet().sheet.getSheetId();
-		assertEquals(lastSheetId+1, sheetId);
+		try {
+    		int sheetId = (int)wb.createSheet().sheet.getSheetId();
+    		assertEquals(1, sheetId);
+    		sheetId = (int)wb.createSheet().sheet.getSheetId();
+    		assertEquals(2, sheetId);
+    
+    		//test file with gaps in the sheetId sequence
+    		XSSFWorkbook wbBack = XSSFTestDataSamples.openSampleWorkbook("47089.xlsm");
+    		try {
+        		int lastSheetId = (int)wbBack.getSheetAt(wbBack.getNumberOfSheets() - 1).sheet.getSheetId();
+        		sheetId = (int)wbBack.createSheet().sheet.getSheetId();
+        		assertEquals(lastSheetId+1, sheetId);
+    		} finally {
+    		    wbBack.close();
+    		}
+		} finally {
+		    wb.close();
+		}
 	}
 
 	/**
 	 *  Test setting of core properties such as Title and Author
+	 * @throws IOException 
 	 */
-	public void testWorkbookProperties() {
+    @Test
+	public void workbookProperties() throws IOException {
 		XSSFWorkbook workbook = new XSSFWorkbook();
-		POIXMLProperties props = workbook.getProperties();
-		assertNotNull(props);
-		//the Application property must be set for new workbooks, see Bugzilla #47559
-		assertEquals("Apache POI", props.getExtendedProperties().getUnderlyingProperties().getApplication());
-
-		PackagePropertiesPart opcProps = props.getCoreProperties().getUnderlyingProperties();
-		assertNotNull(opcProps);
-
-		opcProps.setTitleProperty("Testing Bugzilla #47460");
-		assertEquals("Apache POI", opcProps.getCreatorProperty().getValue());
-		opcProps.setCreatorProperty("poi-dev@poi.apache.org");
-
-		workbook = XSSFTestDataSamples.writeOutAndReadBack(workbook);
-		assertEquals("Apache POI", workbook.getProperties().getExtendedProperties().getUnderlyingProperties().getApplication());
-		opcProps = workbook.getProperties().getCoreProperties().getUnderlyingProperties();
-		assertEquals("Testing Bugzilla #47460", opcProps.getTitleProperty().getValue());
-		assertEquals("poi-dev@poi.apache.org", opcProps.getCreatorProperty().getValue());
+		try {
+    		POIXMLProperties props = workbook.getProperties();
+    		assertNotNull(props);
+    		//the Application property must be set for new workbooks, see Bugzilla #47559
+    		assertEquals("Apache POI", props.getExtendedProperties().getUnderlyingProperties().getApplication());
+    
+    		PackagePropertiesPart opcProps = props.getCoreProperties().getUnderlyingProperties();
+    		assertNotNull(opcProps);
+    
+    		opcProps.setTitleProperty("Testing Bugzilla #47460");
+    		assertEquals("Apache POI", opcProps.getCreatorProperty().getValue());
+    		opcProps.setCreatorProperty("poi-dev@poi.apache.org");
+    
+    		XSSFWorkbook wbBack = XSSFTestDataSamples.writeOutAndReadBack(workbook);
+    		assertEquals("Apache POI", wbBack.getProperties().getExtendedProperties().getUnderlyingProperties().getApplication());
+    		opcProps = wbBack.getProperties().getCoreProperties().getUnderlyingProperties();
+    		assertEquals("Testing Bugzilla #47460", opcProps.getTitleProperty().getValue());
+    		assertEquals("poi-dev@poi.apache.org", opcProps.getCreatorProperty().getValue());
+		} finally {
+		    workbook.close();
+		}
 	}
 
 	/**
 	 * Verify that the attached test data was not modified. If this test method
 	 * fails, the test data is not working properly.
 	 */
-	public void testBug47668() throws Exception {
+    @Test
+	public void bug47668() throws Exception {
 		XSSFWorkbook workbook = XSSFTestDataSamples.openSampleWorkbook("47668.xlsx");
 		List<XSSFPictureData> allPictures = workbook.getAllPictures();
 		assertEquals(1, allPictures.size());
@@ -327,7 +377,8 @@ public final class TestXSSFWorkbook extends BaseTestWorkbook {
 	/**
 	 * When deleting a sheet make sure that we adjust sheet indices of named ranges
 	 */
-	public void testBug47737() {
+    @Test
+	public void bug47737() {
 		XSSFWorkbook wb = XSSFTestDataSamples.openSampleWorkbook("47737.xlsx");
 		assertEquals(2, wb.getNumberOfNames());
 		assertNotNull(wb.getCalculationChain());
@@ -353,7 +404,8 @@ public final class TestXSSFWorkbook extends BaseTestWorkbook {
 	/**
 	 * Problems with XSSFWorkbook.removeSheetAt when workbook contains charts
 	 */
-	public void testBug47813() {
+    @Test
+	public void bug47813() {
 		XSSFWorkbook wb = XSSFTestDataSamples.openSampleWorkbook("47813.xlsx");
 		assertEquals(3, wb.getNumberOfSheets());
 		assertNotNull(wb.getCalculationChain());
@@ -380,7 +432,8 @@ public final class TestXSSFWorkbook extends BaseTestWorkbook {
 	 * Problems with the count of the number of styles
 	 *  coming out wrong
 	 */
-	public void testBug49702() throws Exception {
+    @Test
+	public void bug49702() throws Exception {
 	    // First try with a new file
 	    XSSFWorkbook wb = new XSSFWorkbook();
 
@@ -427,47 +480,58 @@ public final class TestXSSFWorkbook extends BaseTestWorkbook {
 	    } catch(IndexOutOfBoundsException e) {}
 	}
 
-    public void testRecalcId() {
+    @Test
+    public void recalcId() throws IOException {
         XSSFWorkbook wb = new XSSFWorkbook();
-        assertFalse(wb.getForceFormulaRecalculation());
-        CTWorkbook ctWorkbook = wb.getCTWorkbook();
-        assertFalse(ctWorkbook.isSetCalcPr());
-
-        wb.setForceFormulaRecalculation(true); // resets the EngineId flag to zero
-
-        CTCalcPr calcPr = ctWorkbook.getCalcPr();
-        assertNotNull(calcPr);
-        assertEquals(0, (int) calcPr.getCalcId());
-
-        calcPr.setCalcId(100);
-        assertTrue(wb.getForceFormulaRecalculation());
-
-        wb.setForceFormulaRecalculation(true); // resets the EngineId flag to zero
-        assertEquals(0, (int) calcPr.getCalcId());
-        assertFalse(wb.getForceFormulaRecalculation());
-
-        // calcMode="manual" is unset when forceFormulaRecalculation=true
-        calcPr.setCalcMode(STCalcMode.MANUAL);
-        wb.setForceFormulaRecalculation(true);
-        assertEquals(STCalcMode.AUTO, calcPr.getCalcMode());
-
+        try {
+            assertFalse(wb.getForceFormulaRecalculation());
+            CTWorkbook ctWorkbook = wb.getCTWorkbook();
+            assertFalse(ctWorkbook.isSetCalcPr());
+    
+            wb.setForceFormulaRecalculation(true); // resets the EngineId flag to zero
+    
+            CTCalcPr calcPr = ctWorkbook.getCalcPr();
+            assertNotNull(calcPr);
+            assertEquals(0, (int) calcPr.getCalcId());
+    
+            calcPr.setCalcId(100);
+            assertTrue(wb.getForceFormulaRecalculation());
+    
+            wb.setForceFormulaRecalculation(true); // resets the EngineId flag to zero
+            assertEquals(0, (int) calcPr.getCalcId());
+            assertFalse(wb.getForceFormulaRecalculation());
+    
+            // calcMode="manual" is unset when forceFormulaRecalculation=true
+            calcPr.setCalcMode(STCalcMode.MANUAL);
+            wb.setForceFormulaRecalculation(true);
+            assertEquals(STCalcMode.AUTO, calcPr.getCalcMode());
+        } finally {
+            wb.close();
+        }
     }
 
-    public void testChangeSheetNameWithSharedFormulas() {
+    @Test
+    public void changeSheetNameWithSharedFormulas() {
         changeSheetNameWithSharedFormulas("shared_formulas.xlsx");
     }
 
-    public void testSetTabColor() {
+    @Test
+    public void setTabColor() throws IOException {
         XSSFWorkbook wb = new XSSFWorkbook();
-        XSSFSheet sh = wb.createSheet();
-        assertTrue(sh.getCTWorksheet().getSheetPr() == null || !sh.getCTWorksheet().getSheetPr().isSetTabColor());
-        sh.setTabColor(IndexedColors.RED.index);
-        assertTrue(sh.getCTWorksheet().getSheetPr().isSetTabColor());
-        assertEquals(IndexedColors.RED.index,
-                sh.getCTWorksheet().getSheetPr().getTabColor().getIndexed());
+        try {
+            XSSFSheet sh = wb.createSheet();
+            assertTrue(sh.getCTWorksheet().getSheetPr() == null || !sh.getCTWorksheet().getSheetPr().isSetTabColor());
+            sh.setTabColor(IndexedColors.RED.index);
+            assertTrue(sh.getCTWorksheet().getSheetPr().isSetTabColor());
+            assertEquals(IndexedColors.RED.index,
+                    sh.getCTWorksheet().getSheetPr().getTabColor().getIndexed());
+        } finally {
+            wb.close();
+        }
     }
 
-	public void testColumnWidthPOI52233() throws Exception {
+    @Test
+	public void columnWidthPOI52233() throws Exception {
 		XSSFWorkbook workbook = new XSSFWorkbook();
 		XSSFSheet sheet = workbook.createSheet();
 		XSSFRow row = sheet.createRow(0);
@@ -500,8 +564,8 @@ public final class TestXSSFWorkbook extends BaseTestWorkbook {
 		assertEquals(2048, workbook.getSheetAt(0).getColumnWidth(0)); // <-works
 	}
 
-
-	public void testBug48495() {
+    @Test
+	public void bug48495() {
 		try {
 			Workbook wb = XSSFTestDataSamples.openSampleWorkbook("48495.xlsx");
 			
@@ -532,7 +596,8 @@ public final class TestXSSFWorkbook extends BaseTestWorkbook {
 		System.out.println("Done");
 	}
 	
-	public void testBug47090a() {
+    @Test
+	public void bug47090a() {
 	    Workbook workbook = XSSFTestDataSamples.openSampleWorkbook("47090.xlsx");
 		assertSheetOrder(workbook, "Sheet1", "Sheet2");
 	    workbook.removeSheetAt(0);
@@ -543,7 +608,8 @@ public final class TestXSSFWorkbook extends BaseTestWorkbook {
 		assertSheetOrder(read, "Sheet2", "Sheet1");
 	}
 	
-	public void testBug47090b() {
+    @Test
+	public void bug47090b() {
 	    Workbook workbook = XSSFTestDataSamples.openSampleWorkbook("47090.xlsx");
 	    assertSheetOrder(workbook, "Sheet1", "Sheet2");
 	    workbook.removeSheetAt(1);
@@ -554,7 +620,8 @@ public final class TestXSSFWorkbook extends BaseTestWorkbook {
 		assertSheetOrder(read, "Sheet1", "Sheet0");
 	}
 
-	public void testBug47090c() {
+    @Test
+	public void bug47090c() {
 	    Workbook workbook = XSSFTestDataSamples.openSampleWorkbook("47090.xlsx");
 	    assertSheetOrder(workbook, "Sheet1", "Sheet2");
 	    workbook.removeSheetAt(0);
@@ -565,7 +632,8 @@ public final class TestXSSFWorkbook extends BaseTestWorkbook {
 		assertSheetOrder(read, "Sheet2", "Sheet2 (2)");
 	}
 	
-	public void testBug47090d() {
+    @Test
+	public void bug47090d() {
 	    Workbook workbook = XSSFTestDataSamples.openSampleWorkbook("47090.xlsx");
 	    assertSheetOrder(workbook, "Sheet1", "Sheet2");
 	    workbook.createSheet();
@@ -578,7 +646,8 @@ public final class TestXSSFWorkbook extends BaseTestWorkbook {
 		assertSheetOrder(read, "Sheet2", "Sheet0", "Sheet1");
 	}
 	
-	public void testBug51158() throws IOException {
+    @Test
+	public void bug51158() throws IOException {
         // create a workbook
         final XSSFWorkbook workbook = new XSSFWorkbook();
         XSSFSheet sheet = workbook.createSheet("Test Sheet");
@@ -614,27 +683,32 @@ public final class TestXSSFWorkbook extends BaseTestWorkbook {
         assertEquals("test2", readBack.getSheetAt(0).getRow(2).getCell(4).getStringCellValue());
 	}
 	
-	public void testBug51158a() throws IOException {
+    @Test
+	public void bug51158a() throws IOException {
         // create a workbook
         final XSSFWorkbook workbook = new XSSFWorkbook();
-        workbook.createSheet("Test Sheet");
-
-        XSSFSheet sheetBack = workbook.getSheetAt(0);
-
-        // committing twice did add the XML twice without clearing the part in between
-        sheetBack.commit();
-
-        // ensure that a memory based package part does not have lingering data from previous commit() calls
-        if(sheetBack.getPackagePart() instanceof MemoryPackagePart) {
-            ((MemoryPackagePart)sheetBack.getPackagePart()).clear();
+        try {
+            workbook.createSheet("Test Sheet");
+    
+            XSSFSheet sheetBack = workbook.getSheetAt(0);
+    
+            // committing twice did add the XML twice without clearing the part in between
+            sheetBack.commit();
+    
+            // ensure that a memory based package part does not have lingering data from previous commit() calls
+            if(sheetBack.getPackagePart() instanceof MemoryPackagePart) {
+                ((MemoryPackagePart)sheetBack.getPackagePart()).clear();
+            }
+    
+            sheetBack.commit();
+    
+            String str = new String(IOUtils.toByteArray(sheetBack.getPackagePart().getInputStream()));
+            System.out.println(str);
+            
+            assertEquals(1, countMatches(str, "<worksheet"));
+        } finally {
+            workbook.close();
         }
-
-        sheetBack.commit();
-
-        String str = new String(IOUtils.toByteArray(sheetBack.getPackagePart().getInputStream()));
-        System.out.println(str);
-        
-        assertEquals(1, countMatches(str, "<worksheet"));
     }	
 	
     private static final int INDEX_NOT_FOUND = -1;
@@ -659,4 +733,140 @@ public final class TestXSSFWorkbook extends BaseTestWorkbook {
     private static int indexOf(CharSequence cs, CharSequence searchChar, int start) {
         return cs.toString().indexOf(searchChar.toString(), start);
     }
+
+    @Test
+    public void testAddPivotCache() throws IOException {
+        XSSFWorkbook wb = new XSSFWorkbook();
+        try {
+            CTWorkbook ctWb = wb.getCTWorkbook();
+            CTPivotCache pivotCache = wb.addPivotCache("0");
+            //Ensures that pivotCaches is initiated
+            assertTrue(ctWb.isSetPivotCaches());
+            assertSame(pivotCache, ctWb.getPivotCaches().getPivotCacheArray(0));
+            assertEquals("0", pivotCache.getId());
+        } finally {
+            wb.close();
+        }
+    }
+
+    public void setPivotData(XSSFWorkbook wb){
+        XSSFSheet sheet = wb.createSheet();
+
+        Row row1 = sheet.createRow(0);
+        // Create a cell and put a value in it.
+        Cell cell = row1.createCell(0);
+        cell.setCellValue("Names");
+        Cell cell2 = row1.createCell(1);
+        cell2.setCellValue("#");
+        Cell cell7 = row1.createCell(2);
+        cell7.setCellValue("Data");
+
+        Row row2 = sheet.createRow(1);
+        Cell cell3 = row2.createCell(0);
+        cell3.setCellValue("Jan");
+        Cell cell4 = row2.createCell(1);
+        cell4.setCellValue(10);
+        Cell cell8 = row2.createCell(2);
+        cell8.setCellValue("Apa");
+
+        Row row3 = sheet.createRow(2);
+        Cell cell5 = row3.createCell(0);
+        cell5.setCellValue("Ben");
+        Cell cell6 = row3.createCell(1);
+        cell6.setCellValue(9);
+        Cell cell9 = row3.createCell(2);
+        cell9.setCellValue("Bepa");
+
+        AreaReference source = new AreaReference("A1:B2");
+        sheet.createPivotTable(source, new CellReference("H5"));
+    }
+
+    @Test
+    public void testLoadWorkbookWithPivotTable() throws Exception {
+        File file = TempFile.createTempFile("ooxml-pivottable", ".xlsx");
+
+        XSSFWorkbook wb = new XSSFWorkbook();
+        try {
+            setPivotData(wb);
+    
+            FileOutputStream fileOut = new FileOutputStream(file);
+            wb.write(fileOut);
+            fileOut.close();
+        } finally {
+            wb.close();
+        }
+
+        XSSFWorkbook wb2 = (XSSFWorkbook) WorkbookFactory.create(file);
+        try {
+            assertTrue(wb2.getPivotTables().size() == 1);
+        } finally {
+            wb2.close();
+        }
+        
+        assertTrue(file.delete());
+    }
+
+    @Test
+    public void testAddPivotTableToWorkbookWithLoadedPivotTable() throws Exception {
+        File file = TempFile.createTempFile("ooxml-pivottable", ".xlsx");
+
+        XSSFWorkbook wb = new XSSFWorkbook();
+        try {
+            setPivotData(wb);
+    
+            FileOutputStream fileOut = new FileOutputStream(file);
+            try {
+                wb.write(fileOut);
+            } finally {
+                fileOut.close();
+            }
+        } finally {
+            wb.close();
+        }
+
+        XSSFWorkbook wb2 = (XSSFWorkbook) WorkbookFactory.create(file);
+        try {
+            setPivotData(wb2);
+            assertTrue(wb2.getPivotTables().size() == 2);
+        } finally {
+            wb2.close();
+        }
+        
+        assertTrue(file.delete());
+    }
+    
+    @Test
+    public void testSetFirstVisibleTab_57373() throws IOException {
+        Workbook wb = new XSSFWorkbook();
+        
+        try {
+            /*Sheet sheet1 =*/ wb.createSheet();
+            Sheet sheet2 = wb.createSheet();
+            int idx2 = wb.getSheetIndex(sheet2);
+            Sheet sheet3 = wb.createSheet();
+            int idx3 = wb.getSheetIndex(sheet3);
+            
+            // add many sheets so "first visible" is relevant
+            for(int i = 0; i < 30;i++) {
+                wb.createSheet();
+            }
+            
+            wb.setFirstVisibleTab(idx2);
+            wb.setActiveSheet(idx3);
+            
+            //wb.write(new FileOutputStream(new File("C:\\temp\\test.xlsx")));
+
+            assertEquals(idx2, wb.getFirstVisibleTab());
+            assertEquals(idx3, wb.getActiveSheetIndex());
+
+            Workbook wbBack = XSSFTestDataSamples.writeOutAndReadBack(wb);
+
+            sheet2 = wbBack.getSheetAt(idx2);
+            sheet3 = wbBack.getSheetAt(idx3);
+            assertEquals(idx2, wb.getFirstVisibleTab());
+            assertEquals(idx3, wb.getActiveSheetIndex());
+        } finally {
+            wb.close();
+        }
+    }
 }
