@@ -17,6 +17,14 @@
 
 package org.apache.poi.hslf.usermodel;
 
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertTrue;
+
+import java.io.ByteArrayOutputStream;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.IOException;
 import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.Date;
@@ -27,14 +35,19 @@ import java.util.Map;
 import java.util.Set;
 
 import junit.framework.AssertionFailedError;
-import junit.framework.TestCase;
 
 import org.apache.poi.POIDataSamples;
+import org.apache.poi.ddf.EscherArrayProperty;
+import org.apache.poi.ddf.EscherColorRef;
+import org.apache.poi.ddf.EscherOptRecord;
+import org.apache.poi.ddf.EscherProperties;
 import org.apache.poi.hslf.HSLFSlideShow;
 import org.apache.poi.hslf.HSLFTestDataSamples;
 import org.apache.poi.hslf.exceptions.OldPowerPointFormatException;
+import org.apache.poi.hslf.model.AutoShape;
 import org.apache.poi.hslf.model.Background;
 import org.apache.poi.hslf.model.Fill;
+import org.apache.poi.hslf.model.HeadersFooters;
 import org.apache.poi.hslf.model.MasterSheet;
 import org.apache.poi.hslf.model.Notes;
 import org.apache.poi.hslf.model.Picture;
@@ -46,6 +59,15 @@ import org.apache.poi.hslf.model.TextBox;
 import org.apache.poi.hslf.model.TextRun;
 import org.apache.poi.hslf.model.TextShape;
 import org.apache.poi.hslf.model.TitleMaster;
+import org.apache.poi.hslf.record.Document;
+import org.apache.poi.hslf.record.Record;
+import org.apache.poi.hslf.record.SlideListWithText;
+import org.apache.poi.hslf.record.SlideListWithText.SlideAtomsSet;
+import org.apache.poi.hslf.record.TextHeaderAtom;
+import org.apache.poi.util.LittleEndian;
+import org.apache.poi.util.StringUtil;
+import org.apache.poi.util.Units;
+import org.junit.Test;
 
 /**
  * Testcases for bugs entered in bugzilla
@@ -53,13 +75,14 @@ import org.apache.poi.hslf.model.TitleMaster;
  *
  * @author Yegor Kozlov
  */
-public final class TestBugs extends TestCase {
+public final class TestBugs {
     private static POIDataSamples _slTests = POIDataSamples.getSlideShowInstance();
 
     /**
      * Bug 41384: Array index wrong in record creation
      */
-    public void test41384() throws Exception {
+    @Test
+    public void bug41384() throws Exception {
         HSLFSlideShow hslf = new HSLFSlideShow(_slTests.openResourceAsStream("41384.ppt"));
 
         SlideShow ppt = new SlideShow(hslf);
@@ -77,7 +100,8 @@ public final class TestBugs extends TestCase {
      * First fix from Bug 42474: NPE in RichTextRun.isBold()
      * when the RichTextRun comes from a Notes model object
      */
-    public void test42474_1() throws Exception {
+    @Test
+    public void bug42474_1() throws Exception {
         HSLFSlideShow hslf = new HSLFSlideShow(_slTests.openResourceAsStream("42474-1.ppt"));
 
         SlideShow ppt = new SlideShow(hslf);
@@ -105,7 +129,8 @@ public final class TestBugs extends TestCase {
     /**
      * Second fix from Bug 42474: Incorrect matching of notes to slides
      */
-    public void test42474_2() throws Exception {
+    @Test
+    public void bug42474_2() throws Exception {
         HSLFSlideShow hslf = new HSLFSlideShow(_slTests.openResourceAsStream("42474-2.ppt"));
 
         SlideShow ppt = new SlideShow(hslf);
@@ -135,7 +160,8 @@ public final class TestBugs extends TestCase {
     /**
      * Bug 42485: All TextBoxes inside ShapeGroups have null TextRuns
      */
-    public void test42485 () throws Exception {
+    @Test
+    public void bug42485 () throws Exception {
         HSLFSlideShow hslf = new HSLFSlideShow(_slTests.openResourceAsStream("42485.ppt"));
 
         SlideShow ppt = new SlideShow(hslf);
@@ -157,7 +183,8 @@ public final class TestBugs extends TestCase {
     /**
      * Bug 42484: NullPointerException from ShapeGroup.getAnchor()
      */
-    public void test42484 () throws Exception {
+    @Test
+    public void bug42484 () throws Exception {
         HSLFSlideShow hslf = new HSLFSlideShow(_slTests.openResourceAsStream("42485.ppt"));
 
         SlideShow ppt = new SlideShow(hslf);
@@ -178,7 +205,8 @@ public final class TestBugs extends TestCase {
     /**
      * Bug 41381: Exception from Slide.getMasterSheet() on a seemingly valid PPT file
      */
-    public void test41381() throws Exception {
+    @Test
+    public void bug41381() throws Exception {
         HSLFSlideShow hslf = new HSLFSlideShow(_slTests.openResourceAsStream("alterman_security.ppt"));
 
         SlideShow ppt = new SlideShow(hslf);
@@ -197,7 +225,8 @@ public final class TestBugs extends TestCase {
     /**
      * Bug 42486:  Failure parsing a seemingly valid PPT
      */
-    public void test42486 () throws Exception {
+    @Test
+    public void bug42486 () throws Exception {
         HSLFSlideShow hslf = new HSLFSlideShow(_slTests.openResourceAsStream("42486.ppt"));
 
         SlideShow ppt = new SlideShow(hslf);
@@ -213,7 +242,8 @@ public final class TestBugs extends TestCase {
     /**
      * Bug 42524:  NPE in Shape.getShapeType()
      */
-    public void test42524 () throws Exception {
+    @Test
+    public void bug42524 () throws Exception {
         HSLFSlideShow hslf = new HSLFSlideShow(_slTests.openResourceAsStream("42486.ppt"));
 
         SlideShow ppt = new SlideShow(hslf);
@@ -240,7 +270,8 @@ public final class TestBugs extends TestCase {
     /**
      * Bug 42520:  NPE in Picture.getPictureData()
      */
-    public void test42520 () throws Exception {
+    @Test
+    public void bug42520 () throws Exception {
         HSLFSlideShow hslf = new HSLFSlideShow(_slTests.openResourceAsStream("42520.ppt"));
 
         SlideShow ppt = new SlideShow(hslf);
@@ -277,7 +308,8 @@ public final class TestBugs extends TestCase {
      * Bug 38256:  RuntimeException: Couldn't instantiate the class for type with id 0.
      * ( also fixed followup: getTextRuns() returns no text )
      */
-    public void test38256 () throws Exception {
+    @Test
+    public void bug38256 () throws Exception {
         SlideShow ppt = new SlideShow(_slTests.openResourceAsStream("38256.ppt"));
 
         assertTrue("No Exceptions while reading file", true);
@@ -304,14 +336,24 @@ public final class TestBugs extends TestCase {
      * Bug 38256:  RuntimeException: Couldn't instantiate the class for type with id 0.
      * ( also fixed followup: getTextRuns() returns no text )
      */
-    public void test43781 () throws Exception {
+    @Test
+    public void bug43781 () throws Exception {
         SlideShow ppt = new SlideShow(_slTests.openResourceAsStream("43781.ppt"));
 
         assertTrue("No Exceptions while reading file", true);
 
+        // Check the first slide
         Slide slide = ppt.getSlides()[0];
-        TextRun[] tr1 = slide.getTextRuns();
+        TextRun[] slTr = slide.getTextRuns();
+        
+        // Has two text runs, one from slide text, one from drawing
+        assertEquals(2, slTr.length);
+        assertEquals(false, slTr[0].isDrawingBased());
+        assertEquals(true, slTr[1].isDrawingBased());
+        assertEquals("First run", slTr[0].getText());
+        assertEquals("Second run", slTr[1].getText());
 
+        // Check the shape based text runs
         List<TextRun> lst = new ArrayList<TextRun>();
         Shape[] shape = slide.getShapes();
         for (int i = 0; i < shape.length; i++) {
@@ -323,19 +365,18 @@ public final class TestBugs extends TestCase {
             }
 
         }
-        TextRun[] tr2 = new TextRun[lst.size()];
-        lst.toArray(tr2);
-
-        assertEquals(tr1.length, tr2.length);
-        for (int i = 0; i < tr1.length; i++) {
-            assertEquals(tr1[i].getText(), tr2[i].getText());
-        }
+        // There should be only one shape based one found
+        assertEquals(1, lst.size());
+        
+        // And it should be the second one
+        assertEquals("Second run", lst.get(0).getText());
     }
 
     /**
      * Bug 44296: HSLF Not Extracting Slide Background Image
      */
-    public void test44296  () throws Exception {
+    @Test
+    public void bug44296  () throws Exception {
         SlideShow ppt = new SlideShow(_slTests.openResourceAsStream("44296.ppt"));
 
         Slide slide = ppt.getSlides()[0];
@@ -352,7 +393,8 @@ public final class TestBugs extends TestCase {
     /**
      * Bug 44770: java.lang.RuntimeException: Couldn't instantiate the class for type with id 1036 on class class org.apache.poi.hslf.record.PPDrawing
      */
-    public void test44770() throws Exception {
+    @Test
+    public void bug44770() throws Exception {
         try {
              new SlideShow(_slTests.openResourceAsStream("44770.ppt"));
         } catch (RuntimeException e) {
@@ -366,7 +408,8 @@ public final class TestBugs extends TestCase {
     /**
      * Bug 41071: Will not extract text from Powerpoint TextBoxes
      */
-    public void test41071() throws Exception {
+    @Test
+    public void bug41071() throws Exception {
         SlideShow ppt = new SlideShow(_slTests.openResourceAsStream("41071.ppt"));
 
         Slide slide = ppt.getSlides()[0];
@@ -385,23 +428,20 @@ public final class TestBugs extends TestCase {
      * PowerPoint 95 files should throw a more helpful exception
      * @throws Exception
      */
-    public void test41711() throws Exception {
+    @Test(expected=OldPowerPointFormatException.class)
+    public void bug41711() throws Exception {
     	// New file is fine
         new SlideShow(_slTests.openResourceAsStream("SampleShow.ppt"));
 
         // PowerPoint 95 gives an old format exception
-        try {
-        	new SlideShow(_slTests.openResourceAsStream("PPT95.ppt"));
-        	fail("OldPowerPointFormatException should've been thrown");
-        } catch(OldPowerPointFormatException e) {
-        	// Good
-        }
+    	new SlideShow(_slTests.openResourceAsStream("PPT95.ppt"));
     }
     
     /**
      * Changing text from Ascii to Unicode
      */
-    public void test49648() throws Exception {
+    @Test
+    public void bug49648() throws Exception {
        SlideShow ppt = new SlideShow(_slTests.openResourceAsStream("49648.ppt"));
        for(Slide slide : ppt.getSlides()) {
           for(TextRun run : slide.getTextRuns()) {
@@ -415,7 +455,8 @@ public final class TestBugs extends TestCase {
     /**
      * Bug 41246: AIOOB with illegal note references
      */
-    public void test41246a() throws Exception {
+    @Test
+    public void bug41246a() throws Exception {
         InputStream fis = _slTests.openResourceAsStream("41246-1.ppt");
         HSLFSlideShow hslf = new HSLFSlideShow(fis);
         fis.close();
@@ -427,7 +468,8 @@ public final class TestBugs extends TestCase {
         assertTrue("No Exceptions while rewriting file", true);
     }
 
-    public void test41246b() throws Exception {
+    @Test
+    public void bug41246b() throws Exception {
         InputStream fis = _slTests.openResourceAsStream("41246-2.ppt");
         HSLFSlideShow hslf = new HSLFSlideShow(fis);
         fis.close();
@@ -442,7 +484,8 @@ public final class TestBugs extends TestCase {
     /**
      * Bug 45776: Fix corrupt file problem using TextRun.setText
      */
-    public void test45776() throws Exception {
+    @Test
+    public void bug45776() throws Exception {
         InputStream is = _slTests.openResourceAsStream("45776.ppt");
         SlideShow ppt = new SlideShow(new HSLFSlideShow(is));
         is.close();
@@ -466,4 +509,159 @@ public final class TestBugs extends TestCase {
         }
     }
 
+    @Test
+    public void bug55732() throws Exception {
+        File file = _slTests.getFile("bug55732.ppt");
+        
+        HSLFSlideShow ss = new HSLFSlideShow(file.getAbsolutePath());
+        SlideShow _show = new SlideShow(ss);
+        Slide[] _slides = _show.getSlides();
+
+        /* Iterate over slides and extract text */
+        for( Slide slide : _slides ) {
+            HeadersFooters hf = slide.getHeadersFooters();
+            /*boolean visible =*/ hf.isHeaderVisible(); // exception happens here
+        }
+        assertTrue("No Exceptions while reading headers", true);
+    }
+    
+    @Test
+    public void bug56260() throws Exception {
+        File file = _slTests.getFile("56260.ppt");
+        
+        HSLFSlideShow ss = new HSLFSlideShow(file.getAbsolutePath());
+        SlideShow _show = new SlideShow(ss);
+        Slide[] _slides = _show.getSlides();
+        assertEquals(13, _slides.length);
+        
+        // Check the number of TextHeaderAtoms on Slide 1
+        Document dr = _show.getDocumentRecord();
+        SlideListWithText slidesSLWT = dr.getSlideSlideListWithText();
+        SlideAtomsSet s1 = slidesSLWT.getSlideAtomsSets()[0];
+
+        int tha = 0;
+        for (Record r : s1.getSlideRecords()) {
+            if (r instanceof TextHeaderAtom) tha++;
+        }
+        assertEquals(2, tha);
+        
+        // Check to see that we have a pair next to each other
+        assertEquals(TextHeaderAtom.class, s1.getSlideRecords()[0].getClass());
+        assertEquals(TextHeaderAtom.class, s1.getSlideRecords()[1].getClass());
+        
+        
+        // Check the number of text runs based on the slide (not textbox)
+        // Will have skipped the empty one
+        int str = 0;
+        for (TextRun tr : _slides[0].getTextRuns()) {
+            if (! tr.isDrawingBased()) str++;
+        }
+        assertEquals(1, str);
+    }
+    
+    @Test
+    public void bug37625() throws IOException {
+        InputStream inputStream = new FileInputStream(_slTests.getFile("37625.ppt"));
+        try {
+            SlideShow slideShow = new SlideShow(inputStream);
+            assertEquals(29, slideShow.getSlides().length);
+            
+            SlideShow slideBack = HSLFTestDataSamples.writeOutAndReadBack(slideShow);
+            assertNotNull(slideBack);
+            assertEquals(29, slideBack.getSlides().length);
+        } finally {
+            inputStream.close();
+        }
+    }
+    
+    @Test
+    public void bug57272() throws Exception {
+        InputStream inputStream = new FileInputStream(_slTests.getFile("57272_corrupted_usereditatom.ppt"));
+        try {
+            SlideShow slideShow = new SlideShow(inputStream);
+            assertEquals(6, slideShow.getSlides().length);
+
+            SlideShow slideBack = HSLFTestDataSamples.writeOutAndReadBack(slideShow);
+            assertNotNull(slideBack);
+            assertEquals(6, slideBack.getSlides().length);
+        } finally {
+            inputStream.close();
+        }
+    }
+
+    @Test
+    public void bug49541() throws Exception {
+        InputStream inputStream = new FileInputStream(_slTests.getFile("49541_symbol_map.ppt"));
+        try {
+            SlideShow slideShow = new SlideShow(inputStream);
+            Slide slide = slideShow.getSlides()[0];
+            ShapeGroup sg = (ShapeGroup)slide.getShapes()[0];
+            TextBox tb = (TextBox)sg.getShapes()[0];
+            String text = StringUtil.mapMsCodepointString(tb.getText());
+            assertEquals("\u226575 years", text);
+        } finally {
+            inputStream.close();
+        }
+    }
+    
+    @Test
+    public void bug47261() throws Exception {
+        InputStream inputStream = new FileInputStream(_slTests.getFile("bug47261.ppt"));
+        try {
+            SlideShow slideShow = new SlideShow(inputStream);
+            slideShow.removeSlide(0);
+            slideShow.createSlide();
+            ByteArrayOutputStream bos = new ByteArrayOutputStream();
+            slideShow.write(bos);
+        } finally {
+            inputStream.close();
+        }
+    }
+    
+    @Test
+    public void bug56240() throws Exception {
+        InputStream inputStream = new FileInputStream(_slTests.getFile("bug56240.ppt"));
+        try {
+            SlideShow slideShow = new SlideShow(inputStream);
+            int slideCnt = slideShow.getSlides().length;
+            assertEquals(105, slideCnt);
+            ByteArrayOutputStream bos = new ByteArrayOutputStream();
+            slideShow.write(bos);
+            bos.close();
+        } finally {
+            inputStream.close();
+        }
+    }
+    
+    @Test
+    public void bug46441() throws Exception {
+        InputStream inputStream = new FileInputStream(_slTests.getFile("bug46441.ppt"));
+        try {
+            SlideShow slideShow = new SlideShow(inputStream);
+            AutoShape as = (AutoShape)slideShow.getSlides()[0].getShapes()[0];
+            EscherOptRecord opt = as.getEscherOptRecord();
+            EscherArrayProperty ep = Shape.getEscherProperty(opt, EscherProperties.FILL__SHADECOLORS);
+            double exp[][] = {
+                // r, g, b, position
+                { 94, 158, 255, 0 },
+                { 133, 194, 255, 0.399994 },
+                { 196, 214, 235, 0.699997 },
+                { 255, 235, 250, 1 }                    
+            };
+            
+            int i = 0;
+            for (byte data[] : ep) {
+                EscherColorRef ecr = new EscherColorRef(data, 0, 4);
+                int rgb[] = ecr.getRGB();
+                double pos = Units.fixedPointToDecimal(LittleEndian.getInt(data, 4));
+                assertEquals((int)exp[i][0], rgb[0]);
+                assertEquals((int)exp[i][1], rgb[1]);
+                assertEquals((int)exp[i][2], rgb[2]);
+                assertEquals(exp[i][3], pos, 0.01);
+                i++;
+            }
+        } finally {
+            inputStream.close();
+        }
+    }
 }
