@@ -17,28 +17,33 @@
 
 package org.apache.poi.hpsf.basic;
 
+import static org.hamcrest.core.IsEqual.equalTo;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertThat;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
+
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FileFilter;
 import java.io.FileInputStream;
+import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
-import java.io.PrintWriter;
-import java.io.StringWriter;
 import java.io.UnsupportedEncodingException;
 import java.nio.charset.Charset;
 import java.util.Date;
 import java.util.HashMap;
 import java.util.Map;
 
-import junit.framework.Assert;
-import junit.framework.TestCase;
 import org.apache.poi.POIDataSamples;
 import org.apache.poi.hpsf.ClassID;
-import org.apache.poi.hpsf.HPSFRuntimeException;
+import org.apache.poi.hpsf.DocumentSummaryInformation;
+import org.apache.poi.hpsf.HPSFException;
 import org.apache.poi.hpsf.IllegalPropertySetDataException;
 import org.apache.poi.hpsf.MutableProperty;
 import org.apache.poi.hpsf.MutablePropertySet;
@@ -59,17 +64,25 @@ import org.apache.poi.hpsf.wellknown.SectionIDMap;
 import org.apache.poi.poifs.eventfilesystem.POIFSReader;
 import org.apache.poi.poifs.eventfilesystem.POIFSReaderEvent;
 import org.apache.poi.poifs.eventfilesystem.POIFSReaderListener;
+import org.apache.poi.poifs.filesystem.DirectoryEntry;
+import org.apache.poi.poifs.filesystem.DocumentNode;
+import org.apache.poi.poifs.filesystem.NDocumentInputStream;
+import org.apache.poi.poifs.filesystem.NDocumentOutputStream;
+import org.apache.poi.poifs.filesystem.NPOIFSDocument;
+import org.apache.poi.poifs.filesystem.NPOIFSFileSystem;
 import org.apache.poi.poifs.filesystem.POIFSFileSystem;
 import org.apache.poi.util.CodePageUtil;
+import org.apache.poi.util.IOUtils;
 import org.apache.poi.util.LittleEndian;
 import org.apache.poi.util.TempFile;
+import org.junit.Assume;
+import org.junit.Before;
+import org.junit.Test;
 
 /**
  * <p>Tests HPSF's writing functionality.</p>
- *
- * @author Rainer Klute (klute@rainer-klute.de)
  */
-public class TestWrite extends TestCase
+public class TestWrite
 {
     private static final POIDataSamples _samples = POIDataSamples.getHPSFInstance();
 
@@ -92,24 +105,20 @@ public class TestWrite extends TestCase
 
     POIFile[] poiFiles;
 
-
-    /**
-     * @see TestCase#setUp()
-     */
+    @Before
     public void setUp()
     {
         VariantSupport.setLogUnsupportedTypes(false);
     }
 
-
-
     /**
      * <p>Writes an empty property set to a POIFS and reads it back
      * in.</p>
      *
      * @exception IOException if an I/O exception occurs
      */
-    public void testNoFormatID() throws IOException
+    @Test
+    public void withoutAFormatID() throws IOException
     {
         final File filename = TempFile.createTempFile(POI_FS, ".doc");
 
@@ -132,11 +141,11 @@ public class TestWrite extends TestCase
                                  SummaryInformation.DEFAULT_STREAM_NAME);
             poiFs.writeFilesystem(out);
             out.close();
-            Assert.fail("Should have thrown a NoFormatIDException.");
+            fail("Should have thrown a NoFormatIDException.");
         }
         catch (Exception ex)
         {
-            Assert.assertTrue(ex instanceof NoFormatIDException);
+            assertTrue(ex instanceof NoFormatIDException);
         }
         finally
         {
@@ -154,7 +163,8 @@ public class TestWrite extends TestCase
      * @exception UnsupportedVariantTypeException if HPSF does not yet support
      * a variant type to be written
      */
-    public void testWriteEmptyPropertySet()
+    @Test
+    public void writeEmptyPropertySet()
         throws IOException, UnsupportedVariantTypeException
     {
         final File dataDir = _samples.getFile("");
@@ -181,7 +191,12 @@ public class TestWrite extends TestCase
         final POIFSReader r = new POIFSReader();
         r.registerListener(new MyPOIFSReaderListener(),
                            SummaryInformation.DEFAULT_STREAM_NAME);
-        r.read(new FileInputStream(filename));
+        FileInputStream stream = new FileInputStream(filename);
+        try {
+            r.read(stream);
+        } finally {
+            stream.close();
+        }
     }
 
 
@@ -194,7 +209,8 @@ public class TestWrite extends TestCase
      * @exception UnsupportedVariantTypeException if HPSF does not yet support
      * a variant type to be written
      */
-    public void testWriteSimplePropertySet()
+    @Test
+    public void writeSimplePropertySet()
         throws IOException, UnsupportedVariantTypeException
     {
         final String AUTHOR = "Rainer Klute";
@@ -242,15 +258,21 @@ public class TestWrite extends TestCase
 
             },
             SummaryInformation.DEFAULT_STREAM_NAME);
-        r.read(new FileInputStream(filename));
-        Assert.assertNotNull(psa[0]);
-        Assert.assertTrue(psa[0].isSummaryInformation());
+        
+        InputStream stream = new FileInputStream(filename);
+        try {
+            r.read(stream);
+        } finally {
+            stream.close();
+        }
+        assertNotNull(psa[0]);
+        assertTrue(psa[0].isSummaryInformation());
 
-        final Section s = (Section) (psa[0].getSections().get(0));
+        final Section s = (psa[0].getSections().get(0));
         Object p1 = s.getProperty(PropertyIDMap.PID_AUTHOR);
         Object p2 = s.getProperty(PropertyIDMap.PID_TITLE);
-        Assert.assertEquals(AUTHOR, p1);
-        Assert.assertEquals(TITLE, p2);
+        assertEquals(AUTHOR, p1);
+        assertEquals(TITLE, p2);
     }
 
 
@@ -263,7 +285,8 @@ public class TestWrite extends TestCase
      * @exception WritingNotSupportedException if HPSF does not yet support
      * a variant type to be written
      */
-    public void testWriteTwoSections()
+    @Test
+    public void writeTwoSections()
         throws WritingNotSupportedException, IOException
     {
         final String STREAM_NAME = "PropertySetStream";
@@ -319,15 +342,20 @@ public class TestWrite extends TestCase
                 }
             },
             STREAM_NAME);
-        r.read(new FileInputStream(filename));
-        Assert.assertNotNull(psa[0]);
-        Section s = (Section) (psa[0].getSections().get(0));
+        FileInputStream stream = new FileInputStream(filename);
+        try {
+            r.read(stream);
+        } finally {
+            stream.close();
+        }
+        assertNotNull(psa[0]);
+        Section s = (psa[0].getSections().get(0));
         assertEquals(s.getFormatID(), formatID);
         Object p = s.getProperty(2);
-        Assert.assertEquals(SECTION1, p);
-        s = (Section) (psa[0].getSections().get(1));
+        assertEquals(SECTION1, p);
+        s = (psa[0].getSections().get(1));
         p = s.getProperty(2);
-        Assert.assertEquals(SECTION2, p);
+        assertEquals(SECTION2, p);
     }
 
 
@@ -360,7 +388,8 @@ public class TestWrite extends TestCase
      * <p>Writes and reads back various variant types and checks whether the
      * stuff that has been read back equals the stuff that was written.</p>
      */
-    public void testVariantTypes()
+    @Test
+    public void variantTypes()
     {
         Throwable t = null;
         final int codepage = CODEPAGE_DEFAULT;
@@ -445,7 +474,8 @@ public class TestWrite extends TestCase
      * checks whether the stuff that has been read back equals the stuff that
      * was written.</p>
      */
-    public void testCodepages()
+    @Test
+    public void codepages()
     {
         Throwable thr = null;
         final int[] validCodepages = new int[]
@@ -531,7 +561,8 @@ public class TestWrite extends TestCase
      * <p>Tests whether writing 8-bit characters to a Unicode property
      * succeeds.</p>
      */
-    public void testUnicodeWrite8Bit()
+    @Test
+    public void unicodeWrite8Bit()
     {
         final String TITLE = "This is a sample title";
         final MutablePropertySet mps = new MutablePropertySet();
@@ -553,7 +584,7 @@ public class TestWrite extends TestCase
 
             PropertySet psr = new PropertySet(bytes);
             assertTrue(psr.isSummaryInformation());
-            Section sr = (Section) psr.getSections().get(0);
+            Section sr = psr.getSections().get(0);
             String title = (String) sr.getProperty(PropertyIDMap.PID_TITLE);
             assertEquals(TITLE, title);
         }
@@ -665,19 +696,27 @@ public class TestWrite extends TestCase
      * the origin file and check whether they are equal.</p></li>
      *
      * </ul>
+     * @throws IOException 
      */
-    public void testRecreate()
+    @Test
+    public void recreate() throws IOException
     {
         final File dataDir = _samples.getFile("");
         final File[] fileList = dataDir.listFiles(new FileFilter()
             {
+                @Override
                 public boolean accept(final File f)
                 {
-                    return f.getName().startsWith("Test");
+                    return f.getName().startsWith("Test") && TestReadAllFiles.checkExclude(f);
                 }
             });
-        for (int i = 0; i < fileList.length; i++)
-            testRecreate(fileList[i]);
+        for (int i = 0; i < fileList.length; i++) {
+            try {
+                testRecreate(fileList[i]);
+            } catch (Exception e) {
+                throw new IOException("While handling file " + fileList[i], e);
+            }
+        }
     }
 
 
@@ -687,58 +726,53 @@ public class TestWrite extends TestCase
      * POI filesystem.</p>
      *
      * @param f the POI filesystem to check
+     * @throws IOException 
+     * @throws HPSFException
      */
-    private void testRecreate(final File f)
+    private void testRecreate(final File f) throws IOException, HPSFException
     {
-        try
+        /* Read the POI filesystem's property set streams: */
+        final POIFile[] psf1 = Util.readPropertySets(f);
+
+        /* Create a new POI filesystem containing the origin file's
+         * property set streams: */
+        final File copy = TempFile.createTempFile(f.getName(), "");
+        copy.deleteOnExit();
+        final OutputStream out = new FileOutputStream(copy);
+        final POIFSFileSystem poiFs = new POIFSFileSystem();
+        for (int i = 0; i < psf1.length; i++)
         {
-            /* Read the POI filesystem's property set streams: */
-            final POIFile[] psf1 = Util.readPropertySets(f);
-
-            /* Create a new POI filesystem containing the origin file's
-             * property set streams: */
-            final File copy = TempFile.createTempFile(f.getName(), "");
-            copy.deleteOnExit();
-            final OutputStream out = new FileOutputStream(copy);
-            final POIFSFileSystem poiFs = new POIFSFileSystem();
-            for (int i = 0; i < psf1.length; i++)
-            {
-                final InputStream in =
-                    new ByteArrayInputStream(psf1[i].getBytes());
-                final PropertySet psIn = PropertySetFactory.create(in);
-                final MutablePropertySet psOut = new MutablePropertySet(psIn);
-                final ByteArrayOutputStream psStream =
-                    new ByteArrayOutputStream();
-                psOut.write(psStream);
-                psStream.close();
-                final byte[] streamData = psStream.toByteArray();
-                poiFs.createDocument(new ByteArrayInputStream(streamData),
-                                     psf1[i].getName());
-                poiFs.writeFilesystem(out);
-            }
-            out.close();
-
-
-            /* Read the property set streams from the POI filesystem just
-             * created. */
-            final POIFile[] psf2 = Util.readPropertySets(copy);
-            for (int i = 0; i < psf2.length; i++)
-            {
-                final byte[] bytes1 = psf1[i].getBytes();
-                final byte[] bytes2 = psf2[i].getBytes();
-                final InputStream in1 = new ByteArrayInputStream(bytes1);
-                final InputStream in2 = new ByteArrayInputStream(bytes2);
-                final PropertySet ps1 = PropertySetFactory.create(in1);
-                final PropertySet ps2 = PropertySetFactory.create(in2);
-
-                /* Compare the property set stream with the corresponding one
-                 * from the origin file and check whether they are equal. */
-                assertEquals("Equality for file " + f.getName(), ps1, ps2);
-            }
+            final InputStream in =
+                new ByteArrayInputStream(psf1[i].getBytes());
+            final PropertySet psIn = PropertySetFactory.create(in);
+            final MutablePropertySet psOut = new MutablePropertySet(psIn);
+            final ByteArrayOutputStream psStream =
+                new ByteArrayOutputStream();
+            psOut.write(psStream);
+            psStream.close();
+            final byte[] streamData = psStream.toByteArray();
+            poiFs.createDocument(new ByteArrayInputStream(streamData),
+                                 psf1[i].getName());
+            poiFs.writeFilesystem(out);
         }
-        catch (Exception ex)
+        out.close();
+
+
+        /* Read the property set streams from the POI filesystem just
+         * created. */
+        final POIFile[] psf2 = Util.readPropertySets(copy);
+        for (int i = 0; i < psf2.length; i++)
         {
-            handle(ex);
+            final byte[] bytes1 = psf1[i].getBytes();
+            final byte[] bytes2 = psf2[i].getBytes();
+            final InputStream in1 = new ByteArrayInputStream(bytes1);
+            final InputStream in2 = new ByteArrayInputStream(bytes2);
+            final PropertySet ps1 = PropertySetFactory.create(in1);
+            final PropertySet ps2 = PropertySetFactory.create(in2);
+
+            /* Compare the property set stream with the corresponding one
+             * from the origin file and check whether they are equal. */
+            assertEquals("Equality for file " + f.getName(), ps1, ps2);
         }
     }
 
@@ -746,75 +780,267 @@ public class TestWrite extends TestCase
 
     /**
      * <p>Tests writing and reading back a proper dictionary.</p>
+     * @throws IOException 
+     * @throws HPSFException 
      */
-    public void testDictionary()
+    @Test
+    public void dictionary() throws IOException, HPSFException
     {
-        try
-        {
-            final File copy = TempFile.createTempFile("Test-HPSF", "ole2");
-            copy.deleteOnExit();
+        final File copy = TempFile.createTempFile("Test-HPSF", "ole2");
+        copy.deleteOnExit();
 
-            /* Write: */
-            final OutputStream out = new FileOutputStream(copy);
-            final POIFSFileSystem poiFs = new POIFSFileSystem();
-            final MutablePropertySet ps1 = new MutablePropertySet();
-            final MutableSection s = (MutableSection) ps1.getSections().get(0);
-            final Map<Long,String> m = new HashMap<Long,String>(3, 1.0f);
-            m.put(Long.valueOf(1), "String 1");
-            m.put(Long.valueOf(2), "String 2");
-            m.put(Long.valueOf(3), "String 3");
-            s.setDictionary(m);
-            s.setFormatID(SectionIDMap.DOCUMENT_SUMMARY_INFORMATION_ID[0]);
-            int codepage = CodePageUtil.CP_UNICODE;
-            s.setProperty(PropertyIDMap.PID_CODEPAGE, Variant.VT_I2,
-                          Integer.valueOf(codepage));
-            poiFs.createDocument(ps1.toInputStream(), "Test");
-            poiFs.writeFilesystem(out);
-            out.close();
+        /* Write: */
+        final OutputStream out = new FileOutputStream(copy);
+        final POIFSFileSystem poiFs = new POIFSFileSystem();
+        final MutablePropertySet ps1 = new MutablePropertySet();
+        final MutableSection s = (MutableSection) ps1.getSections().get(0);
+        final Map<Long,String> m = new HashMap<Long,String>(3, 1.0f);
+        m.put(Long.valueOf(1), "String 1");
+        m.put(Long.valueOf(2), "String 2");
+        m.put(Long.valueOf(3), "String 3");
+        s.setDictionary(m);
+        s.setFormatID(SectionIDMap.DOCUMENT_SUMMARY_INFORMATION_ID[0]);
+        int codepage = CodePageUtil.CP_UNICODE;
+        s.setProperty(PropertyIDMap.PID_CODEPAGE, Variant.VT_I2,
+                      Integer.valueOf(codepage));
+        poiFs.createDocument(ps1.toInputStream(), "Test");
+        poiFs.writeFilesystem(out);
+        out.close();
 
-            /* Read back: */
-            final POIFile[] psf = Util.readPropertySets(copy);
-            Assert.assertEquals(1, psf.length);
-            final byte[] bytes = psf[0].getBytes();
-            final InputStream in = new ByteArrayInputStream(bytes);
-            final PropertySet ps2 = PropertySetFactory.create(in);
+        /* Read back: */
+        final POIFile[] psf = Util.readPropertySets(copy);
+        assertEquals(1, psf.length);
+        final byte[] bytes = psf[0].getBytes();
+        final InputStream in = new ByteArrayInputStream(bytes);
+        final PropertySet ps2 = PropertySetFactory.create(in);
 
-            /* Check if the result is a DocumentSummaryInformation stream, as
-             * specified. */
-            assertTrue(ps2.isDocumentSummaryInformation());
+        /* Check if the result is a DocumentSummaryInformation stream, as
+         * specified. */
+        assertTrue(ps2.isDocumentSummaryInformation());
 
-            /* Compare the property set stream with the corresponding one
-             * from the origin file and check whether they are equal. */
-            assertEquals(ps1, ps2);
-        }
-        catch (Exception ex)
-        {
-            handle(ex);
-        }
+        /* Compare the property set stream with the corresponding one
+         * from the origin file and check whether they are equal. */
+        assertEquals(ps1, ps2);
     }
 
+    /**
+     * Tests that when using NPOIFS, we can do an in-place write
+     *  without needing to stream in + out the whole kitchen sink
+     */
+    @Test
+    public void inPlaceNPOIFSWrite() throws Exception {
+        try {
+            NPOIFSFileSystem fs = null;
+            DirectoryEntry root = null;
+            DocumentNode sinfDoc = null;
+            DocumentNode dinfDoc = null;
+            SummaryInformation sinf = null;
+            DocumentSummaryInformation dinf = null;
+            
+            // We need to work on a File for in-place changes, so create a temp one
+            final File copy = TempFile.createTempFile("Test-HPSF", "ole2");
+            copy.deleteOnExit();
+            
+            // Copy a test file over to our temp location
+            InputStream inp = _samples.openResourceAsStream("TestShiftJIS.doc");
+            FileOutputStream out = new FileOutputStream(copy);
+            IOUtils.copy(inp, out);
+            inp.close();
+            out.close();
+            
+            
+            // Open the copy in read/write mode
+            fs = new NPOIFSFileSystem(copy, false);
+            root = fs.getRoot();
+            
+            
+            // Read the properties in there
+            sinfDoc = (DocumentNode)root.getEntry(SummaryInformation.DEFAULT_STREAM_NAME);
+            dinfDoc = (DocumentNode)root.getEntry(DocumentSummaryInformation.DEFAULT_STREAM_NAME);
+    
+            sinf = (SummaryInformation)PropertySetFactory.create(new NDocumentInputStream(sinfDoc));
+            assertEquals(131077, sinf.getOSVersion());
+            
+            dinf = (DocumentSummaryInformation)PropertySetFactory.create(new NDocumentInputStream(dinfDoc));
+            assertEquals(131077, dinf.getOSVersion());
+            
+            
+            // Check they start as we expect
+            assertEquals("Reiichiro Hori", sinf.getAuthor());
+            assertEquals("Microsoft Word 9.0", sinf.getApplicationName());
+            assertEquals("\u7b2c1\u7ae0", sinf.getTitle());
+            
+            assertEquals("", dinf.getCompany());
+            assertEquals(null, dinf.getManager());
+            
+            
+            // Do an in-place replace via an InputStream
+            new NPOIFSDocument(sinfDoc).replaceContents(sinf.toInputStream());
+            new NPOIFSDocument(dinfDoc).replaceContents(dinf.toInputStream());
+            
+            
+            // Check it didn't get changed
+            sinfDoc = (DocumentNode)root.getEntry(SummaryInformation.DEFAULT_STREAM_NAME);
+            dinfDoc = (DocumentNode)root.getEntry(DocumentSummaryInformation.DEFAULT_STREAM_NAME);
+            
+            sinf = (SummaryInformation)PropertySetFactory.create(new NDocumentInputStream(sinfDoc));
+            assertEquals(131077, sinf.getOSVersion());
+            
+            dinf = (DocumentSummaryInformation)PropertySetFactory.create(new NDocumentInputStream(dinfDoc));
+            assertEquals(131077, dinf.getOSVersion());
+    
+            
+            // Start again!
+            fs.close();
+            inp = _samples.openResourceAsStream("TestShiftJIS.doc");
+            out = new FileOutputStream(copy);
+            IOUtils.copy(inp, out);
+            inp.close();
+            out.close();
+            
+            fs = new NPOIFSFileSystem(copy, false);
+            root = fs.getRoot();
+            
+            // Read the properties in once more
+            sinfDoc = (DocumentNode)root.getEntry(SummaryInformation.DEFAULT_STREAM_NAME);
+            dinfDoc = (DocumentNode)root.getEntry(DocumentSummaryInformation.DEFAULT_STREAM_NAME);
+    
+            sinf = (SummaryInformation)PropertySetFactory.create(new NDocumentInputStream(sinfDoc));
+            assertEquals(131077, sinf.getOSVersion());
+            
+            dinf = (DocumentSummaryInformation)PropertySetFactory.create(new NDocumentInputStream(dinfDoc));
+            assertEquals(131077, dinf.getOSVersion());
+            
+            
+            // Have them write themselves in-place with no changes, as an OutputStream
+            sinf.write(new NDocumentOutputStream(sinfDoc));
+            dinf.write(new NDocumentOutputStream(dinfDoc));
+            
+            // And also write to some bytes for checking
+            ByteArrayOutputStream sinfBytes = new ByteArrayOutputStream();
+            sinf.write(sinfBytes);
+            ByteArrayOutputStream dinfBytes = new ByteArrayOutputStream();
+            dinf.write(dinfBytes);
+            
+            
+            // Check that the filesystem can give us back the same bytes
+            sinfDoc = (DocumentNode)root.getEntry(SummaryInformation.DEFAULT_STREAM_NAME);
+            dinfDoc = (DocumentNode)root.getEntry(DocumentSummaryInformation.DEFAULT_STREAM_NAME);
+    
+            byte[] sinfData = IOUtils.toByteArray(new NDocumentInputStream(sinfDoc));
+            byte[] dinfData = IOUtils.toByteArray(new NDocumentInputStream(dinfDoc));
+            assertThat(sinfBytes.toByteArray(), equalTo(sinfData));
+            assertThat(dinfBytes.toByteArray(), equalTo(dinfData));
+    
+            
+            // Read back in as-is
+            sinf = (SummaryInformation)PropertySetFactory.create(new NDocumentInputStream(sinfDoc));
+            assertEquals(131077, sinf.getOSVersion());
+            
+            dinf = (DocumentSummaryInformation)PropertySetFactory.create(new NDocumentInputStream(dinfDoc));
+            assertEquals(131077, dinf.getOSVersion());
+            
+            assertEquals("Reiichiro Hori", sinf.getAuthor());
+            assertEquals("Microsoft Word 9.0", sinf.getApplicationName());
+            assertEquals("\u7b2c1\u7ae0", sinf.getTitle());
+            
+            assertEquals("", dinf.getCompany());
+            assertEquals(null, dinf.getManager());
+            
+    
+            // Now alter a few of them
+            sinf.setAuthor("Changed Author");
+            sinf.setTitle("Le titre \u00e9tait chang\u00e9");
+            dinf.setManager("Changed Manager");
+            
+            
+            // Save this into the filesystem
+            sinf.write(new NDocumentOutputStream(sinfDoc));
+            dinf.write(new NDocumentOutputStream(dinfDoc));
+            
+            
+            // Read them back in again
+            sinfDoc = (DocumentNode)root.getEntry(SummaryInformation.DEFAULT_STREAM_NAME);
+            sinf = (SummaryInformation)PropertySetFactory.create(new NDocumentInputStream(sinfDoc));
+            assertEquals(131077, sinf.getOSVersion());
+            
+            dinfDoc = (DocumentNode)root.getEntry(DocumentSummaryInformation.DEFAULT_STREAM_NAME);
+            dinf = (DocumentSummaryInformation)PropertySetFactory.create(new NDocumentInputStream(dinfDoc));
+            assertEquals(131077, dinf.getOSVersion());
+    
+            assertEquals("Changed Author", sinf.getAuthor());
+            assertEquals("Microsoft Word 9.0", sinf.getApplicationName());
+            assertEquals("Le titre \u00e9tait chang\u00e9", sinf.getTitle());
+            
+            assertEquals("", dinf.getCompany());
+            assertEquals("Changed Manager", dinf.getManager());
+    
+            
+            // Close the whole filesystem, and open it once more
+            fs.writeFilesystem();
+            fs.close();
+            
+            fs = new NPOIFSFileSystem(copy);
+            root = fs.getRoot();
+            
+            // Re-check on load
+            sinfDoc = (DocumentNode)root.getEntry(SummaryInformation.DEFAULT_STREAM_NAME);
+            sinf = (SummaryInformation)PropertySetFactory.create(new NDocumentInputStream(sinfDoc));
+            assertEquals(131077, sinf.getOSVersion());
+            
+            dinfDoc = (DocumentNode)root.getEntry(DocumentSummaryInformation.DEFAULT_STREAM_NAME);
+            dinf = (DocumentSummaryInformation)PropertySetFactory.create(new NDocumentInputStream(dinfDoc));
+            assertEquals(131077, dinf.getOSVersion());
+    
+            assertEquals("Changed Author", sinf.getAuthor());
+            assertEquals("Microsoft Word 9.0", sinf.getApplicationName());
+            assertEquals("Le titre \u00e9tait chang\u00e9", sinf.getTitle());
+            
+            assertEquals("", dinf.getCompany());
+            assertEquals("Changed Manager", dinf.getManager());
+            
+            
+            // Tidy up
+            fs.close();
+            copy.delete();
+        } catch (FileNotFoundException e) {
+            // On Windows this might always fail, as the nio classes
+            // leave memory mapped buffers active, even when the corresponding channel is closed
+            // The buffers are closed on garbage-collection (but System.gc() can't be forced)
+            // or via sun.misc.Cleaner, but this is regarded unsafe
+            // http://stackoverflow.com/questions/2972986
+            // http://bugs.java.com/view_bug.do?bug_id=4724038
+            Assume.assumeFalse(System.getProperty("os.name").toLowerCase().contains("win"));
+            throw e;
+        }
+    }
 
 
     /**
      * <p>Tests writing and reading back a proper dictionary with an invalid
      * codepage. (HPSF writes Unicode dictionaries only.)</p>
+     * @throws IOException 
+     * @throws HPSFException 
      */
-    public void testDictionaryWithInvalidCodepage()
+    @Test
+    public void dictionaryWithInvalidCodepage() throws IOException, HPSFException
     {
+        final File copy = TempFile.createTempFile("Test-HPSF", "ole2");
+        copy.deleteOnExit();
+        
+        /* Write: */
+        final OutputStream out = new FileOutputStream(copy);
+        
+        final POIFSFileSystem poiFs = new POIFSFileSystem();
+        final MutablePropertySet ps1 = new MutablePropertySet();
+        final MutableSection s = (MutableSection) ps1.getSections().get(0);
+        final Map<Long,String> m = new HashMap<Long, String>(3, 1.0f);
+        m.put(Long.valueOf(1), "String 1");
+        m.put(Long.valueOf(2), "String 2");
+        m.put(Long.valueOf(3), "String 3");
+
         try
         {
-            final File copy = TempFile.createTempFile("Test-HPSF", "ole2");
-            copy.deleteOnExit();
-
-            /* Write: */
-            final OutputStream out = new FileOutputStream(copy);
-            final POIFSFileSystem poiFs = new POIFSFileSystem();
-            final MutablePropertySet ps1 = new MutablePropertySet();
-            final MutableSection s = (MutableSection) ps1.getSections().get(0);
-            final Map<Long,String> m = new HashMap<Long, String>(3, 1.0f);
-            m.put(Long.valueOf(1), "String 1");
-            m.put(Long.valueOf(2), "String 2");
-            m.put(Long.valueOf(3), "String 3");
             s.setDictionary(m);
             s.setFormatID(SectionIDMap.DOCUMENT_SUMMARY_INFORMATION_ID[0]);
             int codepage = 12345;
@@ -827,46 +1053,9 @@ public class TestWrite extends TestCase
         }
         catch (IllegalPropertySetDataException ex)
         {
+            out.close();
             assertTrue(true);
         }
-        catch (Exception ex)
-        {
-            handle(ex);
-        }
-    }
-
-
-
-    /**
-     * <p>Handles unexpected exceptions in testcases.</p>
-     *
-     * @param ex The exception that has been thrown.
-     */
-    private void handle(final Exception ex)
-    {
-        final StringWriter sw = new StringWriter();
-        final PrintWriter pw = new PrintWriter(sw);
-        Throwable t = ex;
-        while (t != null)
-        {
-            t.printStackTrace(pw);
-            if (t instanceof HPSFRuntimeException)
-                t = ((HPSFRuntimeException) t).getReason();
-            else
-                t = null;
-            if (t != null)
-                pw.println("Caused by:");
-        }
-        pw.close();
-        try
-        {
-            sw.close();
-        }
-        catch (IOException ex2)
-        {
-            ex.printStackTrace();
-        }
-        fail(sw.toString());
     }
 
 
