@@ -27,9 +27,12 @@ import static org.junit.Assert.fail;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
+import java.io.FileInputStream;
 import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
 import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Calendar;
@@ -54,6 +57,7 @@ import org.apache.poi.hssf.record.aggregates.FormulaRecordAggregate;
 import org.apache.poi.hssf.record.aggregates.PageSettingsBlock;
 import org.apache.poi.hssf.record.aggregates.RecordAggregate;
 import org.apache.poi.hssf.record.common.UnicodeString;
+import org.apache.poi.hssf.record.crypto.Biff8EncryptionKey;
 import org.apache.poi.poifs.filesystem.NPOIFSFileSystem;
 import org.apache.poi.poifs.filesystem.POIFSFileSystem;
 import org.apache.poi.ss.formula.ptg.Area3DPtg;
@@ -63,10 +67,12 @@ import org.apache.poi.ss.usermodel.BaseTestBugzillaIssues;
 import org.apache.poi.ss.usermodel.Cell;
 import org.apache.poi.ss.usermodel.CellStyle;
 import org.apache.poi.ss.usermodel.DataFormatter;
+import org.apache.poi.ss.usermodel.FormulaEvaluator;
 import org.apache.poi.ss.usermodel.Name;
 import org.apache.poi.ss.usermodel.Row;
 import org.apache.poi.ss.usermodel.Sheet;
 import org.apache.poi.ss.usermodel.Workbook;
+import org.apache.poi.ss.util.CellRangeAddress;
 import org.apache.poi.util.TempFile;
 import org.junit.Ignore;
 import org.junit.Test;
@@ -138,21 +144,6 @@ public final class TestBugs extends BaseTestBugzillaIssues {
         writeOutAndReadBack(wb);
     }
 
-     /** test hyperlinks
-      * open resulting file in excel, and check that there is a link to Google
-      */
-    @Test
-    public void bug15353() {
-        HSSFWorkbook wb = new HSSFWorkbook();
-        HSSFSheet sheet = wb.createSheet("My sheet");
-
-        HSSFRow row = sheet.createRow( 0 );
-        HSSFCell cell = row.createCell( 0 );
-        cell.setCellFormula("HYPERLINK(\"http://google.com\",\"Google\")");
-
-        writeOutAndReadBack(wb);
-    }
-
     /** test reading of a formula with a name and a cell ref in one
      **/
     @Test
@@ -1622,7 +1613,7 @@ public final class TestBugs extends BaseTestBugzillaIssues {
 
     	HSSFSheet s = wb.getSheetAt(0);
         HSSFCell cell1 = s.getRow(0).getCell(0);
-        assertEquals("test ", cell1.getStringCellValue().toString());
+        assertEquals("test ", cell1.getStringCellValue());
 
         HSSFCell cell2 = s.getRow(0).getCell(1);
         assertEquals(1.0, cell2.getNumericCellValue(), 0.0);
@@ -1643,7 +1634,11 @@ public final class TestBugs extends BaseTestBugzillaIssues {
 
     @Test
     public void bug47251() {
+        // Firstly, try with one that triggers on InterfaceHdrRecord
         openSample("47251.xls");
+        
+        // Now with one that triggers on NoteRecord
+        openSample("47251_1.xls");
     }
     
     /**
@@ -2107,6 +2102,8 @@ public final class TestBugs extends BaseTestBugzillaIssues {
      */
     @Test
     public void bug50833() throws Exception {
+       Biff8EncryptionKey.setCurrentUserPassword(null); 
+        
        HSSFWorkbook wb = openSample("50833.xls");
        HSSFSheet s = wb.getSheetAt(0);
        assertEquals("Sheet1", s.getSheetName());
@@ -2208,44 +2205,6 @@ public final class TestBugs extends BaseTestBugzillaIssues {
        assertEquals(54.0+24.0-60, s.getRow(7).getCell(0).getNumericCellValue(), 0);
     }
     
-    /**
-     * HLookup and VLookup with optional arguments 
-     */
-    @Test
-    public void bug51024() throws Exception {
-       HSSFWorkbook wb = new HSSFWorkbook();
-       HSSFSheet s = wb.createSheet();
-       HSSFRow r1 = s.createRow(0);
-       HSSFRow r2 = s.createRow(1);
-       
-       r1.createCell(0).setCellValue("v A1");
-       r2.createCell(0).setCellValue("v A2");
-       r1.createCell(1).setCellValue("v B1");
-       
-       HSSFCell c = r1.createCell(4);
-       
-       HSSFFormulaEvaluator eval = new HSSFFormulaEvaluator(wb);
-       
-       c.setCellFormula("VLOOKUP(\"v A1\", A1:B2, 1)");
-       assertEquals("v A1", eval.evaluate(c).getStringValue());
-       
-       c.setCellFormula("VLOOKUP(\"v A1\", A1:B2, 1, 1)");
-       assertEquals("v A1", eval.evaluate(c).getStringValue());
-       
-       c.setCellFormula("VLOOKUP(\"v A1\", A1:B2, 1, )");
-       assertEquals("v A1", eval.evaluate(c).getStringValue());
-       
-       
-       c.setCellFormula("HLOOKUP(\"v A1\", A1:B2, 1)");
-       assertEquals("v A1", eval.evaluate(c).getStringValue());
-       
-       c.setCellFormula("HLOOKUP(\"v A1\", A1:B2, 1, 1)");
-       assertEquals("v A1", eval.evaluate(c).getStringValue());
-       
-       c.setCellFormula("HLOOKUP(\"v A1\", A1:B2, 1, )");
-       assertEquals("v A1", eval.evaluate(c).getStringValue());
-    }
-    
     /**
      * Mixture of Ascii and Unicode strings in a 
      *  NameComment record
@@ -2344,14 +2303,9 @@ public final class TestBugs extends BaseTestBugzillaIssues {
      * Normally encrypted files have BOF then FILEPASS, but
      *  some may squeeze a WRITEPROTECT in the middle
      */
-    @Test
+    @Test(expected=EncryptedDocumentException.class)
     public void bug51832() {
-       try {
-          openSample("51832.xls");
-          fail("Encrypted file");
-       } catch(EncryptedDocumentException e) {
-          // Good
-       }
+        openSample("51832.xls");
     }
 
     @Test
@@ -2377,6 +2331,31 @@ public final class TestBugs extends BaseTestBugzillaIssues {
         writeOutAndReadBack(workbook);
     }
     
+    /**
+     * Note - part of this test is still failing, see
+     * {@link TestUnfixedBugs#test49612()}
+     */
+    @Test
+    public void bug49612_part() throws IOException {
+        HSSFWorkbook wb = HSSFTestDataSamples.openSampleWorkbook("49612.xls");
+        HSSFSheet sh = wb.getSheetAt(0);
+        HSSFRow row = sh.getRow(0);
+        HSSFCell c1 = row.getCell(2);
+        HSSFCell d1 = row.getCell(3);
+        HSSFCell e1 = row.getCell(2);
+
+        assertEquals("SUM(BOB+JIM)", c1.getCellFormula());
+
+        // Problem 1: See TestUnfixedBugs#test49612()
+        // Problem 2: TestUnfixedBugs#test49612()
+
+        // Problem 3: These used to fail, now pass
+        HSSFFormulaEvaluator eval = new HSSFFormulaEvaluator(wb);
+        assertEquals("evaluating c1", 30.0, eval.evaluate(c1).getNumberValue(), 0.001);
+        assertEquals("evaluating d1", 30.0, eval.evaluate(d1).getNumberValue(), 0.001);
+        assertEquals("evaluating e1", 30.0, eval.evaluate(e1).getNumberValue(), 0.001);
+    }
+    
     @Test
     public void bug51675(){
         final List<Short> list = new ArrayList<Short>();
@@ -2474,10 +2453,277 @@ public final class TestBugs extends BaseTestBugzillaIssues {
         assertEquals(rstyle.getBorderBottom(), HSSFCellStyle.BORDER_DOUBLE);
     }
 
-    @Test(expected=EncryptedDocumentException.class)
+    @Test
     public void bug35897() throws Exception {
         // password is abc
-        openSample("xor-encryption-abc.xls");
+        try {
+            Biff8EncryptionKey.setCurrentUserPassword("abc");
+            openSample("xor-encryption-abc.xls");
+        } finally {
+            Biff8EncryptionKey.setCurrentUserPassword(null);
+        }
+        
+        // One using the only-recently-documented encryption header type 4,
+        //  and the RC4 CryptoAPI encryption header structure
+        try {
+            openSample("35897-type4.xls");
+            fail("POI doesn't currently support the RC4 CryptoAPI encryption header structure");
+        } catch (EncryptedDocumentException e) {}
     }
 
+    @Test
+    public void bug56450() {
+        HSSFWorkbook wb = openSample("56450.xls");
+        HSSFSheet sheet = wb.getSheetAt(0);
+        int comments = 0;
+        for (Row r : sheet) {
+            for (Cell c : r) {
+                if (c.getCellComment() != null) {
+                    assertNotNull(c.getCellComment().getString().getString());
+                    comments++;
+                }
+            }
+        }
+        assertEquals(0, comments);
+    }
+    
+    /**
+     * Files initially created with Excel 2010 can have >3 CF rules
+     */
+    @Test
+    public void bug56482() {
+        HSSFWorkbook wb = openSample("56482.xls");
+        assertEquals(1, wb.getNumberOfSheets());
+        
+        HSSFSheet sheet = wb.getSheetAt(0);
+        HSSFSheetConditionalFormatting cf = sheet.getSheetConditionalFormatting();
+        
+        assertEquals(5, cf.getNumConditionalFormattings());
+    }
+
+    @Test
+    public void bug56325() throws IOException {
+        HSSFWorkbook wb;
+
+        File file = HSSFTestDataSamples.getSampleFile("56325.xls");
+        InputStream stream = new FileInputStream(file);
+        try {
+            POIFSFileSystem fs = new POIFSFileSystem(stream);
+            wb = new HSSFWorkbook(fs);            
+        } finally {
+            stream.close();
+        }
+        
+        assertEquals(3, wb.getNumberOfSheets());
+        wb.removeSheetAt(0);
+        assertEquals(2, wb.getNumberOfSheets());
+
+        wb = HSSFTestDataSamples.writeOutAndReadBack(wb);
+        assertEquals(2, wb.getNumberOfSheets());
+        wb.removeSheetAt(0);
+        assertEquals(1, wb.getNumberOfSheets());
+        wb.removeSheetAt(0);
+        assertEquals(0, wb.getNumberOfSheets());
+        
+        wb = HSSFTestDataSamples.writeOutAndReadBack(wb);
+        assertEquals(0, wb.getNumberOfSheets());
+    }
+
+    @Test
+    public void bug56325a() throws IOException {
+        HSSFWorkbook wb = HSSFTestDataSamples.openSampleWorkbook("56325a.xls");
+        
+        HSSFSheet sheet = wb.cloneSheet(2);
+        wb.setSheetName(3, "Clone 1");
+        sheet.setRepeatingRows(CellRangeAddress.valueOf("2:3"));
+        wb.setPrintArea(3, "$A$4:$C$10");
+        
+        sheet = wb.cloneSheet(2);
+        wb.setSheetName(4, "Clone 2");
+        sheet.setRepeatingRows(CellRangeAddress.valueOf("2:3"));
+        wb.setPrintArea(4, "$A$4:$C$10");
+        
+        wb.removeSheetAt(2);
+
+        Workbook wbBack = HSSFTestDataSamples.writeOutAndReadBack(wb);
+        assertEquals(4, wbBack.getNumberOfSheets());
+        
+//        OutputStream fOut = new FileOutputStream("/tmp/56325a.xls");
+//        try {
+//        	wb.write(fOut);
+//        } finally {
+//        	fOut.close();
+//        }
+    }
+    
+    /**
+     * Formulas which reference named ranges, either in other
+     *  sheets, or workbook scoped but in other workbooks.
+     * Used to fail with 
+     * java.lang.RuntimeException: Unexpected eval class (org.apache.poi.ss.formula.eval.NameXEval)
+     */
+    @Test
+    public void bug56737() throws IOException {
+        Workbook wb = openSample("56737.xls");
+        
+        // Check the named range definitions
+        Name nSheetScope = wb.getName("NR_To_A1");
+        Name nWBScope = wb.getName("NR_Global_B2");
+
+        assertNotNull(nSheetScope);
+        assertNotNull(nWBScope);
+        
+        assertEquals("Defines!$A$1", nSheetScope.getRefersToFormula());
+        assertEquals("Defines!$B$2", nWBScope.getRefersToFormula());
+        
+        // Check the different kinds of formulas
+        Sheet s = wb.getSheetAt(0);
+        Cell cRefSName = s.getRow(1).getCell(3);
+        Cell cRefWName = s.getRow(2).getCell(3);
+        
+        assertEquals("Defines!NR_To_A1", cRefSName.getCellFormula());
+        
+        // TODO Correct this, so that the filename is shown too, see bug #56742
+        // This is what Excel itself shows
+        //assertEquals("'56737.xls'!NR_Global_B2", cRefWName.getCellFormula());
+        // TODO This isn't right, but it's what we currently generate....
+        assertEquals("NR_Global_B2", cRefWName.getCellFormula());
+        
+        // Try to evaluate them
+        FormulaEvaluator eval = wb.getCreationHelper().createFormulaEvaluator();
+        assertEquals("Test A1", eval.evaluate(cRefSName).getStringValue());
+        assertEquals(142, (int)eval.evaluate(cRefWName).getNumberValue());
+        
+        // Try to evaluate everything
+        eval.evaluateAll();
+    }
+    
+    /**
+     * ClassCastException in HSSFOptimiser - StyleRecord cannot be cast to 
+     * ExtendedFormatRecord when removing un-used styles
+     */
+    @Test
+    public void bug54443() throws Exception {
+        HSSFWorkbook workbook = new HSSFWorkbook( );
+        HSSFCellStyle style = workbook.createCellStyle();
+        HSSFCellStyle newStyle = workbook.createCellStyle();
+                    
+        HSSFSheet mySheet = workbook.createSheet();
+        HSSFRow row = mySheet.createRow(0);
+        HSSFCell cell = row.createCell(0);
+        
+        // Use style
+        cell.setCellStyle(style);
+        // Switch to newStyle, style is now un-used
+        cell.setCellStyle(newStyle);
+        
+        // Optimise
+        HSSFOptimiser.optimiseCellStyles(workbook);
+    }
+    
+    /**
+     * Intersection formula ranges, eg =(C2:D3 D3:E4)
+     */
+    @Test
+    public void bug52111() throws Exception {
+        Workbook wb = openSample("Intersection-52111.xls");
+        Sheet s = wb.getSheetAt(0);
+
+        // Check we can read it correctly
+        Cell intF = s.getRow(2).getCell(0);
+        assertEquals(Cell.CELL_TYPE_FORMULA, intF.getCellType());
+        assertEquals(Cell.CELL_TYPE_NUMERIC, intF.getCachedFormulaResultType());
+        
+        assertEquals("(C2:D3 D3:E4)", intF.getCellFormula());
+        
+        // Check we can evaluate it correctly
+        FormulaEvaluator eval = wb.getCreationHelper().createFormulaEvaluator();
+        assertEquals("4.0", eval.evaluate(intF).formatAsString());
+    }
+
+    @Test
+    public void bug42016() {
+        Workbook wb = openSample("42016.xls");
+        Sheet s = wb.getSheetAt(0);
+        for(int row = 0;row < 7;row++) {
+            assertEquals("A$1+B$1", s.getRow(row).getCell(2).getCellFormula());
+        }        
+    }
+
+    /**
+     * Unexpected record type (org.apache.poi.hssf.record.ColumnInfoRecord)
+     */
+    @Test
+    public void bug53984() {
+        Workbook wb = openSample("53984.xls");
+        Sheet s = wb.getSheetAt(0);
+        assertEquals("International Communication Services SA", s.getRow(2).getCell(0).getStringCellValue());
+        assertEquals("Saudi Arabia-Riyadh", s.getRow(210).getCell(0).getStringCellValue());
+    }
+    
+    /**
+     * Read, write, read for formulas point to cells in other files.
+     * See {@link #bug46670()} for the main test, this just
+     *  covers reading an existing file and checking it.
+     * TODO Fix this so that it works - formulas are ending up as
+     *  #REF when being changed
+     */
+//    @Test
+    public void bug46670_existing() {
+        HSSFWorkbook wb;
+        Sheet s;
+        Cell c;
+        
+        // Expected values
+        String refLocal = "'[refs/airport.xls]Sheet1'!$A$2";
+        String refHttp  = "'[9http://www.principlesofeconometrics.com/excel/airline.xls]Sheet1'!$A$2";
+        
+        // Check we can read them correctly
+        wb = openSample("46670_local.xls");
+        s = wb.getSheetAt(0);
+        assertEquals(refLocal, s.getRow(0).getCell(0).getCellFormula());
+        
+        wb = openSample("46670_http.xls");
+        s = wb.getSheetAt(0);
+        assertEquals(refHttp, s.getRow(0).getCell(0).getCellFormula());
+        
+        // Now try to set them to the same values, and ensure that
+        //  they end up as they did before, even with a save and re-load
+        wb = openSample("46670_local.xls");
+        s = wb.getSheetAt(0);
+        c = s.getRow(0).getCell(0);
+        c.setCellFormula(refLocal);
+        assertEquals(refLocal, c.getCellFormula());
+        
+        wb = HSSFTestDataSamples.writeOutAndReadBack(wb);
+        s = wb.getSheetAt(0);
+        assertEquals(refLocal, s.getRow(0).getCell(0).getCellFormula());
+
+        
+        wb = openSample("46670_http.xls");
+        s = wb.getSheetAt(0);
+        c = s.getRow(0).getCell(0);
+        c.setCellFormula(refHttp);
+        assertEquals(refHttp, c.getCellFormula());
+        
+        wb = HSSFTestDataSamples.writeOutAndReadBack(wb);
+        s = wb.getSheetAt(0);
+        assertEquals(refHttp, s.getRow(0).getCell(0).getCellFormula());
+    }
+
+    @Test
+    public void test57456() throws IOException {
+        Workbook wb = openSample("57456.xls");
+        wb.close();
+    }
+    
+    @Test
+    public void test57163() throws IOException {
+        Workbook wb = openSample("57163.xls");
+        
+        while (wb.getNumberOfSheets() > 1) {
+            wb.removeSheetAt(1);
+        }
+        wb.close();
+    }
 }
