@@ -17,9 +17,18 @@
 
 package org.apache.poi.hssf.usermodel;
 
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
+import static org.apache.poi.POITestCase.assertContains;
+
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
+import java.io.FileInputStream;
 import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
 import java.io.IOException;
@@ -33,6 +42,7 @@ import org.apache.poi.ddf.EscherBSERecord;
 import org.apache.poi.hpsf.ClassID;
 import org.apache.poi.hssf.HSSFITestDataProvider;
 import org.apache.poi.hssf.HSSFTestDataSamples;
+import org.apache.poi.hssf.OldExcelFormatException;
 import org.apache.poi.hssf.model.HSSFFormulaParser;
 import org.apache.poi.hssf.model.InternalSheet;
 import org.apache.poi.hssf.model.InternalWorkbook;
@@ -52,6 +62,7 @@ import org.apache.poi.ss.usermodel.Name;
 import org.apache.poi.ss.util.CellRangeAddress;
 import org.apache.poi.util.LittleEndian;
 import org.apache.poi.util.TempFile;
+import org.junit.Test;
 
 /**
  * Tests for {@link HSSFWorkbook}
@@ -68,7 +79,8 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
         return wb.getWorkbook();
     }
 
-    public void testWindowOneDefaults() {
+    @Test
+    public void windowOneDefaults() {
         HSSFWorkbook b = new HSSFWorkbook( );
         try {
             assertEquals(b.getActiveSheetIndex(), 0);
@@ -81,7 +93,8 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
     /**
      * Tests for {@link HSSFWorkbook#isHidden()} etc
      */
-    public void testHidden() {
+    @Test
+    public void hidden() {
         HSSFWorkbook wb = new HSSFWorkbook();
 
         WindowOneRecord w1 = wb.getWorkbook().getWindowOne();
@@ -105,7 +118,8 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
         assertEquals(false, w1.getHidden());
     }
 
-    public void testSheetClone() {
+    @Test
+    public void sheetClone() {
         // First up, try a simple file
         HSSFWorkbook b = new HSSFWorkbook();
         assertEquals(0, b.getNumberOfSheets());
@@ -122,8 +136,9 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
         b.cloneSheet(0);
         assertEquals(2, b.getNumberOfSheets());
     }
-    
-    public void testReadWriteWithCharts() {
+
+    @Test
+    public void readWriteWithCharts() {
         HSSFWorkbook b;
         HSSFSheet s;
 
@@ -188,7 +203,8 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
     }
 
     @SuppressWarnings("deprecation")
-    public void testSelectedSheet_bug44523() {
+    @Test
+    public void selectedSheet_bug44523() {
         HSSFWorkbook wb=new HSSFWorkbook();
         HSSFSheet sheet1 = wb.createSheet("Sheet1");
         HSSFSheet sheet2 = wb.createSheet("Sheet2");
@@ -221,7 +237,9 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
         confirmActiveSelected(sheet4, false);
     }
 
-    public void testSelectMultiple() {
+    @SuppressWarnings("unused")
+    @Test
+    public void selectMultiple() {
         HSSFWorkbook wb=new HSSFWorkbook();
         HSSFSheet sheet1 = wb.createSheet("Sheet1");
         HSSFSheet sheet2 = wb.createSheet("Sheet2");
@@ -276,7 +294,8 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
     }
 
 
-    public void testActiveSheetAfterDelete_bug40414() {
+    @Test
+    public void activeSheetAfterDelete_bug40414() {
         HSSFWorkbook wb=new HSSFWorkbook();
         HSSFSheet sheet0 = wb.createSheet("Sheet0");
         HSSFSheet sheet1 = wb.createSheet("Sheet1");
@@ -359,7 +378,8 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
      * errors are particularly hard to track down.  This test ensures that HSSFWorkbook throws
      * a specific exception as soon as the situation is detected. See bugzilla 45066
      */
-    public void testSheetSerializeSizeMismatch_bug45066() {
+    @Test
+    public void sheetSerializeSizeMismatch_bug45066() {
         HSSFWorkbook wb = new HSSFWorkbook();
         InternalSheet sheet = wb.createSheet("Sheet1").getSheet();
         List<RecordBase> sheetRecords = sheet.getRecords();
@@ -379,7 +399,8 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
      * Checks that us and HSSFName play nicely with named ranges
      *  that point to deleted sheets
      */
-    public void testNamesToDeleteSheets() {
+    @Test
+    public void namesToDeleteSheets() {
         HSSFWorkbook b = HSSFTestDataSamples.openSampleWorkbook("30978-deleted.xls");
         assertEquals(3, b.getNumberOfNames());
 
@@ -484,7 +505,8 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
     /**
      * The sample file provided with bug 45582 seems to have one extra byte after the EOFRecord
      */
-    public void testExtraDataAfterEOFRecord() {
+    @Test
+    public void extraDataAfterEOFRecord() {
         try {
             HSSFTestDataSamples.openSampleWorkbook("ex45582-22397.xls");
         } catch (RecordFormatException e) {
@@ -498,7 +520,8 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
      * Test to make sure that NameRecord.getSheetNumber() is interpreted as a
      * 1-based sheet tab index (not a 1-based extern sheet index)
      */
-    public void testFindBuiltInNameRecord() {
+    @Test
+    public void findBuiltInNameRecord() {
         // testRRaC has multiple (3) built-in name records
         // The second print titles name record has getSheetNumber()==4
         HSSFWorkbook wb = HSSFTestDataSamples.openSampleWorkbook("testRRaC.xls");
@@ -527,7 +550,8 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
     /**
      * Test that the storage clsid property is preserved
      */
-    public void test47920() throws IOException {
+    @Test
+    public void bug47920() throws IOException {
         POIFSFileSystem fs1 = new POIFSFileSystem(POIDataSamples.getSpreadSheetInstance().openResourceAsStream("47920.xls"));
         HSSFWorkbook wb = new HSSFWorkbook(fs1);
         ClassID clsid1 = fs1.getRoot().getStorageClsid();
@@ -541,11 +565,46 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
         assertTrue(clsid1.equals(clsid2));
     }
     
+    /**
+     * If we try to open an old (pre-97) workbook, we get a helpful
+     *  Exception give to explain what we've done wrong
+     */
+    @Test
+    public void helpfulExceptionOnOldFiles() throws Exception {
+        InputStream excel4 = POIDataSamples.getSpreadSheetInstance().openResourceAsStream("testEXCEL_4.xls");
+        try {
+            new HSSFWorkbook(excel4);
+            fail("Shouldn't be able to load an Excel 4 file");
+        } catch (OldExcelFormatException e) {
+            assertContains(e.getMessage(), "BIFF4");
+        }
+        excel4.close();
+        
+        InputStream excel5 = POIDataSamples.getSpreadSheetInstance().openResourceAsStream("testEXCEL_5.xls");
+        try {
+            new HSSFWorkbook(excel5);
+            fail("Shouldn't be able to load an Excel 5 file");
+        } catch (OldExcelFormatException e) {
+            assertContains(e.getMessage(), "BIFF5");
+        }
+        excel5.close();
+        
+        InputStream excel95 = POIDataSamples.getSpreadSheetInstance().openResourceAsStream("testEXCEL_95.xls");
+        try {
+            new HSSFWorkbook(excel95);
+            fail("Shouldn't be able to load an Excel 95 file");
+        } catch (OldExcelFormatException e) {
+            assertContains(e.getMessage(), "BIFF5");
+        }
+        excel95.close();
+    }
+
     /**
      * Tests that we can work with both {@link POIFSFileSystem}
      *  and {@link NPOIFSFileSystem}
      */
-    public void testDifferentPOIFS() throws Exception {
+    @Test
+    public void differentPOIFS() throws Exception {
        // Open the two filesystems
        DirectoryNode[] files = new DirectoryNode[2];
        files[0] = (new POIFSFileSystem(HSSFTestDataSamples.openSampleFileStream("Simple.xls"))).getRoot();
@@ -572,8 +631,9 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
            npoifsFileSystem.close();
        }
     }
-    
-    public void testWordDocEmbeddedInXls() throws IOException {
+
+    @Test
+    public void wordDocEmbeddedInXls() throws IOException {
        // Open the two filesystems
        DirectoryNode[] files = new DirectoryNode[2];
        files[0] = (new POIFSFileSystem(HSSFTestDataSamples.openSampleFileStream("WithEmbeddedObjects.xls"))).getRoot();
@@ -610,24 +670,31 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
      *  again (via POIFS) and have it be valid
      * @throws IOException
      */
-    public void testWriteWorkbookFromNPOIFS() throws IOException {
+    @Test
+    public void writeWorkbookFromNPOIFS() throws IOException {
        InputStream is = HSSFTestDataSamples.openSampleFileStream("WithEmbeddedObjects.xls");
-       NPOIFSFileSystem fs = new NPOIFSFileSystem(is);
-       
-       // Start as NPOIFS
-       HSSFWorkbook wb = new HSSFWorkbook(fs.getRoot(), true);
-       assertEquals(3, wb.getNumberOfSheets());
-       assertEquals("Root xls", wb.getSheetAt(0).getRow(0).getCell(0).getStringCellValue());
-       
-       // Will switch to POIFS
-       wb = HSSFTestDataSamples.writeOutAndReadBack(wb);
-       assertEquals(3, wb.getNumberOfSheets());
-       assertEquals("Root xls", wb.getSheetAt(0).getRow(0).getCell(0).getStringCellValue());
-       
-       fs.close();
+       try {
+           NPOIFSFileSystem fs = new NPOIFSFileSystem(is);
+           try {
+               // Start as NPOIFS
+               HSSFWorkbook wb = new HSSFWorkbook(fs.getRoot(), true);
+               assertEquals(3, wb.getNumberOfSheets());
+               assertEquals("Root xls", wb.getSheetAt(0).getRow(0).getCell(0).getStringCellValue());
+
+               // Will switch to POIFS
+               wb = HSSFTestDataSamples.writeOutAndReadBack(wb);
+               assertEquals(3, wb.getNumberOfSheets());
+               assertEquals("Root xls", wb.getSheetAt(0).getRow(0).getCell(0).getStringCellValue());
+           } finally {
+        	   fs.close();
+           }
+       } finally {
+    	   is.close();
+       }
     }
-   
-    public void testCellStylesLimit() {
+
+    @Test
+    public void cellStylesLimit() {
         HSSFWorkbook wb = new HSSFWorkbook();
         int numBuiltInStyles = wb.getNumCellStyles();
         int MAX_STYLES = 4030;
@@ -647,7 +714,8 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
         assertEquals(MAX_STYLES, wb.getNumCellStyles());
     }
 
-    public void testSetSheetOrderHSSF(){
+    @Test
+    public void setSheetOrderHSSF(){
         HSSFWorkbook wb = new HSSFWorkbook();
         HSSFSheet s1 = wb.createSheet("first sheet");
         HSSFSheet s2 = wb.createSheet("other sheet");
@@ -702,7 +770,7 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
         assertEquals("'first sheet'!D1", cf.getRule(0).getFormula1());
         assertEquals("'other sheet'!D1", cf.getRule(0).getFormula2());
     }
-    
+
     private boolean hasEntry(DirectoryNode dirNode, String entryName) {
        try {
            dirNode.getEntry(entryName);
@@ -712,11 +780,12 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
        }
    }
 
-    public void testClonePictures() throws IOException {
+    @Test
+    public void clonePictures() throws IOException {
         HSSFWorkbook wb = HSSFTestDataSamples.openSampleWorkbook("SimpleWithImages.xls");
         InternalWorkbook iwb = wb.getWorkbook();
         iwb.findDrawingGroup();
-        
+
         for(int pictureIndex=1; pictureIndex <= 4; pictureIndex++){
             EscherBSERecord bse = iwb.getBSERecord(pictureIndex);
             assertEquals(1, bse.getRef());
@@ -735,11 +804,13 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
         }
     }
 
-    public void testChangeSheetNameWithSharedFormulas() {
+    @Test
+    public void changeSheetNameWithSharedFormulas() {
         changeSheetNameWithSharedFormulas("shared_formulas.xls");
     }
 
-    public void testEmptyDirectoryNode() throws IOException {
+    @Test
+    public void emptyDirectoryNode() throws IOException {
         try {
             assertNotNull(new HSSFWorkbook(new POIFSFileSystem()));
             fail("Should catch exception about invalid POIFSFileSystem");
@@ -747,9 +818,10 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
             assertTrue(e.getMessage(), e.getMessage().contains("does not contain a BIFF8"));
         }
     }
-    
+
     @SuppressWarnings("deprecation")
-    public void testSelectedSheetShort() {
+    @Test
+    public void selectedSheetShort() {
         HSSFWorkbook wb=new HSSFWorkbook();
         HSSFSheet sheet1 = wb.createSheet("Sheet1");
         HSSFSheet sheet2 = wb.createSheet("Sheet2");
@@ -762,7 +834,7 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
         confirmActiveSelected(sheet4, false);
 
         wb.setSelectedTab((short)1);
-        
+
         // see Javadoc, in this case selected means "active"
         assertEquals(wb.getActiveSheetIndex(), wb.getSelectedTab());
 
@@ -780,14 +852,15 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
         confirmActiveSelected(sheet2, true);
         confirmActiveSelected(sheet3, false);
         confirmActiveSelected(sheet4, false);
-        
+
         assertEquals(0, wb.getFirstVisibleTab());
         wb.setDisplayedTab((short)2);
         assertEquals(2, wb.getFirstVisibleTab());
         assertEquals(2, wb.getDisplayedTab());
     }
 
-    public void testAddSheetTwice() {
+    @Test
+    public void addSheetTwice() {
         HSSFWorkbook wb=new HSSFWorkbook();
         HSSFSheet sheet1 = wb.createSheet("Sheet1");
         assertNotNull(sheet1);
@@ -798,19 +871,20 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
             assertTrue(e.getMessage(), e.getMessage().contains("already contains a sheet of this name"));
         }
     }
-    
-    public void testGetSheetIndex() {
+
+    @Test
+    public void getSheetIndex() {
         HSSFWorkbook wb=new HSSFWorkbook();
         HSSFSheet sheet1 = wb.createSheet("Sheet1");
         HSSFSheet sheet2 = wb.createSheet("Sheet2");
         HSSFSheet sheet3 = wb.createSheet("Sheet3");
         HSSFSheet sheet4 = wb.createSheet("Sheet4");
-        
+
         assertEquals(0, wb.getSheetIndex(sheet1));
         assertEquals(1, wb.getSheetIndex(sheet2));
         assertEquals(2, wb.getSheetIndex(sheet3));
         assertEquals(3, wb.getSheetIndex(sheet4));
-        
+
         // remove sheets
         wb.removeSheetAt(0);
         wb.removeSheetAt(2);
@@ -823,54 +897,57 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
     }
 
     @SuppressWarnings("deprecation")
-    public void testGetExternSheetIndex() {
+    @Test
+    public void getExternSheetIndex() {
         HSSFWorkbook wb=new HSSFWorkbook();
         wb.createSheet("Sheet1");
         wb.createSheet("Sheet2");
-        
+
         assertEquals(0, wb.getExternalSheetIndex(0));
         assertEquals(1, wb.getExternalSheetIndex(1));
-        
+
         assertEquals("Sheet1", wb.findSheetNameFromExternSheet(0));
         assertEquals("Sheet2", wb.findSheetNameFromExternSheet(1));
         //assertEquals(null, wb.findSheetNameFromExternSheet(2));
-        
+
         assertEquals(0, wb.getSheetIndexFromExternSheetIndex(0));
         assertEquals(1, wb.getSheetIndexFromExternSheetIndex(1));
         assertEquals(-1, wb.getSheetIndexFromExternSheetIndex(2));
         assertEquals(-1, wb.getSheetIndexFromExternSheetIndex(100));
     }
-    
+
     @SuppressWarnings("deprecation")
-    public void testSSTString() {
+    @Test
+    public void sstString() {
         HSSFWorkbook wb=new HSSFWorkbook();
-        
+
         int sst = wb.addSSTString("somestring");
         assertEquals("somestring", wb.getSSTString(sst));
         //assertNull(wb.getSSTString(sst+1));
     }
-    
-    public void testNames() {
+
+    @Test
+    public void names() {
         HSSFWorkbook wb=new HSSFWorkbook();
-        
+
         try {
             wb.getNameAt(0);
             fail("Fails without any defined names");
         } catch (IllegalStateException e) {
             assertTrue(e.getMessage(), e.getMessage().contains("no defined names"));
         }
-        
+
         HSSFName name = wb.createName();
         assertNotNull(name);
-        
+
         assertNull(wb.getName("somename"));
 
         name.setNameName("myname");
         assertNotNull(wb.getName("myname"));
-        
+
         assertEquals(0, wb.getNameIndex(name));
         assertEquals(0, wb.getNameIndex("myname"));
-        
+
         try {
             wb.getNameAt(5);
             fail("Fails without any defined names");
@@ -885,33 +962,36 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
             assertTrue(e.getMessage(), e.getMessage().contains("outside the allowable range"));
         }
     }
-    
-    public void testTestMethods() {
+
+    @Test
+    public void testMethods() {
         HSSFWorkbook wb=new HSSFWorkbook();
         wb.insertChartRecord();
         //wb.dumpDrawingGroupRecords(true);
         //wb.dumpDrawingGroupRecords(false);
     }
-    
-    public void testWriteProtection() {
+
+    @Test
+    public void writeProtection() {
         HSSFWorkbook wb=new HSSFWorkbook();
-        
+
         assertFalse(wb.isWriteProtected());
-        
+
         wb.writeProtectWorkbook("mypassword", "myuser");
         assertTrue(wb.isWriteProtected());
-        
+
         wb.unwriteProtectWorkbook();
         assertFalse(wb.isWriteProtected());
     }
 
-	public void testBug50298() throws Exception {
+    @Test
+	public void bug50298() throws Exception {
 		HSSFWorkbook wb = HSSFTestDataSamples.openSampleWorkbook("50298.xls");
 
 		assertSheetOrder(wb, "Invoice", "Invoice1", "Digest", "Deferred", "Received");
 
 		HSSFSheet sheet = wb.cloneSheet(0);
-		
+
 		assertSheetOrder(wb, "Invoice", "Invoice1", "Digest", "Deferred", "Received", "Invoice (2)");
 
 		wb.setSheetName(wb.getSheetIndex(sheet), "copy");
@@ -923,27 +1003,28 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
 		assertSheetOrder(wb, "copy", "Invoice", "Invoice1", "Digest", "Deferred", "Received");
 
 		wb.removeSheetAt(0);
-		
+
 		assertSheetOrder(wb, "Invoice", "Invoice1", "Digest", "Deferred", "Received");
 
 		// check that the overall workbook serializes with its correct size
 		int expected = wb.getWorkbook().getSize();
 		int written = wb.getWorkbook().serialize(0, new byte[expected*2]);
-		
+
 		assertEquals("Did not have the expected size when writing the workbook: written: " + written + ", but expected: " + expected,
 				expected, written);
-		
+
 		HSSFWorkbook read = HSSFTestDataSamples.writeOutAndReadBack(wb);
 		assertSheetOrder(read, "Invoice", "Invoice1", "Digest", "Deferred", "Received");
 	}
 
-	public void testBug50298a() throws Exception {
+    @Test
+	public void bug50298a() throws Exception {
 		HSSFWorkbook wb = HSSFTestDataSamples.openSampleWorkbook("50298.xls");
 
 		assertSheetOrder(wb, "Invoice", "Invoice1", "Digest", "Deferred", "Received");
 
 		HSSFSheet sheet = wb.cloneSheet(0);
-		
+
 		assertSheetOrder(wb, "Invoice", "Invoice1", "Digest", "Deferred", "Received", "Invoice (2)");
 
 		wb.setSheetName(wb.getSheetIndex(sheet), "copy");
@@ -955,29 +1036,30 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
 		assertSheetOrder(wb, "copy", "Invoice", "Invoice1", "Digest", "Deferred", "Received");
 
 		wb.removeSheetAt(0);
-		
+
 		assertSheetOrder(wb, "Invoice", "Invoice1", "Digest", "Deferred", "Received");
 
 		wb.removeSheetAt(1);
-		
+
 		assertSheetOrder(wb, "Invoice", "Digest", "Deferred", "Received");
-		
+
 		wb.setSheetOrder("Digest", 3);
 
 		assertSheetOrder(wb, "Invoice", "Deferred", "Received", "Digest");
-		
+
 		// check that the overall workbook serializes with its correct size
 		int expected = wb.getWorkbook().getSize();
 		int written = wb.getWorkbook().serialize(0, new byte[expected*2]);
-		
+
 		assertEquals("Did not have the expected size when writing the workbook: written: " + written + ", but expected: " + expected,
 				expected, written);
-		
+
 		HSSFWorkbook read = HSSFTestDataSamples.writeOutAndReadBack(wb);
 		assertSheetOrder(read, "Invoice", "Deferred", "Received", "Digest");
 	}
 	
-	public void testBug54500() throws Exception {
+    @Test
+	public void bug54500() throws Exception {
 		String nameName = "AName";
 		String sheetName = "ASheet";
 		HSSFWorkbook wb = HSSFTestDataSamples.openSampleWorkbook("54500.xls");
@@ -1025,4 +1107,36 @@ public final class TestHSSFWorkbook extends BaseTestWorkbook {
 	private void expectName(HSSFWorkbook wb, String name, String expect) {
 		assertEquals(expect, wb.getName(name).getRefersToFormula());
 	}
+
+	@Test
+	public void test49423() throws Exception
+    {
+		HSSFWorkbook workbook = HSSFTestDataSamples.openSampleWorkbook("49423.xls");
+		
+		boolean found = false;
+        int numSheets = workbook.getNumberOfSheets();
+        for (int i = 0; i < numSheets; i++) {
+            HSSFSheet sheet = workbook.getSheetAt(i);
+            List<HSSFShape> shapes = sheet.getDrawingPatriarch().getChildren();
+            for(HSSFShape shape : shapes){
+                HSSFAnchor anchor = shape.getAnchor();
+    
+                if(anchor instanceof HSSFClientAnchor){
+                    // absolute coordinates
+                    HSSFClientAnchor clientAnchor = (HSSFClientAnchor)anchor;
+                    assertNotNull(clientAnchor);
+                    //System.out.println(clientAnchor.getRow1() + "," + clientAnchor.getRow2());
+                    found = true;
+                } else if (anchor instanceof HSSFChildAnchor){
+                    // shape is grouped and the anchor is expressed in the coordinate system of the group 
+                    HSSFChildAnchor childAnchor = (HSSFChildAnchor)anchor;
+                    assertNotNull(childAnchor);
+                    //System.out.println(childAnchor.getDy1() + "," + childAnchor.getDy2());
+                    found = true;
+                }
+            }
+        }
+        
+        assertTrue("Should find some images via Client or Child anchors, but did not find any at all", found);
+    }
 }
